@@ -1,3 +1,6 @@
+/*
+ * $Id: events.c,v 1.5 1998/05/28 22:14:17 u27113 Exp $
+ */
 /*****************************************************************************/
 /**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
 /**                          Salt Lake City, Utah                           **/
@@ -827,11 +830,10 @@ button_press(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 
     scr->mwm_event = win;
 
-    /* we have to execute a function or pop up a menu
-     */
+    /* we have to execute a function or pop up a menu */
     modifier = (event->xbutton.state & MOD_MASK);
-    /* need to search for an appropriate mouse binding */
 
+    /* need to search for an appropriate mouse binding */
     click = is_click(scr, event->xbutton.x, event->xbutton.y, event);
     if (click && (MISC_FetchEventTime() - last_time) < Mwm.double_click_time)
     {
@@ -840,6 +842,9 @@ button_press(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 
     last_time = MISC_FetchEventTime();
 
+    /*
+     * This loop only seems to handle double-clicked stuff
+     */
     MouseEntry = scr->buttons;
     while (click2 && MouseEntry != (MouseButton *)0)
     {
@@ -868,6 +873,8 @@ button_press(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 	}
 	MouseEntry = MouseEntry->next;
     }
+
+    /* Still only for double-clicks as "match" can only be set in the loop */
     if (match)
     {
 	scr->pressed_win = None;
@@ -883,9 +890,23 @@ button_press(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 			     (scr->mwm_highlight == scr->mwm_event), False);
 	}
 	scr->mwm_event = NULL;
+
 	return;
     }
 
+    /*
+     * If we had a match above (in double-clicks), we don't come here.
+     *
+     * Now we traverse the list of button bindings.
+     *
+     * Careful inspection of mwmparse.y reveals that this is made up from
+     * two sources: builtin bindings, and bindings read from the configuration
+     * file.
+     *
+     * FIX ME the builtins are not completely motif compliant (e.g. the window
+     * menu that's tied to <Btn1Down> on the root window).
+     * BTW the builtins are defined in mwm.h.
+     */
     MouseEntry = scr->buttons;
     while (MouseEntry != (MouseButton *)0)
     {
@@ -898,13 +919,35 @@ button_press(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 	{
 	    /* got a match, now process it, unless the user clicked, and it's
 	     * interactive */
+
+	    /*
+	     * Rewrote the test below so I can understand it. Is it just me?
+	     */
+#if 1
+	    if (MouseEntry->func == F_NOP) {
+		MouseEntry = MouseEntry->next;
+		continue;
+	    }
+
+	    if (click && !click2 && (MouseEntry->func == F_MOVE ||
+				       MouseEntry->func == F_RESIZE ||
+				       MouseEntry->func == F_POPUP ||
+				       MouseEntry->func == F_WINDOWLIST ||
+				       MouseEntry->func == F_W_POPUP)) {
+		MouseEntry = MouseEntry->next;
+		continue;
+	    }
+#else
+	    /* Uh oh. */
 	    if (MouseEntry->func != F_NOP &&
 		!(click && !click2 && (MouseEntry->func == F_MOVE ||
 				       MouseEntry->func == F_RESIZE ||
 				       MouseEntry->func == F_WINDOWLIST ||
 				       MouseEntry->func == F_POPUP ||
 				       MouseEntry->func == F_W_POPUP)))
+#endif
 	    {
+
 		if (!(MouseEntry->mask & ButtonReleaseMask) &&
 		    MouseEntry->count == 0)
 		{
@@ -922,6 +965,46 @@ button_press(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 	MouseEntry = MouseEntry->next;
     }
 
+#if 1
+    /* Hack.
+	Support a single button press only for F_W_POPUP
+     */
+    MouseEntry = scr->buttons;
+    while (!click && !click2 && MouseEntry != (MouseButton *)0)
+    {
+	if ((MouseEntry->button == event->xbutton.button ||
+	     MouseEntry->button == 0) &&
+	    (MouseEntry->context & scr->event_context) &&
+	    (MouseEntry->modifier == AnyModifier ||
+	     MouseEntry->modifier == (modifier & (~LockMask))) &&
+	    (MouseEntry->mask & (ButtonPressMask | ButtonReleaseMask)))
+	{
+
+	    /* got a match, now process it */
+	    if (MouseEntry->func == F_W_POPUP)
+	    {
+#if 0
+		if (MouseEntry->count == 1)
+#endif
+		{
+		    Action = MouseEntry->item ? MouseEntry->item->action : NULL;
+		    /* win->custom_menu = NULL;	/* FIX ME */
+		    if (MouseEntry->menu == NULL)
+			MouseEntry->menu = (MenuRoot *)DEFAULT_WIN_MENU_NAME;
+		    FUNC_Execute(scr, F_W_POPUP, Action,
+				 event->xany.window, win, event,
+				 scr->event_context,
+				 MouseEntry->val1, MouseEntry->val2,
+				 MouseEntry->val1_unit, MouseEntry->val2_unit,
+				 MouseEntry->menu);
+		}
+	    }
+	}
+	MouseEntry = MouseEntry->next;
+    }
+#endif
+
+    /* This only works for clicks, not for <Btn1Down> */
     MouseEntry = scr->buttons;
     while (click && !click2 && MouseEntry != (MouseButton *)0)
     {
