@@ -233,7 +233,7 @@ get_xpm_file(ScreenInfo *scr, MwmWindow *tmp_win)
     xpm_attributes.valuemask = XmXpmSize | XmXpmReturnPixels | XmXpmColormap |
 	XmXpmCloseness;
 
-#ifndef LESSTIF_PRODUCTION
+#ifdef NONSTANDARD_CONVERTERS
     if (_XmXpmReadFileToImage(dpy, path, &im, &imshape, &xpm_attributes) ==
 	XmXpmSuccess)
     {
@@ -242,7 +242,8 @@ get_xpm_file(ScreenInfo *scr, MwmWindow *tmp_win)
 					     im->width, im->height,
 					     scr->d_depth);
 
-	tmp_win->icon_mask_pixmap = XCreatePixmap(dpy, scr->root_win,
+	if (imshape)
+	    tmp_win->icon_mask_pixmap = XCreatePixmap(dpy, scr->root_win,
 						  imshape->width,
 						  imshape->height,
 						  1);
@@ -256,7 +257,8 @@ get_xpm_file(ScreenInfo *scr, MwmWindow *tmp_win)
 
 	gc = XCreateGC(dpy, tmp_win->icon_mask_pixmap, 0, NULL);
 
-	XPutImage(dpy, tmp_win->icon_mask_pixmap, gc, imshape, 0, 0, 0, 0,
+	if (imshape)
+	    XPutImage(dpy, tmp_win->icon_mask_pixmap, gc, imshape, 0, 0, 0, 0,
 		  imshape->width, imshape->height);
 
 	XFreeGC(dpy, gc);
