@@ -772,3 +772,84 @@ __XmRGB2HSV(unsigned short r,
     *vr = v;
 }
 #endif
+
+/* Debugging purposes only */
+char *
+_MwmPrintF(int x)
+{
+	switch(x) {
+	case F_NOP:	return "F_NOP";
+	case F_BEEP:	return "F_BEEP";
+	case F_CHANGE_WINDOWS_DESK:	return "F_CHANGE_WINDOWS_DESK";
+	case F_CIRCULATE_DOWN:	return "F_CIRCULATE_DOWN";
+	case F_CIRCULATE_UP:	return "F_CIRCULATE_UP";
+	case F_CLOSE:	return "F_CLOSE";
+	case F_DESK:	return "F_DESK";
+	case F_EXEC:	return "F_EXEC";
+	case F_FOCUS:	return "F_FOCUS";
+	case F_FOCUS_COLOR:	return "F_FOCUS_COLOR";
+	case F_FOCUS_KEY:	return "F_FOCUS_KEY";
+	case F_GOTO_PAGE:	return "F_GOTO_PAGE";
+	case F_ICONIFY:	return "F_ICONIFY";
+	case F_LOWER:	return "F_LOWER";
+	case F_MAXIMIZE:	return "F_MAXIMIZE";
+	case F_MOVE:	return "F_MOVE";
+	case F_MOVECURSOR:	return "F_MOVECURSOR";
+	case F_NEXT_CMAP:	return "F_NEXT_CMAP";
+	case F_NEXT_KEY:	return "F_NEXT_KEY";
+	case F_NORMALIZE:	return "F_NORMALIZE";
+	case F_NORM_AND_RAISE:	return "F_NORM_AND_RAISE";
+	case F_PACK_ICONS:	return "F_PACK_ICONS";
+	case F_PASS_KEYS:	return "F_PASS_KEYS";
+	case F_POPUP:	return "F_POPUP";
+	case F_PREV_CMAP:	return "F_PREV_CMAP";
+	case F_PREV_KEY:	return "F_PREV_KEY";
+	case F_QUIT:	return "F_QUIT";
+	case F_RAISE:	return "F_RAISE";
+	case F_RAISE_IT:	return "F_RAISE_IT";
+	case F_RAISELOWER:	return "F_RAISELOWER";
+	case F_RESIZE:	return "F_RESIZE";
+	case F_RESTART:	return "F_RESTART";
+	case F_REFRESH:	return "F_REFRESH";
+	case F_REFRESH_WIN:	return "F_REFRESH_WIN";
+	case F_RESTORE_AND_RAISE:	return "F_RESTORE_AND_RAISE";
+	case F_SCREEN:	return "F_SCREEN";
+	case F_SCROLL:	return "F_SCROLL";
+	case F_SEND_MSG:	return "F_SEND_MSG";
+	case F_SET_BEHAVIOR:	return "F_SET_BEHAVIOR";
+	case F_STICK:	return "F_STICK";
+	case F_TITLE:	return "F_TITLE";
+	case F_TOGGLE_PAGE:	return "F_TOGGLE_PAGE";
+	case F_WARP:	return "F_WARP";
+	case F_WINDOWLIST:	return "F_WINDOWLIST";
+	case F_W_POPUP:	return "F_W_POPUP";
+	default:	return "??";
+	}
+}
+
+char *
+_MwmPrintC(int x)
+{
+	static char	res[256];
+
+	res[0] = 0;
+	if (x == C_NO_CONTEXT)
+		return "C_NO_CONTEXT";
+	if (x & C_WINDOW)
+		strcat(res, " | C_WINDOW");
+	if (x & C_TITLE)
+		strcat(res, " | C_TITLE");
+	if (x & C_ICON)
+		strcat(res, " | C_ICON");
+	if (x & C_ROOT)
+		strcat(res, " | C_ROOT");
+	if (x & C_FRAME)
+		strcat(res, " | C_FRAME");
+	if (x & C_MENUB)
+		strcat(res, " | C_MENUB");
+	if (x & C_MINIMIZEB)
+		strcat(res, " | C_MINIMIZEB");
+	if (x & C_MAXIMIZEB)
+		strcat(res, " | C_MAXIMIZEB");
+	return &res[2];
+}
