@@ -58,6 +58,8 @@
 #include <Xm/DisplayP.h>
 #include <Xm/ScreenP.h>
 
+#include <XmI/DebugUtil.h>
+
 extern int putenv();
 
 /*
@@ -452,6 +454,23 @@ main(int argc, char **argv)
 
     initialize_mwm();
 
+    if (verbose) {
+	MouseButton *MouseEntry = Mwm.screen_info[0]->buttons;
+
+	fprintf(stderr, "Button Bindings:\n");
+	while (MouseEntry) {
+		extern char *_MwmPrintF(int);
+		extern char *_MwmPrintC(int);
+		fprintf(stderr,
+			"\tfunc %d %s button %d modifier %d context %s\n",
+			MouseEntry->func, _MwmPrintF(MouseEntry->func),
+			MouseEntry->button, MouseEntry->modifier,
+			_MwmPrintC(MouseEntry->context));
+
+		MouseEntry = MouseEntry->next;
+	}
+    }
+
     while (True)
     {
 	XEvent event;
