@@ -1,6 +1,6 @@
 /**
  *
- * $Id: MacrosI.h,v 1.24 1998/02/28 18:41:53 rwscott Exp $
+ * $Id: MacrosI.h,v 1.26 1998/04/22 19:47:19 u27113 Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -2195,6 +2195,9 @@
 #define Scale_DragCallback(x) \
     (((XmScaleWidget)(x))->scale.drag_callback)
 
+#define	Scale_Editable(x) \
+    (((XmScaleWidget)(x))->scale.editable)
+
 /*
  * SELECTIONBOX WIDGET ACCESS MACROS
  */
@@ -2686,4 +2689,12 @@
 #define Desktop_NumSlots(w) \
    (((XmDesktopRec *)(w))->desktop.num_slots)
 
+/*
+ * Convenient shorthands.
+ */
+#define ColormapOfObject(w) \
+    (XmIsGadget(w) \
+	? CoreColormap(((XmGadget)(w))->object.parent) \
+	: CoreColormap(w))
+
 #endif /* MacrosI.h */
