@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.3 1998/02/09 15:09:13 u27113 Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.5 1998/05/13 21:06:36 u27113 Exp $
  *
  * Copyright (C) 1995-1998 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: PanedW.c,v 1.3 1998/02/09 15:09:13 u27113 Exp $";
+static char rcsid[] = "$Id: PanedW.c,v 1.5 1998/05/13 21:06:36 u27113 Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -766,7 +766,7 @@ set_values(Widget old, Widget request, Widget new_w,
 	XtSetArg(arg, XmNorientation,
 	    (PW_Orientation(new_w) == XmVERTICAL) ? XmHORIZONTAL : XmVERTICAL);
 	for (i=0; i<PW_NumManagedChildren(new_w); i++) {
-	    if (PWC_Separator(PW_ManagedChildren(new_w)[i]), &arg, 1)
+	    if (PWC_Separator(PW_ManagedChildren(new_w)[i]))
 		XtSetValues(PWC_Separator(PW_ManagedChildren(new_w)[i]),
 			&arg, 1);
 	}
@@ -1059,9 +1059,10 @@ insert_child(Widget w)
      */
     PWC_Separator(w) = XtVaCreateWidget("separator", xmSeparatorWidgetClass,
 					(Widget)pw,
-					XmNshadowThickness, 2,
-					XmNseparatorType,
-					XmSHADOW_ETCHED_IN,
+		XmNshadowThickness,	2,
+		XmNseparatorType,	XmSHADOW_ETCHED_IN,
+		XmNorientation,		(PW_Orientation(pw) == XmVERTICAL)
+						? XmHORIZONTAL : XmVERTICAL,
 					NULL);
 
     PWC_Separator(PWC_Separator(w)) = w;
@@ -1417,6 +1418,18 @@ change_managed(Widget w)
 	    XtUnmanageChild(PWC_Sash(PW_ManagedChildren(w)[i]));
 	    XtUnmanageChild(PWC_Separator(PW_ManagedChildren(w)[i]));
 
+	    PW_RecursivelyCalled(w) = False;
+	}
+	if (i == (PW_PaneCount(w) - 1) &&
+	    (XtIsManaged(PWC_Sash(PW_ManagedChildren(w)[i])) ||
+	    (XtIsManaged(PWC_Separator(PW_ManagedChildren(w)[i])))))
+	{
+
+	    PW_RecursivelyCalled(w) = True;
+
+	    XtUnmanageChild(PWC_Sash(PW_ManagedChildren(w)[i]));
+	    XtUnmanageChild(PWC_Separator(PW_ManagedChildren(w)[i]));
+
 	    PW_RecursivelyCalled(w) = False;
 	}
     }
