@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ArrowB.c,v 1.17 1998/02/22 01:00:22 rwscott Exp $
+ * $Id: ArrowB.c,v 1.18 1998/03/24 23:00:45 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: ArrowB.c,v 1.17 1998/02/22 01:00:22 rwscott Exp $";
+static char rcsid[] = "$Id: ArrowB.c,v 1.18 1998/03/24 23:00:45 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -256,8 +256,8 @@ CreateInsensitiveGC(Widget w)
     values.ts_x_origin = values.ts_y_origin = 0;
     values.stipple = XmGetPixmapByDepth(XtScreen(w),
 					XmEVEN_STIPPLE_IMAGE,
-					WhitePixelOfScreen(XtScreen(w)),
-					BlackPixelOfScreen(XtScreen(w)),
+					_XmWhitePixelOfObject(w),
+					_XmBlackPixelOfObject(w),
 					1);
 
     AB_InsensitiveGC(w) = XtGetGC(w, mask, &values);
