@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ArrowBG.c,v 1.12 1998/02/22 01:00:23 rwscott Exp $
+ * $Id: ArrowBG.c,v 1.13 1998/03/24 23:00:47 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: ArrowBG.c,v 1.12 1998/02/22 01:00:23 rwscott Exp $";
+static char rcsid[] = "$Id: ArrowBG.c,v 1.13 1998/03/24 23:00:47 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -294,8 +294,8 @@ CreateInsensitiveGC(Widget w)
 
     values.stipple = XmGetPixmapByDepth(XtScreen(w),
 					XmEVEN_STIPPLE_IMAGE,
-					WhitePixelOfScreen(XtScreen(w)),
-					BlackPixelOfScreen(XtScreen(w)),
+					_XmWhitePixelOfObject(w),
+					_XmBlackPixelOfObject(w),
 					1);
 
     ABG_InsensitiveGC(w) = XtGetGC(w, mask, &values);
