@@ -1,6 +1,6 @@
 /**
  *
- * $Id: BulletinBoard.c,v 1.21 1998/02/22 01:00:24 rwscott Exp $
+ * $Id: BulletinBoard.c,v 1.24 1998/04/14 22:54:44 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: BulletinBoard.c,v 1.21 1998/02/22 01:00:24 rwscott Exp $";
+static char rcsid[] = "$Id: BulletinBoard.c,v 1.24 1998/04/14 22:54:44 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -892,7 +892,16 @@ handle_change_managed(Widget w, XmGeoCreateProc mat_make)
 
     if (!XtIsRealized(w))
     {
+	/* rws 5 Apr 1998
+	   If we are not realized use the width/height. This will be
+	   the user specified size, or 0x0 if not specified.
+	 */
+#if 0
 	wd = ht = 0;
+#else
+	wd = XtWidth(w);
+	ht = XtHeight(w);
+#endif
     }
     else if (BB_ResizePolicy(w) != XmNONE)
     {
@@ -1312,21 +1321,50 @@ _XmBBParentProcess(Widget widget, XmParentProcessData data)
     {
 	if (data->input_action.action == XmPARENT_ACTIVATE)
 	{
-	    _XmBulletinBoardReturn(widget,
+	    if (BB_DefaultButton(widget) || BB_DynamicDefaultButton(widget))
+	    {
+		_XmBulletinBoardReturn(widget,
 				   data->input_action.event,
 				   data->input_action.params,
 				   data->input_action.num_params);
 
-	    return True;
+		return True;
+	    }
+	    else
+	    {
+	    Widget w = XtParent(widget);
+	    XmManagerWidgetClass mwc = (XmManagerWidgetClass)w->core.widget_class;
+
+		if (XmIsManager(w) && mwc->manager_class.parent_process)
+		{
+		    return((*mwc->manager_class.parent_process) (w, data));
+		}
+	    }
 	}
 	else if (data->input_action.action == XmPARENT_CANCEL)
 	{
-	    _XmBulletinBoardCancel(widget,
-				   data->input_action.event,
-				   data->input_action.params,
-				   data->input_action.num_params);
+	    DEBUGOUT(XdbDebug(__FILE__, widget, "_XmBBParentProcess CANCEL\n"));
+	    if (BB_CancelButton(widget))
+	    {
+		DEBUGOUT(XdbDebug(__FILE__, widget, "_XmBBParentProcess CANCEL %s\n",XtName(BB_CancelButton(widget))));
+		_XmBulletinBoardCancel(widget,
+				       data->input_action.event,
+				       data->input_action.params,
+				       data->input_action.num_params);
 
-	    return True;
+		return True;
+	    }
+	    else
+	    {
+	    Widget w = XtParent(widget);
+	    XmManagerWidgetClass mwc = (XmManagerWidgetClass)w->core.widget_class;
+
+		DEBUGOUT(XdbDebug(__FILE__, widget, "_XmBBParentProcess CANCEL no button\n"));
+		if (XmIsManager(w) && mwc->manager_class.parent_process)
+		{
+		    return((*mwc->manager_class.parent_process) (w, data));
+		}
+	    }
 	}
     }
 
@@ -1338,12 +1376,16 @@ _XmBB_CreateButtonG(Widget bb, XmString l_string, char *name)
 {
     Widget button;
     Arg args[2];
+    Cardinal n = 0;
 
-    XtSetArg(args[0], XmNlabelString, l_string);
+    if (l_string)
+    {
+	XtSetArg(args[n], XmNlabelString, l_string); n++;
+    }
 #ifdef USE_WIDGETS
-    button = XmCreatePushButton(bb, name, args, 1);
+    button = XmCreatePushButton(bb, name, args, n);
 #else
-    button = XmCreatePushButtonGadget(bb, name, args, 1);
+    button = XmCreatePushButtonGadget(bb, name, args, n);
 #endif
     _XmBulletinBoardSetDefaultShadow(button);
 
@@ -1355,13 +1397,17 @@ _XmBB_CreateLabelG(Widget bb, XmString l_string, char *name)
 {
     Widget label;
     Arg args[2];
+    Cardinal n = 0;
 
-    XtSetArg(args[0], XmNlabelString, l_string);
+    if (l_string)
+    {
+	XtSetArg(args[n], XmNlabelString, l_string); n++;
+    }
 
 #ifdef USE_WIDGETS
-    label = XmCreateLabel(bb, name, args, 1);
+    label = XmCreateLabel(bb, name, args, n);
 #else
-    label = XmCreateLabelGadget(bb, name, args, 1);
+    label = XmCreateLabelGadget(bb, name, args, n);
 #endif
 
     return label;
@@ -1532,6 +1578,7 @@ _XmBulletinBoardReturn(Widget wid, XEvent *event,
     /*
        rws 12 Jul 1997
        This is now filled in but, is it correct??
+       see man XmBulletinBoard under KActivate
      */
     XmPushButtonCallbackStruct cbs;
 
@@ -1543,11 +1590,14 @@ _XmBulletinBoardReturn(Widget wid, XEvent *event,
 	XtIsManaged(BB_DynamicDefaultButton(wid)) &&
 	XtIsRealized(BB_DynamicDefaultButton(wid)))
     {
-#if 1
 	ArmAndActivate(BB_DynamicDefaultButton(wid), event, params, numParams);
-#else
-	XtCallCallbacks(BB_DynamicDefaultButton(wid), XmNactivateCallback, &cbs);
-#endif
+    }
+    else if (BB_DefaultButton(wid) &&
+	XtIsSensitive(BB_DefaultButton(wid)) &&
+	XtIsManaged(BB_DefaultButton(wid)) &&
+	XtIsRealized(BB_DefaultButton(wid)))
+    {
+	ArmAndActivate(BB_DefaultButton(wid), event, params, numParams);
     }
 }
 
@@ -1557,6 +1607,7 @@ _XmBulletinBoardCancel(Widget wid, XEvent *event,
 {
     XmPushButtonCallbackStruct cbs;
 
+    /* see man XmBulletinBoard under KCancel */
     cbs.reason = XmCR_ACTIVATE;
     cbs.event = event;
 
@@ -1564,9 +1615,6 @@ _XmBulletinBoardCancel(Widget wid, XEvent *event,
 	XtIsManaged(BB_CancelButton(wid)) && XtIsRealized(BB_CancelButton(wid)))
     {
 	ArmAndActivate(BB_CancelButton(wid), event, params, numParams);
-	/*
-	XtCallCallbacks(BB_CancelButton(wid), XmNactivateCallback, &cbs);
-	*/
     }
     else if (BB_DynamicCancelButton(wid) &&
 	     XtIsSensitive(BB_DynamicCancelButton(wid)) &&
@@ -1574,9 +1622,6 @@ _XmBulletinBoardCancel(Widget wid, XEvent *event,
 	     XtIsRealized(BB_DynamicCancelButton(wid)))
     {
 	ArmAndActivate(BB_DynamicCancelButton(wid), event, params, numParams);
-	/*
-	XtCallCallbacks(BB_DynamicCancelButton(wid), XmNactivateCallback, &cbs);
-	*/
     }
 }
 
