@@ -1,6 +1,6 @@
 /**
  *
- * $Id: CascadeB.c,v 1.80 1998/03/07 21:24:00 rwscott Exp $
+ * $Id: CascadeB.c,v 1.82 1998/05/08 19:41:49 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: CascadeB.c,v 1.80 1998/03/07 21:24:00 rwscott Exp $";
+static const char rcsid[] = "$Id: CascadeB.c,v 1.82 1998/05/08 19:41:49 rwscott Exp $";
 
 #define	DO_PIXMAP
 #define	DO_SANITY
@@ -41,6 +41,12 @@ static char rcsid[] = "$Id: CascadeB.c,v 1.80 1998/03/07 21:24:00 rwscott Exp $"
 #include <Xm/MenuUtilP.h>
 #include <Xm/ScreenP.h>
 
+#ifndef __STDC__
+#include <varargs.h>
+#else
+#include <stdarg.h>
+#endif
+
 #include <XmI/DebugUtil.h>
 
 #include <stdio.h>
@@ -611,7 +617,15 @@ expose(Widget w, XEvent *event, Region region)
     		      __FILE__,
 		      CB_IsArmed(w), XdbMenuType2String(Lab_MenuType(w))));
 
+#if 0
+    /* rws 8 May 1998
+       This messes up Mozilla's Bookmark menu buttons which are sub-classed
+       from PushB and CascadeB.
+     */
     resize(w);
+#else
+    (XtClass(w)->core_class.resize)(w);
+#endif
 
     /* The label's expose routine should be called after the cascade button's
      * expose routine. */
@@ -715,10 +729,6 @@ void
 _XmCascadingPopup(Widget cb, XEvent *event, Boolean doCascade)
 {
     /* FIX ME: doCascade? */
-    Widget sub_pane;
-    Widget shell;
-    Dimension shadow;
-    Position my_x, my_y;
     XmAnyCallbackStruct cbs;
 
     RC_MenuCascading(cb, event);
@@ -843,7 +853,7 @@ void
 _XmCBMenuBarSelect(Widget w, XEvent *event,
 		   String *params, Cardinal *num_params)
 {
-    Widget menu, shell, mb;
+    Widget menu, mb;
     Boolean poppedUp;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmCBMenuBarSelect()\n"));
@@ -916,8 +926,6 @@ ArmAndPost(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 MenuBarEnter(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    Widget parent;
-
     DEBUGOUT(XdbDebug(__FILE__, w, "MenuBarEnter\n"));
 
     /* if we're not dragging, do nothing. */
@@ -1139,29 +1147,21 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
 #else
-MenuProcEntry(function, widget, va_alist)
-     int function;
-     Widget widget;
+MenuProcEntry(proc, w, va_alist)
+     int proc;
+     Widget w;
      va_dcl
 {
     va_list arglist;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
-    va_start(arg_list);
+    va_start(arglist);
 #endif
 
     switch (proc)
