@@ -1,6 +1,6 @@
 /**
  *
- * $Id: CascadeBG.c,v 1.74 1998/03/13 23:07:15 helden Exp $
+ * $Id: CascadeBG.c,v 1.75 1998/03/16 21:52:09 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: CascadeBG.c,v 1.74 1998/03/13 23:07:15 helden Exp $";
+static const char rcsid[] = "$Id: CascadeBG.c,v 1.75 1998/03/16 21:52:09 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -39,6 +39,12 @@ static char rcsid[] = "$Id: CascadeBG.c,v 1.74 1998/03/13 23:07:15 helden Exp $"
 #include <Xm/MenuUtilP.h>
 #include <X11/Xfuncs.h>
 
+#ifndef __STDC__
+#include <varargs.h>
+#else
+#include <stdarg.h>
+#endif
+
 #include <XmI/DebugUtil.h>
 
 #define	DO_SANITY
@@ -1224,8 +1230,7 @@ DoSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 MenuBarSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    Widget menu, shell, mb;
-    Boolean was_torn = False;
+    Widget menu, mb;
     Boolean poppedUp;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "MenuBarSelect()\n"));
@@ -1279,8 +1284,6 @@ ArmAndPost(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 MenuBarEnter(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    Widget parent;
-
     DEBUGOUT(XdbDebug(__FILE__, w, "MenuBarEnter()\n"));
 
     /* if we're not dragging, do nothing. */
@@ -1585,29 +1588,21 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
 #else
-MenuProcEntry(function, widget, va_alist)
-     int function;
-     Widget widget;
+MenuProcEntry(proc, w, va_alist)
+     int proc;
+     Widget w;
      va_dcl
 {
     va_list arglist;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
-    va_start(arg_list);
+    va_start(arglist);
 #endif
 
     switch (proc)
