@@ -1,6 +1,6 @@
 /**
  *
- * $Id: CutPaste.c,v 1.4 1997/08/15 23:21:19 miers Exp $
+ * $Id: CutPaste.c,v 1.6 1998/05/14 22:58:01 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: CutPaste.c,v 1.4 1997/08/15 23:21:19 miers Exp $";
+static char rcsid[] = "$Id: CutPaste.c,v 1.6 1998/05/14 22:58:01 u27113 Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -196,6 +196,12 @@ _XmClipboardIsMarkedForDelete(Display *display, XmClipboard * clip, int item)
 
     _XmClipboardFindItem(display, item, &data, &len, NULL, XmCLIP_PROP_ITEM);
 
+    if (data == NULL) {
+	_XmWarning(NULL, "_XmClipboardIsMarkedForDelete: "
+			"_XmClipboardFindItem returned NULL data\n");
+	return True;	/* ??? FIX ME */
+    }
+
     ret = ((XmClipboardItem *) data)->marked_for_delete;
 
     XtFree((char *)data);
@@ -2006,6 +2012,10 @@ XmClipboardCopy(Display *display,
     int fmt;
     XtPointer nbuf;
 
+    DEBUGOUT(XdbDebug(__FILE__, NULL,
+	"XmClipboardCopy(win 0x%X, item_id %ld, format %s)\n",
+	window, item_id, format_name));
+
     status = _XmClipboardLock(display, window);
 
     if (status == XmClipboardLocked)
@@ -2237,6 +2247,10 @@ XmClipboardEndCopy(Display *display,
     int len;
     XmClipboardItem *item;
 
+    DEBUGOUT(XdbDebug(__FILE__, NULL,
+	"XmClipboardEndCopy(win 0x%X, item %ld)\n",
+	window, item_id));
+
     status = _XmClipboardLock(display, window);
     if (status == XmClipboardLocked)
 	return status;
@@ -2282,7 +2296,7 @@ XmClipboardEndCopy(Display *display,
 
     item = (XmClipboardItem *) data;
 
-    if (item->widget)
+    if (item && item->widget)
     {
 	DEBUGOUT(XdbDebug(__FILE__, NULL, "Add EventHandler from EndCopy\n"));
 
@@ -2290,7 +2304,8 @@ XmClipboardEndCopy(Display *display,
 			  _XmClipboardEventHandler, NULL);
     }
 
-    XtFree((char *)data);
+    if (data)
+	XtFree((char *)data);
 
     _XmAssertClipboardSelection(display, window, clip, clip->copy_lock_time);
 
@@ -2712,9 +2727,15 @@ XmClipboardRetrieve(Display *display,
     status = _XmClipboardLock(display, window);
     if (status == XmClipboardLocked)
     {
+	DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+		"XmClipboardRetrieve => XmClipboardLocked\n"));
 	return status;
     }
 
+    DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+	"XmClipboardRetrieve(fmt %s, maxlen %d)\n",
+	format_name ? format_name : "(null)", length));
+
     clip = _XmClipboardOpen(display, 0);
 
     _XmClipboardInitializeSelection(display, clip, window,
@@ -2724,6 +2745,8 @@ XmClipboardRetrieve(Display *display,
 
     if (_XmClipboardWeOwnSelection(display, clip))
     {
+	DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+	    "XmClipboardRetrieve: we own selection\n"));
 
 	format = _XmClipboardFindFormat(display, clip, format_name, 0, 0,
 					&namelen, &numformats, &formatlen);
@@ -2732,15 +2755,25 @@ XmClipboardRetrieve(Display *display,
 
 	if (format != NULL)
 	{
+	    DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+		"XmClipboardRetrieve: format != NULL\n"));
+
 	    format_id = format->format_id;
 
 	    if (format->by_name)
 	    {
+		DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+		    "XmClipboardRetrieve: by name\n"));
+
 		got_format = _XmClipboardRequestDataAndWait(display, window,
 							    format);
 
 		if (got_format)
 		{
+		    DEBUGOUT(XdbDebug(__FILE__,
+			XtWindowToWidget(display, window),
+			"XmClipboardRetrieve: got format\n"));
+
 		    XtFree((char *)format);
 
 		    _XmClipboardFindItem(display, format_id,
@@ -2753,6 +2786,9 @@ XmClipboardRetrieve(Display *display,
 
 	    if (got_format)
 	    {
+		DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+			"XmClipboardRetrieve: got format 2\n"));
+
 		_XmClipboardFindItem(display, format->data_id, &data, &dlen,
 				     NULL, XmCLIP_PROP_DONT_CARE);
 
@@ -2768,6 +2804,10 @@ XmClipboardRetrieve(Display *display,
 		    fetchlen = dlen - format->offset;
 		}
 
+		DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+			"XmClipboardRetrieve: fetchlen %d partial_copy %s\n",
+			fetchlen, XdbBoolean2String(partial_copy)));
+
 		if (clip->retrieve_locked)
 		{
 
@@ -2792,6 +2832,9 @@ XmClipboardRetrieve(Display *display,
     }
     else
     {
+	DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+	    "XmClipboardRetrieve: we don't own selection !!\n"));
+
 	_XmClipboardGetSelection(display, window, format_name, clip,
 				 &data, &dlen);
 
@@ -2871,6 +2914,10 @@ XmClipboardStartCopy(Display *display,
     unsigned new_item_id, new_label_id;
     int llen;
 
+    DEBUGOUT(XdbDebug(__FILE__, NULL,
+	"XmClipboardStartCopy(win 0x%X, clip 0x%X, time %ld)\n",
+	window, clip_label, timestamp));
+
     if (_XmClipboardLock(display, window) == XmClipboardLocked)
     {
 	return XmClipboardLocked;
