@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DebugUtil.c,v 1.25 1998/03/07 01:10:21 rwscott Exp $
+ * $Id: DebugUtil.c,v 1.26 1998/04/08 21:07:28 rwscott Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: DebugUtil.c,v 1.25 1998/03/07 01:10:21 rwscott Exp $";
+static char rcsid[] = "$Id: DebugUtil.c,v 1.26 1998/04/08 21:07:28 rwscott Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -785,6 +785,10 @@ XdbTypes[] =
 	XmNoptionLabel, XdbXMSTRING, NULL
     }
     ,
+    {
+	XmNdirectory, XdbXMSTRING, NULL
+    }
+    ,
     {
 	XmNoptionMnemonic, XdbCHAR, NULL
     }
@@ -957,6 +961,14 @@ XdbTypes[] =
 	XmNmaxLength, XdbINT, NULL
     }
     ,
+    {
+	XmNdirListItemCount, XdbINT, NULL
+    }
+    ,
+    {
+	XmNfileListItemCount, XdbINT, NULL
+    }
+    ,
     {
 	XmNeditMode, XdbEDIT_MODE, NULL
     }
@@ -969,6 +981,14 @@ XdbTypes[] =
 	XmNrecomputeSize, XdbBOOLEAN, NULL
     }
     ,
+    {
+	XmNdirectoryValid, XdbBOOLEAN, NULL
+    }
+    ,
+    {
+	XmNlistUpdated, XdbBOOLEAN, NULL
+    }
+    ,
     {
 	XmNhorizontalScrollBar, XdbWIDGET, NULL
     }
