@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DrawingArea.c,v 1.20 1997/11/21 18:01:08 miers Exp $
+ * $Id: DrawingArea.c,v 1.21 1998/05/04 22:04:36 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: DrawingArea.c,v 1.20 1997/11/21 18:01:08 miers Exp $";
+static char rcsid[] = "$Id: DrawingArea.c,v 1.21 1998/05/04 22:04:36 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -695,10 +695,13 @@ changed_managed(Widget w)
     XtGeometryResult result;
     XtWidgetGeometry req;
 
+    /*
     _XmGMEnforceMargin(w, DA_MarginWidth(w), DA_MarginHeight(w), False);
+    */
 
     if (DA_ResizePolicy(w) != XmRESIZE_NONE)
     {
+	_XmGMEnforceMargin(w, DA_MarginWidth(w), DA_MarginHeight(w), False);
 	PreferedSize(w, NULL, NULL, &wid, &hei);
 	if (wid > XtWidth(w) || hei > XtHeight(w))
 	{
