@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.24 1998/02/22 14:41:58 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.38 1998/04/14 22:54:45 rwscott Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -22,11 +22,13 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.24 1998/02/22 14:41:58 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.38 1998/04/14 22:54:45 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
 
+#include <stdlib.h>
+#include <unistd.h>
 #include <Xm/XmP.h>
 #include <Xm/List.h>
 #include <Xm/SelectioBP.h>
@@ -55,7 +57,10 @@ static void destroy(Widget w);
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
 
+#if 0
+/* Motif has one of these */
 static void get_values_hook(Widget w, ArgList args, Cardinal *num_args);
+#endif
 
 static XmGeoMatrix geoMatrixCreate(Widget _w, Widget _from,
 				   XtWidgetGeometry *_pref);
@@ -74,6 +79,8 @@ static void _XmFileSelectionSearch(Widget w);
 
 static void _XmFsbButton(Widget w, XtPointer client, XtPointer call);
 
+static void _XmFsbFileSelect(Widget w, XtPointer client, XtPointer call);
+
 /* Extern */
 Boolean _XmSelectionBoxMatch(XmSelectionBoxWidget w);
 
@@ -341,7 +348,7 @@ XmFileSelectionBoxClassRec xmFileSelectionBoxClassRec = {
 	/* set_values            */ set_values,
 	/* set_values_hook       */ NULL,
 	/* set_values_almost     */ XtInheritSetValuesAlmost,
-	/* get_values_hook       */ get_values_hook, /* Motif has one */
+	/* get_values_hook       */ NULL /*get_values_hook*/, /* Motif has one */
 	/* accept_focus          */ NULL,
 	/* version               */ XtVersion,
 	/* callback offsets      */ NULL,
@@ -487,6 +494,7 @@ initialize(Widget request, Widget new_w,
     FS_DirList(new_w) = XmCreateScrolledList(new_w, "dirList", argl, argc);
 
     XtManageChild(FS_DirList(new_w));
+    XtAddCallback(FS_DirList(new_w), XmNbrowseSelectionCallback, _XmFsbButton, NULL);
     XtAddCallback(FS_DirList(new_w), XmNdefaultActionCallback, _XmFsbButton, NULL);
 
     /* rws file list always has a scroll bar
@@ -509,11 +517,15 @@ initialize(Widget request, Widget new_w,
     }
     else if (FS_FileListLabelString(new_w) != NULL)
     {
+	files =	XmStringCopy(FS_FileListLabelString(new_w));
+#if 0
+	/* Never free this: - it's in the converter cache */
 	XmStringFree(FS_FileListLabelString(new_w));
 	files = _XmOSGetLocalizedString(NULL,	/* FIX ME */
 					new_w,
 					XmNfileListLabelString,
 					"Files");
+#endif
     }
 
     FS_FileListLabelString(new_w) = files;
@@ -587,6 +599,14 @@ initialize(Widget request, Widget new_w,
     {
 	XtRemoveAllCallbacks(SB_List(new_w), XmNdefaultActionCallback);
 	XtAddCallback(SB_List(new_w), XmNdefaultActionCallback, _XmFsbButton, NULL);
+
+	/*
+	 * We also need to override the selection callback for the
+	 * file list since an "empty" file list (which has a single fake
+	 * entry of " [ .. ] ") cannot be selected from.
+	 */
+	XtRemoveAllCallbacks(SB_List(new_w), XmNbrowseSelectionCallback);
+	XtAddCallback(SB_List(new_w), XmNbrowseSelectionCallback, _XmFsbFileSelect, (XtPointer)new_w);
     }
     if (SB_OkButton(new_w))
     {
@@ -707,13 +727,13 @@ set_values(Widget old, Widget request, Widget new_w,
 	refresh = True;
     }
 
-    if (FS_Directory(new_w) != FS_Directory(old) ||
-	FS_DirMask(new_w) != FS_DirMask(old) ||
-	FS_Pattern(new_w) != FS_Pattern(old) ||
-	FS_FileTypeMask(new_w) != FS_FileTypeMask(old) ||
+    if (!XmStringCompare(FS_Directory(new_w), FS_Directory(old)) ||
+	!XmStringCompare(FS_DirMask(new_w), FS_DirMask(old)) ||
+	!XmStringCompare(FS_Pattern(new_w), FS_Pattern(old)) ||
+	FS_FileTypeMask(new_w) != FS_FileTypeMask(old) /*||
 	FS_QualifySearchDataProc(new_w) != FS_QualifySearchDataProc(old) ||
 	FS_DirSearchProc(new_w) != FS_DirSearchProc(old) ||
-	FS_FileSearchProc(new_w) != FS_FileSearchProc(old))
+	FS_FileSearchProc(new_w) != FS_FileSearchProc(old)*/)
     {
 	_XmFileSelectionSearch(new_w);
     }
@@ -731,6 +751,8 @@ set_values(Widget old, Widget request, Widget new_w,
     return refresh;
 }
 
+#if 0
+/* Motif has one of these */
 static void get_values_hook(Widget w, ArgList args, Cardinal *num_args)
 {
     DEBUGOUT(XdbDebug(__FILE__, w,
@@ -740,6 +762,7 @@ static void get_values_hook(Widget w, ArgList args, Cardinal *num_args)
     DEBUGOUT(XdbPrintArgList(__FILE__, w, args, *num_args, False));
 
 }
+#endif
 
 static Dimension
 GetPreferredListWidth(Widget w)
@@ -824,8 +847,6 @@ _XmTextFix(XmGeoMatrix geoSpec,
 	   XmGeoMajorLayout layoutPtr,
 	   XmKidGeometry rowPtr)
 {
-    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmTextFix() - %s\n",
-    		XdbGeoAction2String(action)));
 
     if (!XmIsTextField(rowPtr->kid))
     {
@@ -846,6 +867,12 @@ _XmTextFix(XmGeoMatrix geoSpec,
     default:
 	break;
     }
+    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmTextFix() - %s %dx%d%+d%+d\n",
+    		XdbGeoAction2String(action),
+    		rowPtr->box.width,
+    		rowPtr->box.height,
+    		rowPtr->box.x,
+    		rowPtr->box.y ));
 }
 
 /*
@@ -872,6 +899,24 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 	    {
 		rowPtr->box.width =
 		    GetPreferredListWidth(SW_WorkWindow(rowPtr->kid));
+		rowPtr->box.height = XtHeight(rowPtr->kid);
+		/* rws 18 Mar 1998
+		   Do not let the list be any narrower than its
+		   label
+		 */
+		if ((&rowPtr[-3])->kid == FS_DirListLabel(geoSpec->composite))
+		{
+		if ((&rowPtr[-3])->box.width > rowPtr->box.width)
+		{
+		    rowPtr->box.width = (&rowPtr[-3])->box.width;
+		}
+		}
+    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
+    		XdbGeoAction2String(action),
+    		rowPtr->box.width,
+    		rowPtr->box.height,
+    		rowPtr->box.x,
+    		rowPtr->box.y ));
 	    }
 	    rowPtr++;
 	}
@@ -895,6 +940,12 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 			    GetPreferredListWidth(SW_WorkWindow(rowPtr->kid)) +
 				      rowPtr->box.width);
 		}
+    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
+    		XdbGeoAction2String(action),
+    		rowPtr->box.width,
+    		rowPtr->box.height,
+    		rowPtr->box.x,
+    		rowPtr->box.y ));
 	    }
 	    rowPtr++;
 	}
@@ -944,7 +995,10 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 	    /*
 	     * Position the file list label
 	     */
-	    fileListLabel->box.x = fileList->box.x;
+	     if (fileListLabel->kid == SB_ListLabel(geoSpec->composite))
+	     {
+		fileListLabel->box.x = fileList->box.x;
+	     }
 	    /*
 	     * Position the dir list
 	     * It takes up the rest of the space in this row
@@ -953,6 +1007,18 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 		- geoSpec->margin_w
 		- layoutPtr->row.space_between
 		- 2 * dirList->box.border_width;
+    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
+    		XdbGeoAction2String(action),
+    		rowPtr->box.width,
+    		rowPtr->box.height,
+    		rowPtr->box.x,
+    		rowPtr->box.y ));
+    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), fileList->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
+    		XdbGeoAction2String(action),
+    		fileList->box.width,
+    		fileList->box.height,
+    		fileList->box.x,
+    		fileList->box.y ));
 	}
 	break;
 
@@ -1395,17 +1461,33 @@ focus_moved_proc(Widget wid, XtPointer client_data, XtPointer data)
     XmFocusMovedCallback cbs = (XmFocusMovedCallback)data;
     Widget bb = (Widget)client_data;
 
-    DEBUGOUT(XdbDebug(__FILE__, wid, "%s:focus_moved_proc()\n", __FILE__));
+    DEBUGOUT(XdbDebug2(__FILE__, wid, bb, "%s:focus_moved_proc() - %s -> %s\n",
+    	__FILE__,
+    	cbs->old_focus ? XtName(cbs->old_focus) : "NULL",
+    	cbs->new_focus ? XtName(cbs->new_focus) : "NULL"));
     /* rws 21 Feb 1998
        I suspect that when the focus leaves the FilterText an update should
        be done. Can anyone verify this??
      */
+    if (cbs->new_focus && cbs->new_focus == FS_FilterText(bb))
+    {
+	BB_DynamicDefaultButton(bb) = SB_ApplyButton(bb);
+    }
+    else if (cbs->new_focus && cbs->new_focus == FS_DirList(bb))
+    {
+	BB_DynamicDefaultButton(bb) = SB_ApplyButton(bb);
+    }
+    else
+    {
+	BB_DynamicDefaultButton(bb) = BB_DefaultButton(bb);
+    }
     _XmBulletinBoardFocusMoved(wid, client_data, data);
 }
 
 static Boolean
 noGeoRequest(XmGeoMatrix _geoSpec)
 {
+	return True;
     if (BB_InSetValues(_geoSpec->composite) &&
 	(XtClass(_geoSpec->composite) == xmFileSelectionBoxWidgetClass))
     {
@@ -1487,6 +1569,13 @@ defaultDirSearchProc(Widget widget,
     FS_ListUpdated(widget) = True;
 }
 
+static int
+vstrcmp(const void *p1,
+        const void *p2)
+{
+    return strcmp(*((char **)p1), *((char **)p2));
+}
+
 static void
 defaultFileSearchProc(Widget widget,
 		      XtPointer data)
@@ -1530,6 +1619,9 @@ defaultFileSearchProc(Widget widget,
     _XmOSBuildFileList(dir, pat, FS_FileTypeMask(widget),
 		       &entries, &numents, &numalloc);
 
+    if (numents > 0)
+	qsort((void *)entries, numents, sizeof(String), vstrcmp);
+
     max = 64;
     table = (XmString *)XtCalloc(max, sizeof(XmString));
 
@@ -1557,6 +1649,7 @@ defaultFileSearchProc(Widget widget,
 }
 
 
+#if 0
 /*
  * dir part.  Must return an allocated string, even if of zero length.
  */
@@ -1699,12 +1792,18 @@ getPatPart(Widget w,
 
     *pattern = pat;
 }
+#endif
 
 static void
 defaultQualifySearchDataProc(Widget w,
 			     XtPointer input_data,
 			     XtPointer output_data)
 {
+/* rws 7 Apr 1998
+   use filesb/test9 if you are going to screw with this!!! If you do not
+   know how to use it don't screw with it.
+ */
+#if 0
     String dir = NULL, pat = NULL, directory = NULL, pattern = NULL;
     String mask = NULL;
     XmFileSelectionBoxCallbackStruct *in =
@@ -1712,6 +1811,7 @@ defaultQualifySearchDataProc(Widget w,
     XmFileSelectionBoxCallbackStruct *out =
 	(XmFileSelectionBoxCallbackStruct *)output_data;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "defaultQualifySearchDataProc\n"));
     /*
      * heuristic as by OSF/Motif Programmer's Reference Release 1.2.
      * What a pain in the a**
@@ -1815,6 +1915,136 @@ defaultQualifySearchDataProc(Widget w,
     out->length = XmStringLength(out->value);
     out->reason = in->reason;
     out->event = in->event;
+#else
+    XmFileSelectionBoxCallbackStruct *in =
+	(XmFileSelectionBoxCallbackStruct *)input_data;
+    XmFileSelectionBoxCallbackStruct *out =
+	(XmFileSelectionBoxCallbackStruct *)output_data;
+    String dir;
+    String pat;
+    String mask;
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "defaultQualifySearchDataProc\n"));
+
+    {
+	if (in->dir)
+	{
+	    if (!XmStringGetLtoR(in->dir, XmFONTLIST_DEFAULT_TAG, &dir))
+	    {
+	    	dir = strdup("");
+	    }
+	}
+	else
+	{
+	    if (in->mask)
+	    {
+	    String mask, pat;
+
+		if (XmStringGetLtoR(in->mask, XmFONTLIST_DEFAULT_TAG, &mask))
+		{
+		    pat = _XmOSFindPatternPart(mask);
+		    if (pat > mask)
+		    {
+			pat[-1] = '\0';
+			dir = strdup(mask);
+		    }
+		    else
+		    {
+			dir = strdup("");
+		    }
+		    XtFree(mask);
+		}
+		else
+		{
+		    dir = strdup("");
+		}
+	    }
+	    else
+	    {
+		if (!XmStringGetLtoR(FS_Directory(w), XmFONTLIST_DEFAULT_TAG, &dir))
+		{
+		    dir = strdup("");
+		}
+	    }
+	}
+    }
+    {
+	if (in->pattern)
+	{
+	    if (!XmStringGetLtoR(in->pattern, XmFONTLIST_DEFAULT_TAG, &pat))
+	    {
+	    	pat = strdup("*");
+	    }
+	}
+	else
+	{
+	    if (in->mask)
+	    {
+	    String mask;
+
+		if (XmStringGetLtoR(in->mask, XmFONTLIST_DEFAULT_TAG, &mask))
+		{
+		String tmp;
+
+		    tmp = _XmOSFindPatternPart(mask);
+		    if (strlen(pat) == 0)
+		    {
+			pat = strdup("*");
+		    }
+		    else
+		    {
+		    	pat = strdup(tmp);
+		    }
+		    XtFree(mask);
+		}
+		else
+		{
+		    pat = strdup("*");
+		}
+	    }
+	    else
+	    {
+		if (!XmStringGetLtoR(FS_Pattern(w), XmFONTLIST_DEFAULT_TAG, &pat))
+		{
+		    pat = strdup("*");
+		}
+	    }
+	}
+    }
+    {
+    String pdir, ppat;
+
+	_XmOSQualifyFileSpec(dir, pat, &pdir, &ppat);
+	XtFree(dir);
+	XtFree(pat);
+	out->dir = XmStringCreateSimple(pdir);
+	out->dir_length = XmStringLength(out->dir);
+	out->pattern = XmStringCreateSimple(ppat);
+	out->pattern_length = XmStringLength(out->pattern);
+
+	mask = XtMalloc(strlen(pdir) + strlen(ppat) + 1);
+	strcpy(mask, pdir);
+	strcat(mask, ppat);
+	XtFree(pdir);
+	XtFree(ppat);
+	out->mask = XmStringCreateSimple(mask);
+	out->mask_length = XmStringLength(out->mask);
+	XtFree(mask);
+    }
+
+    if (in->value)
+    {
+    	out->value = XmStringCopy(in->value);
+    	out->length = XmStringLength(out->value);
+    }
+    else
+    {
+    	out->value = NULL;
+    	out->length = 0;
+    }
+    out->reason = in->reason;
+    out->event = in->event;
+#endif
 }
 
 /*
@@ -1851,6 +2081,12 @@ _XmFileSelectionSearch(Widget w)
     {
 	in.mask_length = XmStringLength(FS_DirMask(w));
     }
+    else
+    {
+    	FS_DirMask(w) = XmStringCreateSimple("");
+	in.mask = FS_DirMask(w);
+    	in.mask_length = XmStringLength(in.mask);
+    }
     DEBUGOUT(XdbDebug(__FILE__, w, "\tdirMask %s (%i)\n", 
     		XdbXmString2String(FS_DirMask(w)),in.mask_length));
 
@@ -1862,6 +2098,12 @@ _XmFileSelectionSearch(Widget w)
 	*/
 	in.dir_length = XmStringLength(FS_Directory(w));
     }
+    else
+    {
+    	FS_Directory(w) = XmStringCreateSimple("");
+	in.dir = FS_Directory(w);
+    	in.dir_length = XmStringLength(in.mask);
+    }
     DEBUGOUT(XdbDebug(__FILE__, w, "\tdirectory %s (%i)\n", 
     		XdbXmString2String(FS_Directory(w)),in.dir_length));
 
@@ -1873,6 +2115,12 @@ _XmFileSelectionSearch(Widget w)
 	*/
 	in.pattern_length = XmStringLength(FS_Pattern(w));
     }
+    else
+    {
+    	FS_Pattern(w) = XmStringCreateSimple("");
+	in.pattern = FS_Pattern(w);
+    	in.pattern_length = XmStringLength(in.mask);
+    }
     DEBUGOUT(XdbDebug(__FILE__, w, "\tpattern %s (%i)\n", 
     		XdbXmString2String(FS_Pattern(w)),in.pattern_length));
 
@@ -1883,10 +2131,25 @@ _XmFileSelectionSearch(Widget w)
 
     /*
      * dir search
+     *
+     * I'm guessing that the XmFS_DIR_SEARCH_PROC bit is a flag
+     * to keep the directory list selection callback from causing
+     * an infinite loop.
+     *		-- mu@echo-on.net 1998.03.17
      */
     FS_DirectoryValid(w) = False;
     FS_ListUpdated(w) = False;
-    (*FS_DirSearchProc(w)) (w, (XtPointer)&out);
+    if ((FS_StateFlags(w) & XmFS_DIR_SEARCH_PROC) == 0)
+    {
+	FS_StateFlags(w) |= XmFS_DIR_SEARCH_PROC;
+	(*FS_DirSearchProc(w)) (w, (XtPointer)&out);
+	FS_StateFlags(w) &= ~XmFS_DIR_SEARCH_PROC;
+    }
+    if (FS_ListUpdated(w))
+    {
+	XmListSelectPos(FS_DirList(w), 1, False);	/* True; Danny */
+	XmListSetKbdItemPos(FS_DirList(w), 1);
+    }
 
     /*
      * file search
@@ -1896,7 +2159,12 @@ _XmFileSelectionSearch(Widget w)
 	char *value;
 
 	FS_ListUpdated(w) = False;
-	(*FS_FileSearchProc(w)) (w, (XtPointer)&out);
+	if ((FS_StateFlags(w) & XmFS_IN_FILE_SEARCH) == 0)
+	{
+	    FS_StateFlags(w) |= XmFS_IN_FILE_SEARCH;
+	    (*FS_FileSearchProc(w)) (w, (XtPointer)&out);
+	    FS_StateFlags(w) &= ~XmFS_IN_FILE_SEARCH;
+	}
 
 	if (FS_ListUpdated(w) && SB_ListItemCount(w) == 0)
 	{
@@ -1904,6 +2172,7 @@ _XmFileSelectionSearch(Widget w)
 	     * This does actually get into the list but, you have to left
 	     * click on the list hsb before it appears?????
 	     */
+	    FS_StateFlags(w) |= XmFS_NO_MATCH;
 	    argc = 0;
 	    XtSetArg(argl[argc], XmNfileListItemCount, 1); argc++;
 	    XtSetArg(argl[argc], XmNfileListItems, &FS_NoMatchString(w)); argc++;
@@ -1926,6 +2195,7 @@ _XmFileSelectionSearch(Widget w)
 	    FS_DirSpec(w) = XmStringCopy(out.dir);
 	    XmStringGetLtoR(out.dir, XmFONTLIST_DEFAULT_TAG, &value);
 
+	    FS_StateFlags(w) &= ~XmFS_NO_MATCH;
 	    argc = 0;
 	    XtSetArg(argl[argc], XmNvalue, value); argc++;
 	    XtSetValues(SB_Text(w), argl, argc);
@@ -2059,6 +2329,7 @@ void
 XmFileSelectionDoSearch(Widget w,
 			XmString dirmask)
 {
+#if 0
     DEBUGOUT(XdbDebug(__FILE__, w, "XmFileSelectionDoSearch() - %s\n",
     		XdbXmString2String(dirmask)));
     if (dirmask != (XmString)NULL)
@@ -2072,10 +2343,84 @@ XmFileSelectionDoSearch(Widget w,
     }
     else
     {
-	FS_DirMask(w) = XmStringCreateSimple(XmTextFieldGetString(FS_FilterText(w)));
+    char *tf_string;
+
+	XmStringFree(FS_DirMask(w));
+	XmStringFree(FS_Pattern(w));
+	XmStringFree(FS_Directory(w));
+	FS_Pattern(w) = (XmString)NULL;
+	FS_DirMask(w) = (XmString)NULL;
+	FS_Directory(w) = (XmString)NULL;
+	if (XtIsManaged(FS_FilterText(w)))
+	{
+	    tf_string = XmTextFieldGetString(FS_FilterText(w));
+	    FS_DirMask(w) = XmStringCreateSimple(tf_string);
+	    XtFree(tf_string);
+	}
+	else if (XtIsManaged(SB_Text(w)))
+	{
+	    tf_string = XmTextFieldGetString(SB_Text(w));
+	    FS_Directory(w) = XmStringCreateSimple(tf_string);
+	    XtFree(tf_string);
+	}
     }
 
     _XmFileSelectionSearch((Widget)w);
+#else
+       	XmFileSelectionBoxCallbackStruct in, out;
+       	String mask;
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "XmFileSelectionDoSearch() - %s\n",
+    		XdbXmString2String(dirmask)));
+
+	in.reason = NULL;
+	in.event = NULL;
+	in.value = NULL;
+	in.length = 0;
+	in.dir = NULL;
+	in.dir_length = 0;
+	in.mask = NULL;
+	in.mask_length = 0;
+	if (dirmask != (XmString)NULL)
+	{
+		in.mask = dirmask;
+	}
+	else
+	{
+		if (XtIsManaged(FS_FilterText(w)))
+		{
+		    mask = XmTextFieldGetString(FS_FilterText(w));
+		    in.mask = XmStringCreateSimple(mask);
+		    in.mask_length = XmStringLength(in.mask);
+		    XtFree(mask);
+		}
+		else
+		{
+		    mask = XmTextFieldGetString(SB_Text(w));
+		    in.dir = XmStringCreateSimple(mask);
+		    in.dir_length = XmStringLength(in.dir);
+		    XtFree(mask);
+		}
+	}
+	in.pattern = NULL;
+	in.pattern_length = 0;
+	(*FS_QualifySearchDataProc(w)) ((Widget)w, (XtPointer)&in, (XtPointer)&out);
+	XmStringFree(FS_DirMask(w));
+	XmStringFree(FS_Pattern(w));
+	XmStringFree(FS_Directory(w));
+	FS_DirMask(w) = out.mask;
+	FS_Pattern(w) = out.pattern;
+	FS_Directory(w) = out.dir;
+
+	if (XmStringGetLtoR(out.mask, XmFONTLIST_DEFAULT_TAG, &mask))
+	{
+	    XmTextFieldSetString(FS_FilterText(w), mask);
+	    XmTextFieldSetInsertionPosition(FS_FilterText(w),
+		    XmTextFieldGetLastPosition(FS_FilterText(w)));
+	    XtFree(mask);
+	}
+	_XmFileSelectionSearch((Widget)w);
+#endif
 }
 
 /*
@@ -2133,9 +2478,13 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
 	else
 	{
 	    /* List double-click */
-
 	    XmListCallbackStruct *ls = (XmListCallbackStruct *) call;
 
+            if (FS_StateFlags(sb) & XmFS_NO_MATCH) {
+                XmListDeselectAllItems(w);
+                goto BailOut;
+            }
+
 	    cbs.value = XmStringCopy(ls->item);
 
 	    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton LIST '%s'\n",
@@ -2180,19 +2529,39 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
     }
     else if (w == FS_FilterText(sb) || w == SB_ApplyButton(sb))
     {
-	/* There is no apply button - this is Filter */
+       	XmFileSelectionBoxCallbackStruct in, out;
+       	String mask;
+
 	DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton FILTER\n"));
 
-	/* rws */ XmStringFree(FS_DirMask(sb));
-	/* rws */ XmStringFree(FS_Pattern(sb));
-	/* rws */ XmStringFree(FS_Directory(sb));
-	/* rws */ FS_DirMask(sb) = XmStringCreateSimple(XmTextFieldGetString(FS_FilterText((Widget)sb)));
-	/* rws */ FS_Pattern(sb) = (XmString)NULL;
-	/* rws */ FS_Directory(sb) = (XmString)NULL;
-	/* rws FS_Pattern(sb) = XmStringCreateSimple(XmTextFieldGetString(FS_FilterText((Widget)sb))); */
+	in.reason = a->reason;
+	in.event = a->event;
+	in.value = NULL;
+	in.length = 0;
+	in.dir = NULL;
+	in.dir_length = 0;
+	mask = XmTextFieldGetString(FS_FilterText(sb));
+	in.mask = XmStringCreateSimple(mask);
+	in.mask_length = XmStringLength(in.mask);
+	XtFree(mask);
+	in.pattern = NULL;
+	in.pattern_length = 0;
+	(*FS_QualifySearchDataProc(sb)) ((Widget)sb, (XtPointer)&in, (XtPointer)&out);
+	XmStringFree(FS_DirMask(sb));
+	XmStringFree(FS_Pattern(sb));
+	XmStringFree(FS_Directory(sb));
+	FS_DirMask(sb) = out.mask;
+	FS_Pattern(sb) = out.pattern;
+	FS_Directory(sb) = out.dir;
 
+	if (XmStringGetLtoR(out.mask, XmFONTLIST_DEFAULT_TAG, &mask))
+	{
+	    XmTextFieldSetString(FS_FilterText(sb), mask);
+	    XmTextFieldSetInsertionPosition(FS_FilterText(sb),
+		    XmTextFieldGetLastPosition(FS_FilterText(sb)));
+	    XtFree(mask);
+	}
 	_XmFileSelectionSearch((Widget)sb);
-
     }
     else if (w == SB_HelpButton(sb))
     {
@@ -2204,50 +2573,51 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
     {
 	/* Somebody selected a directory */
 	XmListCallbackStruct *ls = (XmListCallbackStruct *) call;
+       	XmFileSelectionBoxCallbackStruct in, out;
+       	String mask;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton DIRLIST\n"));
+	DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton DirList\n"));
 
+	in.reason = ls->reason;
+	in.event = ls->event;
+	in.value = NULL;
+	in.length = 0;
+	in.dir = ls->item;
+	in.dir_length = XmStringLength(in.dir);
+	mask = XmTextFieldGetString(FS_FilterText(sb));
+	in.mask = XmStringCreateSimple(mask);
+	in.mask_length = XmStringLength(in.mask);
+	XtFree(mask);
+	in.pattern = NULL;
+	in.pattern_length = 0;
+	(*FS_QualifySearchDataProc(sb)) ((Widget)sb, (XtPointer)&in, (XtPointer)&out);
 	XmStringFree(FS_DirMask(sb));
 	XmStringFree(FS_Pattern(sb));
 	XmStringFree(FS_Directory(sb));
-	FS_DirMask(sb) = XmStringCreateSimple(XmTextFieldGetString(FS_FilterText((Widget)sb)));
-	FS_Pattern(sb) = (XmString)NULL;
-	FS_Directory(sb) = XmStringCopy(ls->item);
-	_XmFileSelectionSearch((Widget)sb);
+	FS_DirMask(sb) = out.mask;
+	FS_Pattern(sb) = out.pattern;
+	FS_Directory(sb) = out.dir;
+
+	if (XmStringGetLtoR(out.mask, XmFONTLIST_DEFAULT_TAG, &mask))
+	{
+	    XmTextFieldSetString(FS_FilterText(sb), mask);
+	    XmTextFieldSetInsertionPosition(FS_FilterText(sb),
+		    XmTextFieldGetLastPosition(FS_FilterText(sb)));
+	    XtFree(mask);
+	}
+       if (ls->reason == XmCR_BROWSE_SELECT)
+       {
+       }
+       else
+       {
+	    _XmFileSelectionSearch((Widget)sb);
+	}
     }
     else
     {
 	DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton : untreated case !!\n"));
     }
 
-    /* Free what you've allocated */
-#if 0
-    /* rws 13 Dec 1997
-       The strings in the cbs were allocated and therefore should be freed,
-       not the instance variables!
-     */
-    if (FS_DirMask(sb))
-    {
-	XmStringFree(FS_DirMask(sb));
-    }
-    if (FS_Directory(sb))
-    {
-	XmStringFree(FS_Directory(sb));
-    }
-    if (FS_Pattern(sb))
-    {
-	XmStringFree(FS_Pattern(sb));
-    }
-#else
-    if (cbs.value)
-    {
-    	XmStringFree(cbs.value);
-    }
-    XmStringFree(cbs.mask);
-    XmStringFree(cbs.dir);
-    XmStringFree(cbs.pattern);
-#endif
-
     /* AutoUnmanage */
     if (BB_AutoUnmanage(sb))
     {
@@ -2275,6 +2645,75 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
 	 *	FS_DirList(sb)
 	 */
     }
+
+BailOut:
+    /* Free what you've allocated */
+    if (cbs.value != NULL)
+	XmStringFree(cbs.value);
+    XmStringFree(cbs.mask);
+    XmStringFree(cbs.dir);
+    XmStringFree(cbs.pattern);
+}
+
+/*
+ * Stolen from SelectionBox.c
+ *
+ * We need a private routine for this here, because we need to keep the
+ * artificial entry in empty file lists from being selected.
+ */
+static void
+_XmFsbFileSelect(Widget w, XtPointer client, XtPointer call)
+{
+    XmFileSelectionBoxWidget fsb = (XmFileSelectionBoxWidget)client;
+    char *s;
+    XmListCallbackStruct *lp = (XmListCallbackStruct *) call;
+
+    /*
+     * Disallow the selection if there was no match.
+     * This will happen if the glob pattern didn't match anything,
+     * in this case, the file list will contain a single entry
+     * which no one is supposed to be able to select.
+     */
+    if (FS_StateFlags(fsb) & XmFS_NO_MATCH)
+    {
+	DEBUGOUT(XdbDebug(__FILE__, w,
+			  "_XmFsbFileSelect (Selection with XmFS_NO_MATCH)\n"));
+
+        /*
+	 * The lp->item_position should always be one so we'll just assume
+	 * that it is.
+	 */
+        XmListDeselectPos(w, 1);
+	return;
+    }
+
+    if (SB_Text(fsb) == NULL)
+    {
+	DEBUGOUT(XdbDebug(__FILE__, w,
+			  "_XmFsbFileSelect (No TextField)\n"));
+	return;
+    }
+
+    if (lp == NULL || lp->item == NULL)
+    {
+	DEBUGOUT(XdbDebug(__FILE__, w,
+			  "_XmFsbFileSelect (NULL)\n"));
+	return;
+    }
+
+    if (!XmStringGetLtoR(lp->item, XmFONTLIST_DEFAULT_TAG, &s))
+    {
+	DEBUGOUT(XdbDebug(__FILE__, w,
+			  "_XmFsbFileSelect (Couldn't convert to string)\n"));
+	return;
+    }
+
+    DEBUGOUT(XdbDebug(__FILE__, w,
+		      "_XmFsbFileSelect '%s'\n", s));
+    XmTextFieldSetString(SB_Text(fsb), s);
+    XmTextFieldSetInsertionPosition(SB_Text(fsb),
+				    XmTextFieldGetLastPosition(SB_Text(fsb)));
+    XtFree(s);
 }
 
 /*
