@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Form.c,v 1.106 1998/02/07 15:20:23 rwscott Exp $
+ * $Id: Form.c,v 1.109 1998/05/09 19:57:40 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Form.c,v 1.106 1998/02/07 15:20:23 rwscott Exp $";
+static const char rcsid[] = "$Id: Form.c,v 1.109 1998/05/09 19:57:40 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -48,6 +48,9 @@ static char rcsid[] = "$Id: Form.c,v 1.106 1998/02/07 15:20:23 rwscott Exp $";
 
 /* Forward Declarations */
 
+static void _XmFormExportOffset(Widget widget, int offset, XtArgVal *value);
+static XmImportOperator _XmFormImportOffset(Widget widget, int offset, XtArgVal *value);
+
 static void class_part_initialize(WidgetClass w_class);
 
 static void initialize(Widget request, Widget new_w,
@@ -262,22 +265,34 @@ static XmSyntheticResource constraint_syn_resources[] =
     {
 	XmNtopOffset,
 	sizeof(int), Offset(atta[TOP].offset),
+	_XmFormExportOffset, _XmFormImportOffset
+	/*
 	_XmFromVerticalPixels, _XmToVerticalPixels
+	*/
     },
     {
 	XmNbottomOffset,
 	sizeof(int), Offset(atta[BOTTOM].offset),
+	_XmFormExportOffset, _XmFormImportOffset
+	/*
 	_XmFromVerticalPixels, _XmToVerticalPixels
+	*/
     },
     {
 	XmNleftOffset,
 	sizeof(int), Offset(atta[LEFT].offset),
+	_XmFormExportOffset, _XmFormImportOffset
+	/*
 	_XmFromHorizontalPixels, _XmToHorizontalPixels
+	*/
     },
     {
 	XmNrightOffset,
 	sizeof(int), Offset(atta[RIGHT].offset),
+	_XmFormExportOffset, _XmFormImportOffset
+	/*
 	_XmFromHorizontalPixels, _XmToHorizontalPixels
+	*/
     }
 };
 #if 0
@@ -721,7 +736,19 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     }
     else
     {
+#if 0
+	/* rws 24 Mar 1998
+	   This was taken out to fix up mgv's magstep form. Essentially if
+	   the parent says NO to a size change then relayout ourself with
+	   the requested geo and the current size.
+	 */
 	return XtGeometryNo;
+#else
+	mr = *request;
+	wd = XtWidth(f);
+	ht = XtHeight(f);
+	_XmFormLayout(f, w, &mr, &wd, &ht);
+#endif
     }
 
     /*
@@ -2817,13 +2844,13 @@ _XmFormConfigureChildren(Widget f, Widget cw, XtWidgetGeometry *cg)
 	       XtX(child) = FCP_X(con);
 	       XtY(child) = FCP_Y(con);
 	       XtWidth(child) = (FCP_W(con) -
-				(2 * child->core.border_width)) <= 0
-					? 1
+				(2 * child->core.border_width)) < 0
+					? 0
 					: FCP_W(con) -
 						(2 * child->core.border_width);
 	       XtHeight(child) = (FCP_H(con) -
-				(2 * child->core.border_width)) <= 0
-					? 1
+				(2 * child->core.border_width)) < 0
+					? 0
 					: FCP_H(con) -
 						(2 * child->core.border_width);
 	}
@@ -2833,13 +2860,13 @@ _XmFormConfigureChildren(Widget f, Widget cw, XtWidgetGeometry *cg)
 			       FCP_X(con),
 			       FCP_Y(con),
 			       (FCP_W(con) -
-				(2 * child->core.border_width)) <= 0
-					? 1
+				(2 * child->core.border_width)) < 0
+					? 0
 					: FCP_W(con) -
 						(2 * child->core.border_width),
 			       (FCP_H(con) -
-				(2 * child->core.border_width)) <= 0
-					? 1
+				(2 * child->core.border_width)) < 0
+					? 0
 					: FCP_H(con) -
 						(2 * child->core.border_width),
 			       child->core.border_width);
@@ -2848,6 +2875,21 @@ _XmFormConfigureChildren(Widget f, Widget cw, XtWidgetGeometry *cg)
 #undef	Wants
 }
 
+static void
+_XmFormExportOffset(Widget widget, int offset, XtArgVal *value)
+{
+	if (*value == XmINVALID_DIMENSION)
+	{
+		*value = 0;
+	}
+}
+
+static XmImportOperator
+_XmFormImportOffset(Widget widget, int offset, XtArgVal *value)
+{
+	return XmSYNTHETIC_LOAD;
+}
+
 /************************ EXTERNAL INTERFACE ****************************/
 
 Widget
