@@ -1,6 +1,6 @@
 /** 
  *
- * $Id: Frame.c,v 1.34 1998/02/22 01:00:35 rwscott Exp $
+ * $Id: Frame.c,v 1.35 1998/04/03 14:22:46 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Frame.c,v 1.34 1998/02/22 01:00:35 rwscott Exp $";
+static char rcsid[] = "$Id: Frame.c,v 1.35 1998/04/03 14:22:46 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -568,6 +568,15 @@ geometry_manager(Widget w,
 
 	return XtGeometryNo;
     }
+    if (((request->request_mode & CWWidth) && request->width == 0) ||
+        ((request->request_mode & CWWidth) && request->width == 0))
+    {
+	_XmWarning(w, "%s(%d):geometry_manager() - Widget requesting %s from %s\n\tReturning XtGeometryNo\n",
+		__FILE__, __LINE__, 
+		XdbWidgetGeometry2String(request),
+		XtName(fw));
+	return XtGeometryNo;
+    }
 
     /* Calculate desired Frame geometry in wd/ht from requested child
      * geometry in "request" */
