@@ -1,6 +1,6 @@
 /**
  *
- * $Id: GeoUtils.c,v 1.18 1997/12/13 22:27:04 rwscott Exp $
+ * $Id: GeoUtils.c,v 1.22 1998/04/10 12:35:37 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: GeoUtils.c,v 1.18 1997/12/13 22:27:04 rwscott Exp $";
+static const char rcsid[] = "$Id: GeoUtils.c,v 1.22 1998/04/10 12:35:37 u27113 Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -811,7 +811,10 @@ _XmGeoArrangeBoxes(XmGeoMatrix geoSpec,
     Position oy;
 
     DEBUGOUT(XdbDebug(__FILE__, geoSpec->composite,
-		      "_XmGeoArrangeBoxes x: %d y: %d\n", x, y));
+		      "_XmGeoArrangeBoxes %dx%d%+d%+d\n", 
+		      pW ? *pW : 0,
+		      pH ? *pH : 0,
+		      x, y));
 
     oy = y;
 
@@ -905,10 +908,11 @@ _XmGeoBoxesSameWidth(XmKidGeometry rowPtr,
     XmKidGeometry boxes = rowPtr;
 
     DEBUGOUT(XdbDebug(__FILE__, NULL,
-		      "_XmGeoBoxesSameWidth: %d\n", width));
+		      "_XmGeoBoxesSameWidth: %d", width));
 
     if (width == 0)
     {
+	DEBUGOUT(XdbDebug0(__FILE__, NULL, "\n"));
 	return 0;
     }
     else if (width >= 1)
@@ -923,14 +927,17 @@ _XmGeoBoxesSameWidth(XmKidGeometry rowPtr,
 	}
     }
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, " %i", wd));
     boxes = rowPtr;
 
     while (boxes->kid != NULL)
     {
+	DEBUGOUT(XdbDebug0(__FILE__, NULL, " %s", XtName(boxes->kid)));
 	boxes->box.width = wd;
 	boxes++;
     }
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "\n"));
     return wd;
 }
 
@@ -945,10 +952,11 @@ _XmGeoBoxesSameHeight(XmKidGeometry rowPtr,
     XmKidGeometry boxes = rowPtr;
 
     DEBUGOUT(XdbDebug(__FILE__, NULL,
-		      "_XmGeoBoxesSameHeight: %d\n", height));
+		      "_XmGeoBoxesSameHeight: %d", height));
 
     if (height == 0)
     {
+	DEBUGOUT(XdbDebug0(__FILE__, NULL, "\n"));
 	return 0;
     }
     else if (height >= 1)
@@ -963,13 +971,16 @@ _XmGeoBoxesSameHeight(XmKidGeometry rowPtr,
 	}
     }
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, " %i", ht));
     boxes = rowPtr;
     while (boxes->kid != NULL)
     {
+	DEBUGOUT(XdbDebug0(__FILE__, NULL, " %s", XtName(boxes->kid)));
 	boxes->box.height = ht;
 	boxes++;
     }
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "\n"));
     return ht;
 }
 
@@ -1463,7 +1474,7 @@ Boolean
 _XmGeoSetupKid(XmKidGeometry geo,
 	       Widget kidWid)
 {
-    DEBUGOUT(XdbDebug(__FILE__, XtParent(kidWid), "_XmGeoSetupKid\n"));
+    DEBUGOUT(XdbDebug2(__FILE__, XtParent(kidWid), kidWid, "_XmGeoSetupKid\n"));
 
     if (!kidWid)
     {
@@ -1488,7 +1499,9 @@ _XmGeometryEqual(Widget wid,
 		 XtWidgetGeometry *geoA,
 		 XtWidgetGeometry *geoB)
 {
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmGeometryEqual\n"));
+    DEBUGOUT(XdbDebug(__FILE__, wid, "_XmGeometryEqual (%s) (%s)\n",
+    	XdbWidgetGeometry2String(geoA),
+    	XdbWidgetGeometry2String(geoB)));
 
     return (bcmp(geoA, geoB, sizeof(XtWidgetGeometry)) == 0);
 }
@@ -1505,7 +1518,7 @@ _XmHandleSizeUpdate(Widget w,
     Dimension wd, ht;
     XtWidgetGeometry request;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmHandleSizeUpdate\n"));
+    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmHandleSizeUpdate %dx%d\n",XtWidth(w),XtHeight(w)));
 
     geo = (createMatrix) (w, NULL, NULL);
 
@@ -1524,14 +1537,21 @@ _XmHandleSizeUpdate(Widget w,
 
     _XmGeoArrangeBoxes(geo, 0, 0, &wd, &ht);
 
+    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmHandleSizeUpdate %dx%d %dx%d\n",XtWidth(w),XtHeight(w),wd,ht));
     if (policy == XmRESIZE_GROW)
     {
 	/* check the return against the original.  If the procedure would
 	 * like the BB to shrink, call again */
 	if (wd < XtWidth(w) || ht < XtHeight(w))
 	{
-	    wd = XtWidth(w);
-	    ht = XtHeight(w);
+	    if (wd < XtWidth(w))
+	    {
+		wd = XtWidth(w);
+	    }
+	    if (ht < XtHeight(w))
+	    {
+		ht = XtHeight(w);
+	    }
 	    _XmGeoArrangeBoxes(geo, 0, 0, &wd, &ht);
 	}
     }
@@ -1901,7 +1921,7 @@ _XmHandleGeometryManager(Widget w,
     switch (res)
     {
     case XtGeometryYes:
-	if (! _XmGeometryEqual(instigator, desired, cache->in_layout))
+	if (_XmGeometryEqual(instigator, desired, cache->in_layout))
 	{
 	    if (desired->request_mode & XtCWQueryOnly)
 	    {
@@ -1925,7 +1945,7 @@ _XmHandleGeometryManager(Widget w,
 	}
 	else
 	{
-	    res = XtGeometryNo;
+	    res = XtGeometryAlmost;
 	    *allowed = *cache->in_layout;
 	}
 	break;
@@ -1978,6 +1998,29 @@ _XmMakeGeometryRequest(Widget w,
 
     g1 = *desired;
 
+    /*
+     * This happens in too many places; take it together here.
+     *
+     * If we ask for 0 width/height, Xt will fail on it.
+     * Prevent this from happening by changing this to 1.
+     *
+     * Is this a hack ? Not really, in some occasions it is possible
+     * that we go through temporary stuff like this. Either we prevent
+     * this from going to Xt in all individual places, or we put it
+     * together here, as we do now.
+     */
+    if ((desired->request_mode & CWWidth) && desired->width == 0) {
+	desired->width = 1;
+	if (XtWidth(w) == 0)
+		XtWidth(w) = 1;
+    }
+    if ((desired->request_mode & CWHeight) && desired->height == 0) {
+	desired->height = 1;
+	if (XtHeight(w) == 0)
+		XtHeight(w) = 1;
+    }
+    /* End Xt crash prevention */
+
     answer = XtMakeGeometryRequest(w, desired, &response);
 
     DEBUGOUT(XdbDebug(__FILE__, w,
