@@ -31,10 +31,6 @@
  * Author:  Chris D. Peterson, Dave Sternlicht, MIT X Consortium
  */
 
-#ifndef LESSTIF_PRODUCTION
-
-#define	NEW_EDITRES
-
 #include <LTconfig.h>
 #include <XmI/DebugUtil.h>
 #include <X11/Xfuncproto.h>
@@ -58,9 +54,7 @@ typedef char *caddr_t;
 #define _LesstifEditResPutBool _LesstifEditResPut8
 #define _LesstifEditResPutResourceType _LesstifEditResPut8
 
-#ifdef	NEW_EDITRES
 static void _LesstifEditResGetStringValues(Widget, Arg *, int);
-#endif
 
 /************************************************************
  *
@@ -1516,11 +1510,9 @@ DumpValues(w, event, stream)	/* ARGSUSED */
     Arg warg[1];
     String res_value = NULL;
     GetValuesEvent *gv_event = (GetValuesEvent *)event;
-#ifdef	NEW_EDITRES
     char buffer[64], *str;
 
     res_value = buffer;
-#endif
 
     /* put the count in the stream. */
 
@@ -1531,7 +1523,6 @@ DumpValues(w, event, stream)	/* ARGSUSED */
     XtSetArg(warg[0], gv_event->name, &res_value);
 /*_XtGetStringValues(gv_event->widgets[0].ids[0], warg, 1);*/
 /* New Editres stuff here */
-#ifdef	NEW_EDITRES
     if ((str = VerifyWidget(w, &(gv_event->widgets[0]))) != NULL)
     {
 	_LesstifEditResPutString8(stream, str);
@@ -1544,11 +1535,6 @@ DumpValues(w, event, stream)	/* ARGSUSED */
 	    res_value = "NoValue";
 	_LesstifEditResPutString8(stream, res_value);
     }
-#else
-    if (!res_value)
-	res_value = "NoValue";
-    _LesstifEditResPutString8(stream, res_value);
-#endif
 /* End New Editres stuff */
     return (NULL);
 }
@@ -2005,7 +1991,6 @@ LoadResources(w)
 			      XtNumber(resources), NULL, (Cardinal)0);
 }
 
-#ifdef	NEW_EDITRES
 static void
 _LesstifEditResCvtWarningHandler(String name,
 				 String type,
@@ -2203,6 +2188,3 @@ _LesstifEditResGetStringValues(Widget w, Arg *warg, int numargs)
 
     XtFree((char *)res_list);
 }
-#endif /* LESSTIF_PRODUCTION */
-
-#endif /* NEW_EDITRES */
