@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.58 1998/03/08 18:43:05 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.63 1998/05/08 19:41:50 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.58 1998/03/08 18:43:05 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.63 1998/05/08 19:41:50 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -503,6 +503,7 @@ CreateInsensitiveGC(Widget w)
 {
     XGCValues values;
     XtGCMask mask;
+    char *stipple_name;
 
     mask = GCForeground | GCBackground | GCFillStyle | GCFunction | GCStipple |
 	GCPlaneMask | GCSubwindowMode | GCGraphicsExposures |
@@ -517,21 +518,14 @@ CreateInsensitiveGC(Widget w)
     values.ts_x_origin = values.ts_y_origin = 0;
 
     if ((Lab_TextRect_x(w) & 1) ^ (Lab_TextRect_y(w) & 1))
-    {
-	values.stipple = XmGetPixmapByDepth(XtScreen(w),
-					    XmODD_STIPPLE_IMAGE,
-					    WhitePixelOfScreen(XtScreen(w)),
-					    BlackPixelOfScreen(XtScreen(w)),
-					    1);
-    }
+	stipple_name = XmODD_STIPPLE_IMAGE;
     else
-    {
-	values.stipple = XmGetPixmapByDepth(XtScreen(w),
-					    XmEVEN_STIPPLE_IMAGE,
-					    WhitePixelOfScreen(XtScreen(w)),
-					    BlackPixelOfScreen(XtScreen(w)),
-					    1);
-    }
+	stipple_name = XmEVEN_STIPPLE_IMAGE;
+    values.stipple = XmGetPixmapByDepth(XtScreen(w),
+					stipple_name,
+					_XmWhitePixelOfObject(w),
+					_XmBlackPixelOfObject(w),
+					1);
 
     Lab_InsensitiveGC(w) = XtGetGC(w, mask, &values);
 }
@@ -775,6 +769,14 @@ set_values(Widget old, Widget request, Widget new_w,
     if (XtSensitive(old) != XtSensitive(new_w))
     {
     	refresh_needed = True;
+	if (Lab_RecomputeSize(new_w))
+	{
+	    relayout_needed = True;
+	}
+	else
+	{
+	    refresh_needed = True;
+	}
     }
     /* This is a Primitive resource but we have the GC's for it... */
     if (Prim_Foreground(new_w) != Prim_Foreground(old) ||
@@ -1028,7 +1030,15 @@ expose(Widget w, XEvent *event, Region region)
      * recompute the labels internals... they could be off if resize
      * hadn't been called after a change (see set_values)
      */
+#if 0
+    /* rws 8 May 1998
+       This messes up Mozilla's Bookmark menu buttons which are sub-classed
+       from PushB and CascadeB.
+     */
     resize(w);
+#else
+    (XtClass(w)->core_class.resize)(w);
+#endif
 
     /* use the right GC */
     if (XtSensitive(w))
@@ -1521,7 +1531,8 @@ _XmLabelShowsAccelerators(Widget w)
     {
 	return False;
     }
-    if (RC_Type(XtParent(w)) != XmMENU_PULLDOWN)
+    if (RC_Type(XtParent(w)) != XmMENU_PULLDOWN &&
+        RC_Type(XtParent(w)) != XmMENU_POPUP)
     {
 	return False;
     }
@@ -1655,7 +1666,14 @@ _XmCalcLabelDimensions(Widget w)
     else
 	/* pixmap */
     {
-	_XmLabelGetPixmapSize(w, Lab_Pixmap(w), &cw, &ch);
+	if (XtIsSensitive(w))
+	{
+	    _XmLabelGetPixmapSize(w, Lab_Pixmap(w), &cw, &ch);
+	}
+	else
+	{
+	    _XmLabelGetPixmapSize(w, Lab_PixmapInsensitive(w), &cw, &ch);
+	}
     }
 
     width = (Lab_Highlight(w)
