@@ -1,6 +1,6 @@
 /**
  *
- * $Id: LabelG.c,v 1.51 1998/03/03 21:58:51 rwscott Exp $
+ * $Id: LabelG.c,v 1.54 1998/04/16 23:32:54 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: LabelG.c,v 1.51 1998/03/03 21:58:51 rwscott Exp $";
+static const char rcsid[] = "$Id: LabelG.c,v 1.54 1998/04/16 23:32:54 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -538,7 +538,14 @@ _XmCalcLabelGDimensions(Widget w)
     }
     else /* pixmap */
     {
-	_XmLabelGetPixmapSize(w, LabG_Pixmap(w), &cw, &ch);
+	if (XtIsSensitive(w))
+	{
+	    _XmLabelGetPixmapSize(w, LabG_Pixmap(w), &cw, &ch);
+	}
+	else
+	{
+	    _XmLabelGetPixmapSize(w, LabG_PixmapInsensitive(w), &cw, &ch);
+	}
     }
 
     width = (LabG_Highlight(w)
@@ -993,6 +1000,7 @@ CreateInsensitiveGC(Widget w)
 {
     XGCValues values;
     XtGCMask mask;
+    char *stipple_name;
 
     mask = GCForeground | GCBackground | GCFillStyle | GCFunction | GCStipple |
 	GCSubwindowMode | GCGraphicsExposures | GCPlaneMask |
@@ -1007,21 +1015,14 @@ CreateInsensitiveGC(Widget w)
     values.ts_x_origin = values.ts_y_origin = 0;
 
     if ((LabG_TextRect_x(w) & 1) ^ (LabG_TextRect_y(w) & 1))
-    {
-	values.stipple = XmGetPixmapByDepth(XtScreen(w),
-					    XmODD_STIPPLE_IMAGE,
-					    WhitePixelOfScreen(XtScreen(w)),
-					    BlackPixelOfScreen(XtScreen(w)),
-					    1);
-    }
+	stipple_name = XmODD_STIPPLE_IMAGE;
     else
-    {
-	values.stipple = XmGetPixmapByDepth(XtScreen(w),
-					    XmEVEN_STIPPLE_IMAGE,
-					    WhitePixelOfScreen(XtScreen(w)),
-					    BlackPixelOfScreen(XtScreen(w)),
-					    1);
-    }
+	stipple_name = XmEVEN_STIPPLE_IMAGE;
+    values.stipple = XmGetPixmapByDepth(XtScreen(w),
+					stipple_name,
+					_XmWhitePixelOfObject(w),
+					_XmBlackPixelOfObject(w),
+					1);
 
     LabG_InsensitiveGC(w) = XtGetGC(w, mask, &values);
 }
@@ -1061,6 +1062,8 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
+    XmString xmstring;
+
     DEBUGOUT(XdbDebug(__FILE__, new_w,
 		      "LabelGadget initialize w,h before: %d %d\n",
 		      XtWidth(new_w), XtHeight(new_w)));
@@ -1083,14 +1086,13 @@ initialize(Widget request, Widget new_w,
     if (LabG_Label(new_w) == (_XmString)XmUNSPECIFIED ||
 	LabG_Label(new_w) == (_XmString)0)
     {				/* Shouldn't be necessary but is */
-	XmString xmstring;
-
 	xmstring = _XmOSGetLocalizedString((char *)NULL,
 					   (Widget)new_w,
 					   XmNlabelString,
 					   XtName(new_w));
 
 	LabG_Label(new_w) = _XmStringCreate(xmstring);
+	XmStringFree(xmstring);
     }
 
     if (_XmStringIsXmString((XmString)LabG_Label(new_w)))
@@ -1105,8 +1107,10 @@ initialize(Widget request, Widget new_w,
     }
     else
     {
+	xmstring = XmStringCreateLocalized("");
 	LabG_AcceleratorText(new_w) =
-	    _XmStringCreate(XmStringCreateLocalized(""));
+	    _XmStringCreate(xmstring);
+	XmStringFree(xmstring);
     }
 
     /* if the parent is a row column, set the menu_type to
@@ -1217,6 +1221,8 @@ destroy(Widget w)
 {
     XtReleaseGC(w, LabG_NormalGC(w));
     XtReleaseGC(w, LabG_InsensitiveGC(w));
+    _XmStringFree(LabG_Label(w));
+    _XmStringFree(LabG_AcceleratorText(w));
     _XmManagerUninstallAccelerator(XtParent(w), w);
     _XmManagerUninstallMnemonic(XtParent(w), w);
 
@@ -1418,6 +1424,7 @@ set_values(Widget old, Widget request, Widget new_w,
     if (XtSensitive(old) != XtSensitive(new_w))
     {
     	refresh_needed = True;
+	relayout_needed = True;
     }
     /* This is a Primitive resource but we have the GC's for it... */
     if (XmParentForeground(new_w) != XmParentForeground(old))
