@@ -1,6 +1,6 @@
 /**
  *
- * $Id: MainW.c,v 1.26 1998/02/22 01:00:42 rwscott Exp $
+ * $Id: MainW.c,v 1.28 1998/03/24 23:01:04 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: MainW.c,v 1.26 1998/02/22 01:00:42 rwscott Exp $";
+static char rcsid[] = "$Id: MainW.c,v 1.28 1998/03/24 23:01:04 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -691,15 +691,24 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
 	vals->cww = XtWidth(MW_CommandWindow(w));
 	vals->cwh = XtHeight(MW_CommandWindow(w));
 
+	DEBUGOUT(XdbDebug2(__FILE__, w, MW_CommandWindow(w),
+	    "_XmMainWindowPreferredSize: command current wid %d ht %d\n",
+	    vals->cww, vals->cwh));
 	if (MW_CommandWindow(w) == child)
 	{
 	    if (Wants(CWWidth))
 	    {
 		vals->cww = cg->width;
+
+		DEBUGOUT(XdbDebug0(__FILE__, w, "\tbut it wants width %d\n",
+			cg->width));
 	    }
 	    if (Wants(CWHeight))
 	    {
 		vals->cwh = cg->height;
+
+		DEBUGOUT(XdbDebug0(__FILE__, w, "\tbut it wants height %d\n",
+			cg->height));
 	    }
 	}
 
@@ -899,6 +908,12 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
 	vals->ShowHSB = SW_HSB(w) ? XtIsManaged(SW_HSB(w)) : False;
 	vals->ShowVSB = SW_VSB(w) ? XtIsManaged(SW_VSB(w)) : False;
 
+	/* Danny 18/3/1998 : if WWW/WWH == 0,0 then take WorkW,WorkH */
+	if (vals->www == 0 && vals->wwh == 0) {
+		vals->www = vals->WorkW;
+		vals->wwh = vals->WorkH;
+	}
+
 	/* If we're being resized (grown), make sure child follows */
 	if (vals->WorkW < vals->www)
 	{
@@ -1909,7 +1924,7 @@ XmMainWindowSetAreas(Widget widget,
 	_widgetBackground.pixel = XtBackground(widget);
  
 	XQueryColor(XtDisplay(widget),
-		    DefaultColormapOfScreen(XtScreen(widget)),
+		    CoreColormap(widget),
 		    &_widgetBackground);
  
 	troughColor.blue = _widgetBackground.blue * .80;
@@ -1917,11 +1932,10 @@ XmMainWindowSetAreas(Widget widget,
 	troughColor.red = _widgetBackground.red * .80;
  
 	if (!XAllocColor(XtDisplay(widget),
-			 DefaultColormapOfScreen(XtScreen(widget)),
+			 CoreColormap(widget),
 			 &troughColor))
 	{
-	    troughColor.pixel =
-		WhitePixelOfScreen(DefaultScreenOfDisplay(XtDisplay(widget)));
+	    troughColor.pixel = _XmWhitePixelOfObject(widget);
 	}
     }
 
