@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Manager.c,v 1.46 1998/03/12 20:26:25 helden Exp $
+ * $Id: Manager.c,v 1.49 1998/04/16 21:46:44 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,9 @@
  *
  **/
 
-static char rcsid[] = "$Id: Manager.c,v 1.46 1998/03/12 20:26:25 helden Exp $";
+static char rcsid[] = "$Id: Manager.c,v 1.49 1998/04/16 21:46:44 u27113 Exp $";
+
+#include <LTconfig.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,7 +32,6 @@ static char rcsid[] = "$Id: Manager.c,v 1.46 1998/03/12 20:26:25 helden Exp $";
 #include <strings.h>
 #endif
 
-#include <LTconfig.h>
 #include <XmI/XmI.h>
 #include <XmI/TraversalI.h>
 
@@ -42,6 +43,7 @@ static char rcsid[] = "$Id: Manager.c,v 1.46 1998/03/12 20:26:25 helden Exp $";
 #include <Xm/TransltnsP.h>
 #include <X11/keysym.h>
 #include <X11/Xfuncs.h>
+#include <Xm/VirtKeysP.h>
 
 #include <XmI/DebugUtil.h>
 
@@ -764,6 +766,9 @@ initialize(Widget request,
     CreateBottomShadowGC(new_w);
     CreateHighlightGC(new_w);
     CreateBackgroundGC(new_w);
+
+    /* Install a Virtual Keys magic event handler on all widgets */
+    XtAddEventHandler(new_w, KeyPressMask, False, _XmVirtKeysHandler, NULL);
 }
 
 static void
@@ -1874,16 +1879,19 @@ _XmManagerInstallMnemonic(Widget m, Widget w, KeySym mn)
 	m = mw;
     }
 
-    /* Put the data in the entry */
-    data.component = w;
-    data.eventType = KeyPress;
-    data.keysym = mn;
-    data.isMnemonic = True;
-    data.key = XKeysymToKeycode(XtDisplay(w), data.keysym);
-    data.modifiers = modifiers;
-    data.needGrab = need_grab;
+    if (XmIsManager(m))
+    {
+	/* Put the data in the entry */
+	data.component = w;
+	data.eventType = KeyPress;
+	data.keysym = mn;
+	data.isMnemonic = True;
+	data.key = XKeysymToKeycode(XtDisplay(w), data.keysym);
+	data.modifiers = modifiers;
+	data.needGrab = need_grab;
 
-    AddKeyboardEntry(m, w, &data);
+	AddKeyboardEntry(m, w, &data);
+    }
 }
 
 void
