@@ -1,6 +1,6 @@
 /**
  * 
- * $Id: MenuShell.c,v 1.65 1998/03/09 22:32:36 u27113 Exp $
+ * $Id: MenuShell.c,v 1.67 1998/04/22 22:00:58 u27113 Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: MenuShell.c,v 1.65 1998/03/09 22:32:36 u27113 Exp $";
+static char rcsid[] = "$Id: MenuShell.c,v 1.67 1998/04/22 22:00:58 u27113 Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -656,6 +656,23 @@ MenuShellPopdownDone(Widget w, XEvent *event,
 	return;
     }
 
+    /*
+     * Don't use an event that's within the multi click time.
+     * Hopefully this is an event which we can discard :-)
+     * If we get bug reports about things not disappearing when
+     * the user is fast, this is the reason.
+     * Danny 22/04/1998
+     *
+     * I also have a bad feeling about the test above. Is it meant to do
+     * what I changed it to below ?
+     */
+    if (_XmMenuGetInPMMode(w) && event && event->type == ButtonRelease &&
+	event->xbutton.time <= state->RC_ButtonEventStatus.time
+				+ XtGetMultiClickTime(XtDisplay(w)))
+    {
+	return;
+    }
+
     /* must be a menu shell here. */
     assert(XmIsMenuShell(w));
 
@@ -943,7 +960,7 @@ MenuShellPopupSharedMenuPane(Widget w, Widget w2, XEvent *event)
     assert(XmIsMenuShell(w));
 
     XRaiseWindow(XtDisplay(w2), XtWindow(w2));
-    _XmResizeObject(w, XtWidth(w2), XtHeight(w2), 0);
+    _XmResizeObject(w, XtWidth(w2) == 0 ? 1 : XtWidth(w2), XtHeight(w2) == 0 ? 1 : XtHeight(w2), 0);
  
     XtRealizeWidget(w);
     XMapRaised(XtDisplay(w), XtWindow(w));
