@@ -1,6 +1,6 @@
 /**
  *
- * $Id: MenuUtil.c,v 1.26 1998/03/09 14:48:47 rwscott Exp $
+ * $Id: MenuUtil.c,v 1.29 1998/05/01 15:07:02 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: MenuUtil.c,v 1.26 1998/03/09 14:48:47 rwscott Exp $";
+static const char rcsid[] = "$Id: MenuUtil.c,v 1.29 1998/05/01 15:07:02 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -484,7 +484,7 @@ _XmSetMenuTraversal(Widget wid, Boolean traversalOn)
 {
 }
 
-static Widget
+Widget
 _XmMenuNextItem(Widget menu, Widget current_item)
 {
     int current_item_index;
@@ -504,12 +504,15 @@ _XmMenuNextItem(Widget menu, Widget current_item)
 	int valid_control;
 
 	next_item_index = current_item_index;
-	for (valid_control = current_item_index + 1 < MGR_NumChildren(menu) ? current_item_index + 1 : 0; valid_control != current_item_index; valid_control = valid_control + 1 < MGR_NumChildren(menu) ? valid_control + 1 : 0)
+	if (MGR_NumChildren(menu) > 1)
 	{
-	    if (XmIsTraversable(MGR_Children(menu)[valid_control]))
+	    for (valid_control = current_item_index + 1 < MGR_NumChildren(menu) ? current_item_index + 1 : 0; valid_control != current_item_index; valid_control = valid_control + 1 < MGR_NumChildren(menu) ? valid_control + 1 : 0)
 	    {
-		next_item_index = valid_control;
-		break;
+		if (XmIsTraversable(MGR_Children(menu)[valid_control]))
+		{
+		    next_item_index = valid_control;
+		    break;
+		}
 	    }
 	}
     }
@@ -534,12 +537,15 @@ _XmMenuPrevItem(Widget menu, Widget current_item)
 	int valid_control;
 
 	next_item_index = current_item_index;
-	for (valid_control = current_item_index - 1 >= 0 ? current_item_index - 1 : MGR_NumChildren(menu) - 1; valid_control != current_item_index; valid_control = valid_control - 1 >= 0 ? valid_control - 1 : MGR_NumChildren(menu) - 1)
+	if (MGR_NumChildren(menu) > 1)
 	{
-	    if (XmIsTraversable(MGR_Children(menu)[valid_control]))
+	    for (valid_control = current_item_index - 1 >= 0 ? current_item_index - 1 : MGR_NumChildren(menu) - 1; valid_control != current_item_index; valid_control = valid_control - 1 >= 0 ? valid_control - 1 : MGR_NumChildren(menu) - 1)
 	    {
-		    next_item_index = valid_control;
-		    break;
+		if (XmIsTraversable(MGR_Children(menu)[valid_control]))
+		{
+			next_item_index = valid_control;
+			break;
+		}
 	    }
 	}
     }
@@ -777,3 +783,15 @@ _XmGetMenuState(Widget widget)
     return (XmMenuState)info->menu_state;
 }
 
+void
+_XmPopup(Widget shell, XtGrabKind grab_kind)
+{
+    XtPopup(shell, grab_kind);
+}
+
+void
+_XmPopdown(Widget shell)
+{
+    XtPopdown(shell);
+}
+
