@@ -1,6 +1,6 @@
 /**
  *
- * $Id: PanedW.c,v 1.32 1998/02/22 01:00:46 rwscott Exp $
+ * $Id: PanedW.c,v 1.36 1998/06/03 01:49:36 jon Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: PanedW.c,v 1.32 1998/02/22 01:00:46 rwscott Exp $";
+static const char rcsid[] = "$Id: PanedW.c,v 1.36 1998/06/03 01:49:36 jon Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -1323,8 +1323,8 @@ change_managed(Widget w)
 	    if (PWC_PaneMinimum(pane) != PWC_PaneMaximum(pane))
 	    {
 		XtManageChild(PWC_Sash(pane));
-	    }
 
+	    }
 	    if (PW_SeparatorOn(w))
 	    {
 		XtManageChild(PWC_Separator(pane));
@@ -1332,7 +1332,19 @@ change_managed(Widget w)
 
 	    PW_RecursivelyCalled(w) = False;
 	}
-	else if (i == (PW_PaneCount(w) - 1) && XtIsManaged(PWC_Sash(pane)))
+	else if (i == (PW_PaneCount(w) - 1) && !XtIsManaged(PWC_Sash(pane)))
+	{
+
+	    PW_RecursivelyCalled(w) = True;
+
+	    XtUnmanageChild(PWC_Sash(PW_ManagedChildren(w)[i]));
+	    XtUnmanageChild(PWC_Separator(PW_ManagedChildren(w)[i]));
+
+	    PW_RecursivelyCalled(w) = False;
+	}
+	if (i == (PW_PaneCount(w) - 1) &&
+	    (XtIsManaged(PWC_Sash(PW_ManagedChildren(w)[i])) ||
+	    (XtIsManaged(PWC_Separator(PW_ManagedChildren(w)[i])))))
 	{
 
 	    PW_RecursivelyCalled(w) = True;
