@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Primitive.c,v 1.27 1997/11/22 18:37:12 miers Exp $
+ * $Id: Primitive.c,v 1.28 1998/04/16 21:46:42 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Primitive.c,v 1.27 1997/11/22 18:37:12 miers Exp $";
+static char rcsid[] = "$Id: Primitive.c,v 1.28 1998/04/16 21:46:42 u27113 Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -33,6 +33,7 @@ static char rcsid[] = "$Id: Primitive.c,v 1.27 1997/11/22 18:37:12 miers Exp $";
 #include <Xm/RepType.h>
 #include <Xm/ManagerP.h>
 #include <Xm/TransltnsP.h>
+#include <Xm/VirtKeysP.h>
 
 #include <XmI/DebugUtil.h>
 
@@ -603,6 +604,9 @@ initialize(Widget request, Widget new_w,
     CreateHighlightGC(pw);
     CreateBottomShadowGC(pw);
     CreateTopShadowGC(pw);
+
+    /* Install a Virtual Keys magic event handler on all widgets */
+    XtAddEventHandler(new_w, KeyPressMask, False, _XmVirtKeysHandler, NULL);
 }
 
 static void
