@@ -1,6 +1,6 @@
 /**
  *
- * $Id: PushB.c,v 1.34 1998/02/28 18:42:00 rwscott Exp $
+ * $Id: PushB.c,v 1.35 1998/03/16 21:52:13 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: PushB.c,v 1.34 1998/02/28 18:42:00 rwscott Exp $";
+static const char rcsid[] = "$Id: PushB.c,v 1.35 1998/03/16 21:52:13 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -36,6 +36,12 @@ static char rcsid[] = "$Id: PushB.c,v 1.34 1998/02/28 18:42:00 rwscott Exp $";
 #include <Xm/CascadeB.h>
 #include <Xm/CascadeBG.h>
 
+#ifndef __STDC__
+#include <varargs.h>
+#else
+#include <stdarg.h>
+#endif
+
 #include <XmI/DebugUtil.h>
 
 /*
@@ -1438,29 +1444,21 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
 #else
-MenuProcEntry(function, widget, va_alist)
-     int function;
-     Widget widget;
+MenuProcEntry(proc, w, va_alist)
+     int proc;
+     Widget w;
      va_dcl
 {
     va_list arglist;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
-    va_start(arg_list);
+    va_start(arglist);
 #endif
 
     switch (proc)
