@@ -1,6 +1,6 @@
 /**
  *
- * $Id: PushBG.c,v 1.33 1998/03/09 04:26:47 rwscott Exp $
+ * $Id: PushBG.c,v 1.37 1998/05/03 17:03:36 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: PushBG.c,v 1.33 1998/03/09 04:26:47 rwscott Exp $";
+static const char rcsid[] = "$Id: PushBG.c,v 1.37 1998/05/03 17:03:36 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -39,6 +39,12 @@ static char rcsid[] = "$Id: PushBG.c,v 1.33 1998/03/09 04:26:47 rwscott Exp $";
 #include <X11/Xfuncs.h>
 #include <X11/ShellP.h>
 
+#ifndef __STDC__
+#include <varargs.h>
+#else
+#include <stdarg.h>
+#endif
+
 #include <XmI/DebugUtil.h>
 
 /*
@@ -1257,7 +1263,13 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmPushButtonCallbackStruct cbs;
     XtExposeProc exp = XtClass(w)->core_class.expose;
 
+    /* rws 17 Mar 1998
+       When the ArmAndActivate action is called focus does not go to the
+       button that was armed.  This means that either ArmAndActivate does
+       not call Arm or this should not be here.  Same goes with the
+       ActiveChild bit at the end of Arm.
     XmProcessTraversal(w, XmTRAVERSE_CURRENT);
+    */
 
     PBG_Armed(w) = True;
 
@@ -1275,7 +1287,9 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 			   PBG_ArmCallback(w),
 			   (XtPointer)&cbs);
     }
+    /*
     MGR_ActiveChild(XtParent(w)) = w;
+    */
 }
 
 static void
@@ -1356,17 +1370,16 @@ ArmTimeout(XtPointer data, XtIntervalId *id)
     XmPushButtonCallbackStruct cbs;
     XmArmTimeoutRec *p = (XmArmTimeoutRec *)data;
     Widget w = p->w;
-    XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
  
     if (PBG_Armed(w))
     {
- 
-	PBG_Armed(w) = False;
-
-	(*exp) (w, NULL, NULL);
-     
+	/* rws 18 Mar 1998
+	   This must be done before the ACTIVATE callback
+	   see comments near the end of ArmAndActivate
+	 */
+	Disarm(w, NULL, NULL, 0);
 	if (!LabG_SkipCallback(w) && PBG_ActivateCallback(w))
 	{
 	    cbs.reason = XmCR_ACTIVATE;
@@ -1387,8 +1400,6 @@ ArmTimeout(XtPointer data, XtIntervalId *id)
 			       PBG_ActivateCallback(w),
 			       (XtPointer)&cbs);
 	}
-
-	Disarm(w, NULL, NULL, 0);
     }
 
     XtFree(data);
@@ -1425,7 +1436,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	    XtRemoveTimeOut(PBG_Timer(w));
 	}
 
-#if 0
+#if 1
 	/* rws 8 Mar 1998
 	   This is wacky. In xdir select Connect->Connect to Anonymous.
 	   Fill in User Name: with anonymous, Password: with you email address
@@ -1803,29 +1814,21 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
 #else
-MenuProcEntry(function, widget, va_alist)
-     int function;
-     Widget widget;
+MenuProcEntry(proc, w, va_alist)
+     int proc;
+     Widget w;
      va_dcl
 {
     va_list arglist;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
-    va_start(arg_list);
+    va_start(arglist);
 #endif
 
     switch (proc)
