@@ -1,6 +1,6 @@
 /**
  *
- * $Id: RCUtils.c,v 1.72 1998/01/22 23:14:10 u27113 Exp $
+ * $Id: RCUtils.c,v 1.74 1998/05/01 22:51:03 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: RCUtils.c,v 1.72 1998/01/22 23:14:10 u27113 Exp $";
+static char rcsid[] = "$Id: RCUtils.c,v 1.74 1998/05/01 22:51:03 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -262,12 +262,17 @@ DoLayoutHT(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
     Dimension current_x = RC_MarginW(rc) + MGR_ShadowThickness(rc);
     Dimension current_y = RC_MarginH(rc) + MGR_ShadowThickness(rc);
     XmRCKidGeometry help_kid_geometry = NULL;
+    Dimension help_kid_width = 0;
 
     DEBUGOUT(XdbDebug2(__FILE__, rc, instig,
 		"DoLayoutHT: req %s, max w %d h %d\n",
 		XdbWidgetGeometry2String(instig_request),
 		max_width, max_height));
 
+    if (RC_HelpPb(rc))
+    {
+	help_kid_width = XtWidth(RC_HelpPb(rc));
+    }
     /* Start layout */
     for (i = 0; i < MGR_NumChildren(rc); i++)
     {
@@ -325,6 +330,54 @@ DoLayoutHT(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 
 	current_x += RC_Spacing(rc) + kid_geometry->box.width;
     }
+    if (RC_HelpPb(rc) && XtIsManaged(RC_HelpPb(rc)) && help_kid_geometry)
+    {
+    	/* treat the help button exactly the same as all of the other
+    	   kids. (ie clone the contents of the above loop)
+    	 */
+	XmRCKidGeometry kid_geometry = help_kid_geometry;
+	Widget kid = kid_geometry->kid;
+
+	/* Height treatment */
+	kid_geometry->box.height = max_height;
+
+	/*
+	 * check for row adjustment
+	 */
+	if (current_x + kid_geometry->box.width > XtWidth(rc) &&
+	    current_x > RC_MarginW(rc) + MGR_ShadowThickness(rc))
+	{
+	    /* compute the local max_height */
+	    lmh = 0;
+	    for (j = first_in_row; j < i; j++)
+	    {
+		if (XtIsManaged(RC_Boxes(rc)[j].kid) &&
+		    RC_Boxes(rc)[j].box.height > lmh)
+		    lmh = RC_Boxes(rc)[j].box.height;
+	    }
+	    /* set the local max_height */
+	    for (j = first_in_row; j < i; j++)
+	    {
+		RC_Boxes(rc)[j].box.height = lmh;
+	    }
+
+	    current_y += lmh;
+	    current_x = RC_MarginW(rc) + MGR_ShadowThickness(rc);
+	    first_in_row = i;
+	}
+
+	kid_geometry->box.x = (XtWidth(rc)
+				    - MGR_ShadowThickness(rc)
+				    - help_kid_geometry->box.width);
+	kid_geometry->box.y = current_y;
+
+	if (kid == instig && instig_request != NULL)
+	{
+	    *instig_request = kid_geometry->box;
+	}
+
+	current_x += RC_Spacing(rc) + kid_geometry->box.width;
+    }
 
     /* compute the local max_height */
     lmh = 0;
@@ -375,6 +428,14 @@ DoLayoutHT(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 	}
     }
 
+#if 0
+    /* rws 1 May 1998
+       The help widget is treated just like all of the other kids
+       immediately following the loop treating all of the other kids.
+       Otherwise if we needed to wrap just to fit the help widget all of
+       the buttons would be too high and the help widget will be tucked
+       behind the last widget.
+     */
     /* In some cases we don't have a help widget here ...
      * like when it's not managed
      */
@@ -385,6 +446,7 @@ DoLayoutHT(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 				    - help_kid_geometry->box.width);
 	help_kid_geometry->box.y = current_y;
     }
+#endif
 }
 
 /* 
@@ -1335,7 +1397,7 @@ _XmRCAdjustSize(Widget rc, Widget instig, XtWidgetGeometry *instig_request)
 	max_height = 1;
     }
 
-    if (!RC_FromResize(rc))
+    if (XtIsRealized(rc) || !RC_FromResize(rc))
     {
 	DEBUGOUT(XdbDebug(__FILE__, rc,
 			  "_XmRCAdjustSize: requesting %s from parent %s\n",
