@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ResConvert.c,v 1.20 1998/03/14 15:00:06 u27113 Exp $
+ * $Id: ResConvert.c,v 1.25 1998/04/23 22:27:32 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,9 @@
  *
  **/
 
-static char rcsid[] = "$Id: ResConvert.c,v 1.20 1998/03/14 15:00:06 u27113 Exp $";
+static char rcsid[] = "$Id: ResConvert.c,v 1.25 1998/04/23 22:27:32 u27113 Exp $";
+
+#include <LTconfig.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -33,7 +35,10 @@ static char rcsid[] = "$Id: ResConvert.c,v 1.20 1998/03/14 15:00:06 u27113 Exp $
 #endif
 #include <ctype.h>
 
-#include <LTconfig.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
 #include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/RepType.h>
@@ -65,6 +70,8 @@ Boolean _XmCvtStringToWidget(Display *, XrmValue *, Cardinal *,
 			XrmValue *, XrmValue *, XtPointer *);
 Boolean _XmCvtStringToCardinal(Display *, XrmValue *, Cardinal *,
 			XrmValue *, XrmValue *, XtPointer *);
+Boolean _XmCvtStringToBooleanDimension(Display *, XrmValue *, Cardinal *,
+			XrmValue *, XrmValue *, XtPointer *);
 
 /* please be careful if you're editting these arrays, as
  * the strings have to be in the _same_ order as the
@@ -709,6 +716,13 @@ _XmRegisterConverters()
 		       XtCacheAll,	/* caching instructions */
 		       NULL);	/* destructor function */
 
+    XtSetTypeConverter(XtRString,	/* source type */
+		       XmRBooleanDimension,	/* target type */
+		       _XmCvtStringToBooleanDimension,	/* converter routine */
+		       NULL,	/* args for converter routine */
+		       0,	/* number of args to converter routine */
+		       XtCacheNone,	/* caching instructions */
+		       NULL);	/* destructor function */
 
 #ifdef NONSTANDARD_CONVERTERS
     _XmRegisterNSEConverters();
@@ -910,8 +924,69 @@ XmCvtXmStringToText(Display *display, XrmValuePtr args,
 		    Cardinal *num_args, XrmValue *from_val, XrmValue *to_val,
 		    XtPointer *converter_data)
 {
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "XmCvtXmStringToText\n"));
-    return False;
+    char *value = strdup("");
+    XmString m = (XmString)from_val->addr;
+    XmStringContext c;
+    XmStringComponentType t = XmSTRING_COMPONENT_UNKNOWN;
+
+    if (*num_args != 0)
+    {
+	XtWarningMsg("wrongParameters", "XmCvtXmStringToText",
+		     "XtToolkitError",
+		     "XmString to String conversion needs no extra arguments",
+		     (String *)NULL, (Cardinal *)NULL);
+    }
+
+    XmStringInitContext(&c, m);
+
+    while (t != XmSTRING_COMPONENT_END)
+    {
+	char *s_text            = NULL;
+	XmStringCharSet s_cs    = NULL;
+	XmStringDirection d     = XmSTRING_DIRECTION_DEFAULT;
+	XmStringComponentType u = XmSTRING_COMPONENT_UNKNOWN;
+	unsigned short ul       = 0;
+	unsigned char *s_uv     = NULL;
+	char *text              = NULL;
+	
+	t = XmStringGetNextComponent(c, &s_text, &s_cs, &d, &u, &ul, &s_uv);
+
+	switch (t)
+	{
+	case XmSTRING_COMPONENT_TEXT:
+	case XmSTRING_COMPONENT_LOCALE_TEXT:
+	    text = s_text;
+	    break;
+
+	case XmSTRING_COMPONENT_SEPARATOR:
+	    text = "\n";
+	    break;
+
+	default:
+	    /* Unknown component */
+	    text = NULL;
+	    break;
+	}
+
+	if (text != NULL)
+	{
+	    value = XtRealloc(value, strlen(value) + strlen(text) + 1);
+	    strcat(value, text);
+	}
+
+	XtFree(s_text);
+	XtFree(s_cs);
+	XtFree((char *)s_uv);
+    }
+
+    to_val->addr = value;
+    to_val->size = strlen(value);  /* FIXME: should this be sizeof(value)? */
+
+    DEBUGOUT(XdbDebug(__FILE__, NULL,
+		      "XmCvtXmStringToText(%p) => %s\n",
+		      from_val->addr, to_val->addr));
+
+    return True;
 }
 
 /*
@@ -1688,7 +1763,11 @@ _XmCvtStringToCardinal(Display *display,
 		    (String *)NULL, (Cardinal *)NULL);
 	}
 
+#ifdef	HAVE_STRTOUL
 	value = strtoul((char *)from->addr, &end, 10);
+#else
+	value = (Cardinal)strtol((char *)from->addr, &end, 10);
+#endif
 
 	if (*end == '\0')
 	{
@@ -2084,3 +2163,93 @@ _XmCvtStringToWidget(	Display *display,
 
     return True;
 }
+
+Boolean                 
+_XmCvtStringToBooleanDimension( Display *display,
+				XrmValue *args,                  
+				Cardinal *num_args, 
+				XrmValue *from,
+				XrmValue *to, 
+				XtPointer *converter_data)
+{                       
+    static Dimension	value;
+    String		str;
+    int			error=0;
+
+    str = (String)from->addr;
+
+    if (*num_args != 0) {
+	XtWarningMsg("wrongParameters", "cvtStringToWidget",
+	    "XtToolkitError",
+	    "String to BooleanDimension conversion needs no extra arguments",
+	    (String *)NULL, (Cardinal *)NULL);
+    }
+
+    if (str != NULL) {
+	if (isdigit(str[0])) {
+	    value = atoi(str);
+	} else {
+	    if ((strcasecmp(str, "true") == 0) || (strcasecmp(str, "yes") == 0)
+			|| (strcasecmp(str, "on") == 0)) {
+		value = True;
+	    } else if ((strcasecmp(str, "false") == 0)
+			|| (strcasecmp(str, "no") == 0)
+			|| (strcasecmp(str, "off") == 0)) {
+		value = False;
+	    } else {
+		XtDisplayStringConversionWarning(display, str,
+				XmRBooleanDimension);
+		return False;
+	    }
+	}
+
+	if (to->addr == NULL) {
+	    to->addr = (XtPointer)&value;
+	    to->size = sizeof(value);
+	} else {
+	    if (to->size >= sizeof(value)) {
+		*((Dimension *)to->addr) = value;
+		to->size = sizeof(Dimension);
+	    } else {
+		XtDisplayStringConversionWarning(display, str,
+				XmRBooleanDimension);
+		return False;
+	    }
+	}
+    } else {
+	XtDisplayStringConversionWarning(display, str, XmRBooleanDimension);
+	return False;
+    }
+
+    return True;
+}
+
+/*
+ * According to Alexander Lukyanov <lav@yars.free.net>, we still lack
+ * a "couple of" converters :
+ * _XmCvtStringToAtomList
+ * _XmCvtStringToCardinal
+ * _XmCvtStringToChar
+ * _XmCvtStringToCharSetTable
+ * _XmCvtStringToCharSetTableDestroy
+ * _XmCvtStringToGadgetPixmap
+ * _XmCvtStringToHorizontalDimension
+ * _XmCvtStringToKeySymTable
+ * _XmCvtStringToKeySymTableDestroy
+ * _XmCvtStringToManBottomShadowPixmap
+ * _XmCvtStringToManForegroundPixmap
+ * _XmCvtStringToManHighlightPixmap
+ * _XmCvtStringToManTopShadowPixmap
+ * _XmCvtStringToPrimBottomShadowPixmap
+ * _XmCvtStringToPrimForegroundPixmap
+ * _XmCvtStringToPrimHighlightPixmap
+ * _XmCvtStringToPrimTopShadowPixmap
+ * _XmCvtStringToTextPosition
+ * _XmCvtStringToTopItemPosition
+ * _XmCvtStringToVerticalDimension
+ * _XmCvtStringToWindow
+ * _XmCvtStringToXmFontList
+ * _XmCvtStringToXmFontListDestroy
+ * _XmCvtStringToXmStringDestroy
+ * _XmCvtStringToXmStringTable
+ */
