@@ -1,6 +1,6 @@
 /**
  *
- * $Id: RowColumn.c,v 1.138 1998/03/13 22:25:33 rwscott Exp $
+ * $Id: RowColumn.c,v 1.148 1998/05/25 13:31:49 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: RowColumn.c,v 1.138 1998/03/13 22:25:33 rwscott Exp $";
+static const char rcsid[] = "$Id: RowColumn.c,v 1.148 1998/05/25 13:31:49 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -63,6 +63,12 @@ static char rcsid[] = "$Id: RowColumn.c,v 1.138 1998/03/13 22:25:33 rwscott Exp
 
 #include <XmI/DebugUtil.h>
 
+/* rws 5 May 1998
+   TEST29 was a hack to get a back pointer from the pulldown in an option
+   menu back to the option menu.  RC_LastSelectToplevel() should now contain
+   this pointer for the pulldown. all of the ifdef TEST29 stuff should get
+   removed.
+ */
 #define TEST29
 #define ALWAYS_INSTALL_ENTRY_CALLBACK
 
@@ -911,23 +917,34 @@ initialize(Widget request, Widget new_w,
 	}
     }
 
-    RC_LastSelectToplevel(new_w) = NULL;
+    RC_LastSelectToplevel(new_w) = NULL;  /* widget id of the menu at the
+    					     top of the heir
+    					   */
 
     if (RC_Type(new_w) == XmMENU_BAR || RC_Type(new_w) == XmMENU_OPTION)
     {
 	RC_LastSelectToplevel(new_w) = new_w;
+	if (RC_OptionSubMenu(new_w))
+	{
+	    /* Make sure the sub-menu has a back pointer to the option menu */
+	    RC_LastSelectToplevel(RC_OptionSubMenu(new_w)) = new_w;
+	}
     }
 
     /* Internal widgets */
 
-    RC_CascadeBtn(new_w) = NULL;
-    RC_PopupPosted(new_w) = NULL;
+    RC_CascadeBtn(new_w) = NULL;	/* widget id of the cascade button
+    					   that posted this menu
+    					 */
+    RC_PopupPosted(new_w) = NULL;	/* widget id of the menu posted from
+    					   any button in this menu
+    					 */
 
     /* Tear off stuff */
-    RC_SetTornOff(new_w, 0);
-    RC_SetFromInit(new_w, 0);
-    RC_SetTearOffDirty(new_w, 0);
-    RC_SetTearOffActive(new_w, 0);
+    RC_SetTornOff(new_w, False);
+    RC_SetFromInit(new_w, False);
+    RC_SetTearOffDirty(new_w, False);
+    RC_SetTearOffActive(new_w, False);
     RC_ParentShell(new_w) = NULL;
     RC_TearOffControl(new_w) = NULL;
     if ((RC_Type(new_w) == XmMENU_POPUP || RC_Type(new_w) == XmMENU_PULLDOWN) &&
@@ -937,7 +954,7 @@ initialize(Widget request, Widget new_w,
     }
 
     /* just to initialize it. */
-    RC_SetFromResize(new_w, 0);
+    RC_SetFromResize(new_w, False);
     RC_SetArmed(new_w, False);
 
     /* menuPost stuff. */
@@ -1138,6 +1155,8 @@ set_values(Widget old, Widget request, Widget new_w,
     {
 	int i;
 
+	/* Make sure the sub-menu has a back pointer to the option menu */
+	RC_LastSelectToplevel(RC_OptionSubMenu(new_w)) = new_w;
 	/* Oh yes this is very efficient :-( */
 	for (i = 0; i < MGR_NumChildren(new_w); i++)
 	{
@@ -1184,6 +1203,15 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 #define superclass (&xmManagerClassRec)
     (*superclass->core_class.realize) (w, value_mask, attributes);
 #undef superclass
+    if (XmIsMenuShell(XtParent(w)) && MS_PrivateShell(XtParent(w)))
+    {
+	/* if this is in a private menu shell it was managed during
+	   insert_child.  However if someone (Mozilla's go menu)
+	   un-realizes the menu and then realizes it again we must
+	   make sure it is managed.
+	 */
+	XtManageChild(w);
+    }
 }
 
 static void
@@ -1441,12 +1469,70 @@ geometry_manager(Widget w,
 static void
 change_managed(Widget w)
 {
+Widget tlm;
+
     DEBUGOUT(XdbDebug(__FILE__, w, "change_managed()\n"));
 
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmRCAdjustSize from change_managed\n"));
 
     _XmRCAdjustSize(w, NULL, NULL);
 
+    /* rws 28 Apr 1998
+       If widget w is a pulldown menu that is associated to an option menu
+       we have a problem here.  The option menu XmNmenuHistory could be
+       pointing to a widget that has just been un-managed.  This becomes
+       a problem if the un-manage is the first step of a destroy.  If we 
+       do not find the associated option menu and fix up its XmNmenuHistory
+       it will be left pointing to a widget that has been destroyed.  What
+       we need is some sort of pointer from the pulldown menu to the 
+       option menu.  RC_CascadeBtn() could be used but, currently it is
+       only valid when the pulldown is visible.  The same can be said
+       for RC_LastSelectToplevel().
+
+       RC_CascadeBtn() is set to NULL in MenuShell:MenuShellPopdownOne()
+       RC_LastSelectToplevel() will be set correctly whenever the menu
+	   gets popped up. If we ensure that it is set correctly whenever
+	   the subMenuId of an option menu changes, that should do it!
+     */
+    {
+    	tlm = RC_LastSelectToplevel(w);
+    	if (tlm && RC_Type(tlm) == XmMENU_OPTION)
+    	{
+    	Widget mem;
+
+    		mem = RC_MemWidget(tlm);
+    		/* rws 25 May 1998
+    		   I do not really like this XtIsSensitive stuff.  We should
+    		   be able to pick a new mem widget even if the menu is not
+    		   sensitive.  This is needed for xmcd 2.3 otherwise it
+    		   screws up on startup because it creates an in-sensitve
+    		   option menu.
+    		 */
+    		if (XtIsSensitive(tlm) && mem && !XtIsManaged(mem))
+    		{
+    		Widget new_mem;
+
+    			/* need to pick a new memWidget */
+    			new_mem = _XmMenuNextItem(w, MGR_Children(tlm)[MGR_NumChildren(tlm) - 1]);
+    			if (new_mem != mem)
+    			{
+			    _XmOptionCallback(new_mem, tlm, NULL);
+    			}
+    			else
+    			{
+			    _XmOptionCallback(NULL, tlm, NULL);
+    			}
+    		}
+    	}
+    }
+    if (tlm && RC_Type(tlm) == XmMENU_OPTION)
+    {
+	/* rws 30 Apr 1998
+	   we need to force the option menu (tlm) to re-calculate its
+	   layout based on the new size of its submenu (w)
+	 */
+    }
+
     _XmNavigChangeManaged(w);
 }
 
@@ -1919,8 +2005,12 @@ MenuProcEntry(function, widget, va_alist)
 	    if (RC_Type(XtParent(w)) == XmMENU_OPTION &&
 		submenu != NULL && XmIsRowColumn(submenu))
 	    {
-		RC_CascadeBtn(submenu) = w;
+		RC_CascadeBtn(submenu) = button;
 		RC_OptionSubMenu(XtParent(w)) = submenu;
+		/* rws 5 May 1998
+		   should be setting RC_LastSelectTopLevel(submenu) = XtParent(button);
+		   I think.
+		 */
 
 		if (XmIsGadget(w))
 		{
@@ -2018,6 +2108,9 @@ MenuProcEntry(function, widget, va_alist)
 	break;
 
     case XmMENU_MEMWIDGET_UPDATE:
+	/* rws 5 May 1998
+	   this could call _XmFixOptionMenu
+	 */
 	break;
 
     case XmMENU_POPDOWN:
@@ -2029,6 +2122,7 @@ MenuProcEntry(function, widget, va_alist)
 	break;
 
     case XmMENU_RESTORE_TEAROFF_TO_TOPLEVEL_SHELL:
+	/* fix up RC_LastSelectTopLevel and RC_CascadeBtn */
 	event = va_arg(arg_list, XEvent *);
 	_XmRestoreTearOffToToplevelShell(w, event);
 	break;
@@ -2038,6 +2132,7 @@ MenuProcEntry(function, widget, va_alist)
 	was_torn = va_arg(arg_list, Boolean *);
 	event = va_arg(arg_list, XEvent *);
 
+	/* fix up RC_LastSelectTopLevel and RC_CascadeBtn */
 	if (RC_TearOffModel(w) == XmTEAR_OFF_ENABLED)
 	{
 	    DEBUGOUT(XdbDebug(__FILE__, w,
@@ -2084,7 +2179,9 @@ _XmFixOptionMenu(Widget new_w, Boolean use_set_values)
      */
 
     XmString Label;
+    /*
     int j;
+    */
 
     DEBUGOUT(XdbDebug(__FILE__, new_w, "_XmFixOptionMenu\n"));
 
@@ -2114,8 +2211,11 @@ _XmFixOptionMenu(Widget new_w, Boolean use_set_values)
 
 		if (XmIsTearOffButton(MGR_Children(RC_OptionSubMenu(new_w))[0]))
 		{
-		    RC_MemWidget(new_w) =
-			MGR_Children(RC_OptionSubMenu(new_w))[1];
+		    if (MGR_NumChildren(RC_OptionSubMenu(new_w)) > 2)
+		    {
+			RC_MemWidget(new_w) =
+			    MGR_Children(RC_OptionSubMenu(new_w))[1];
+		    }
 		}
 		else
 		{
@@ -2191,13 +2291,17 @@ _XmOptionCallback(Widget w,
     {
 	XtVaGetValues(w, XmNlabelString, &ButtonLabel,
 		      NULL);
-	XtVaSetValues(XmOptionButtonGadget(rc), XmNlabelString, ButtonLabel,
-		      NULL);
-
-	RC_MemWidget(rc) = w;
-
-	XmStringFree(ButtonLabel);
     }
+    else
+    {
+    	ButtonLabel = XmStringCreateSimple("");
+    }
+    XtVaSetValues(XmOptionButtonGadget(rc), XmNlabelString, ButtonLabel,
+		  NULL);
+
+    RC_MemWidget(rc) = w;
+
+    XmStringFree(ButtonLabel);
 }
 
 
@@ -2742,7 +2846,7 @@ DoBtnEventCleanupReplay(Widget w, XEvent *event,
 
     DEBUGOUT(XdbDebug(__FILE__, w, "DoBtnEventCleanupReplay\n"));
 
-    _XmGetActiveTopLevelMenu(w, &tlrc);
+    _XmGetActiveTopLevelMenu(w, &tlrc);  /* this is RC_LastSelectToplevel() no?? */
 
     if (!tlrc)
     {
@@ -2762,6 +2866,9 @@ DoBtnEventCleanupReplay(Widget w, XEvent *event,
     {
     Boolean poppedUp;
 
+	/* should use RC_CascadeBtn(RC_PopupPosted(tlrc)) I think.
+	   MGR_ActiveChild is really only for the active Gadget.
+	 */
 	RC_MenuShellPopdown(MGR_ActiveChild(tlrc), event, &poppedUp);
     }
 
@@ -2860,7 +2967,7 @@ _XmMenuBtnUp(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
 
     /*
-     * If event was in a child gadget, then arm it, otherwise determine
+     * If event was in a child gadget, then activate it, otherwise determine
      * whether the event occured outside the menu system, and do a cleanup
      * if it did.
      */
@@ -2959,6 +3066,10 @@ MenuEnter(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	    {
 		_XmMenuDisarmItem(MGR_ActiveChild(w));
 	    }
+            /* This PrivateShell stuff should be in MenuShell somewhere.
+               RC should just be able to bring the menu down and let MenuShell
+               worry about the details of how to do this.
+             */
             if (shell && XmIsMenuShell(shell) && MS_PrivateShell(shell))
             {
                 XtCallActionProc(shell, "XtMenuPopdown",
@@ -3053,6 +3164,13 @@ _XmPostPopupMenu(Widget wid, XEvent *event)
     if (type == 0)
     {
 	type = KeyPress;
+	/* rws 2 Apr 1998
+	   This makes the popup's in ddd and Mozilla work but, the real
+	   problem is the GrabButton is not getting called for some reason.
+	   The grab is placed on the RC in AddPopupHandlers but it never
+	   gets triggered?????
+	 */
+	type = ButtonPress;
     }
 
     state->RC_ButtonEventStatus.waiting_to_be_managed = False;
@@ -3080,6 +3198,11 @@ _XmPostPopupMenu(Widget wid, XEvent *event)
 	RCClass_MenuProcs(XtClass(wid))(XmMENU_ARM, wid, NULL);
     }
 
+    /* rws 7 May 1998
+       Mozilla's Bookmark, Forward, Back button menus do not post without
+       this XtManageChild().
+     */
+    XtManageChild(wid);
     XtCallActionProc(XtParent(wid), "XtMenuPopup", event, NULL, 0);
 
     RC_SetArmed(wid, True);
@@ -3308,6 +3431,7 @@ _XmCallRowColumnUnmapCallback(Widget wid, XEvent *event)
 void
 _XmMenuPopDown(Widget w, XEvent *event, Boolean *popped_up)
 {
+    _XmWarning(w, "%s(%d) - _XmMenuPopDown() not implemented ", __FILE__, __LINE__);
 }
 
 Boolean
