@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Scale.c,v 1.23 1998/02/22 01:00:50 rwscott Exp $
+ * $Id: Scale.c,v 1.25 1998/04/22 19:47:18 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Scale.c,v 1.23 1998/02/22 01:00:50 rwscott Exp $";
+static char rcsid[] = "$Id: Scale.c,v 1.25 1998/04/22 19:47:18 u27113 Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -228,6 +228,13 @@ static XtResource resources[] =
 	sizeof(XtCallbackList), Offset(drag_callback),
 	XmRCallback, (XtPointer)NULL
     },
+#if XmVERSION > 1
+    {
+	XmNeditable, XmCEditable, XmRBoolean,
+	sizeof(int), Offset(editable),
+	XmRImmediate, (XtPointer)True
+    },
+#endif
     {
 	XmNscaleMultiple, XmCScaleMultiple, XmRInt,
 	sizeof(int), Offset(scale_multiple),
@@ -900,6 +907,7 @@ static void
 realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 {
     Dimension wd, ht;
+    Dimension pwd, pht;
     XmKidGeometry boxes;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "XmScale realize()\n"));
@@ -910,10 +918,12 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 
     wd = ht = 0;
     boxes = _XmScalePreferredSize(w, NULL, NULL, &wd, &ht);
+    pwd = wd;
+    pht = ht;
 
     _XmScaleGeomRequest(w, &wd, &ht);
 
-    _XmScaleLayout(w, boxes, NULL, NULL, wd, ht);
+    _XmScaleLayout(w, boxes, NULL, NULL, pwd, pht);
 
     _XmScaleConfigureChildren(w, NULL, boxes);
 
@@ -1693,7 +1703,7 @@ _XmScaleLayout(Widget scale, XmKidGeometry boxes,
 
 	if (XtWidth(scale) > curw)
 	{
-	    for (i = 0; i < 2 && boxes[i].kid; i++)
+	    for (i = 0; boxes[i].kid; i++)
 	    {
 		boxes[i].box.x = XtWidth(scale) - (curw - boxes[i].box.x);
 	    }
@@ -1874,6 +1884,11 @@ _ScaleDrag(Widget sb, XtPointer cd, XtPointer data)
     XmScaleCallbackStruct scbs;
     int scale_value;
 
+#if XmVERSION > 1
+    if (! Scale_Editable(sb))
+	return;
+#endif
+
     Scale_LastValue(XtParent(sb)) = Scale_Value(sb);
     scale_value = _XmScaleConvertSCBValueToScaleValue(XtParent(sb), cbs->value);
     Scale_Value(XtParent(sb)) = scale_value;
