@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ScrollBar.c,v 1.21 1998/03/10 20:20:23 helden Exp $
+ * $Id: ScrollBar.c,v 1.23 1998/04/10 19:10:44 jonf Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: ScrollBar.c,v 1.21 1998/03/10 20:20:23 helden Exp $";
+static char rcsid[] = "$Id: ScrollBar.c,v 1.23 1998/04/10 19:10:44 jonf Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -345,7 +345,9 @@ static XtResource resources[] =
     {
 	XmNhighlightThickness, XmCHighlightThickness, XmRHorizontalDimension,
 	sizeof(Dimension), Prim_Offset(highlight_thickness),
-	XmRCallProc, (XtPointer)_XmPrimitiveHighlightThicknessDefault
+	/*XmRCallProc, (XtPointer)_XmPrimitiveHighlightThicknessDefault*/
+	/* My M*tif manual says that this should be 0 by default */
+	XmRImmediate, (XtPointer)0
     }
 };
 
@@ -595,8 +597,8 @@ CreateUnavailableGC(Widget w)
     values.ts_x_origin = values.ts_y_origin = 0;
     values.stipple = XmGetPixmapByDepth(XtScreen(w),
 					XmEVEN_STIPPLE_IMAGE,
-					WhitePixelOfScreen(XtScreen(w)),
-					BlackPixelOfScreen(XtScreen(w)),
+					_XmWhitePixelOfObject(w),
+					_XmBlackPixelOfObject(w),
 					1);
 
     SCB_UnavailableGC(w) = XtGetGC(w, mask, &values);
@@ -3137,7 +3139,7 @@ _XmScrollBarTroughColorDefault(Widget w,
     _widgetBackground.pixel = XtBackground(w);
 
     XQueryColor(XtDisplay(w),
-		DefaultColormapOfScreen(XtScreen(w)),
+		CoreColormap(w),
 		&_widgetBackground);
 
     troughColor.blue = _widgetBackground.blue * .80;
@@ -3145,11 +3147,10 @@ _XmScrollBarTroughColorDefault(Widget w,
     troughColor.red = _widgetBackground.red * .80;
 
     if (!XAllocColor(XtDisplay(w),
-		     DefaultColormapOfScreen(XtScreen(w)),
+		     CoreColormap(w),
 		     &troughColor))
     {
-	troughColor.pixel =
-	    WhitePixelOfScreen(DefaultScreenOfDisplay(XtDisplay(w)));
+	troughColor.pixel = _XmWhitePixelOfObject(w);
     }
 
     val->addr = (XtPointer)&troughColor.pixel;
