@@ -1,6 +1,6 @@
 /*
  *
- * $Id: ScrolledW.c,v 1.68 1998/02/22 01:00:53 rwscott Exp $
+ * $Id: ScrolledW.c,v 1.78 1998/05/27 18:52:23 u27113 Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: ScrolledW.c,v 1.68 1998/02/22 01:00:53 rwscott Exp $";
+static const char rcsid[] = "$Id: ScrolledW.c,v 1.78 1998/05/27 18:52:23 u27113 Exp $";
 
 /*
  * Current status of changes in layout (Danny 18/4/1997) :
@@ -50,11 +50,16 @@ static char rcsid[] = "$Id: ScrolledW.c,v 1.68 1998/02/22 01:00:53 rwscott Exp $
 #define XmUNSPECIFIED (~0)
 #endif
 
-#define MFM_BUGS	1
+/* rws 25 Mar 1998
+   With this undefined the page list of mgv and mfm work correctly. The mfm
+   problem can be seen if you start it and then select a directory from the
+   middle list.  At this point the list on the right gets real tiny.
+ */
+#undef MFM_BUGS
 
 /* Forward Declarations */
 
-#if 0
+#if 1
 static void class_initialize();
 #endif
 
@@ -282,7 +287,7 @@ static XtActionsRec actions[] =
 };
 
 /* *INDENT-OFF* */
-#if 0
+#if 1
 static XmBaseClassExtRec _XmScrolledWCoreClassExtRec = {
     /* next_extension            */ NULL,
     /* record_type               */ NULLQUARK,                             
@@ -324,7 +329,7 @@ XmScrolledWindowClassRec xmScrolledWindowClassRec = {
 	/* superclass            */ (WidgetClass) &xmManagerClassRec,
         /* class_name            */ "XmScrolledWindow",
 	/* widget_size           */ sizeof(XmScrolledWindowRec),
-	/* class_initialize      */ NULL /*class_initialize*/,
+	/* class_initialize      */ class_initialize,
 	/* class_part_initialize */ class_part_initialize,
 	/* class_inited          */ False,
 	/* initialize            */ initialize,
@@ -352,7 +357,7 @@ XmScrolledWindowClassRec xmScrolledWindowClassRec = {
 	/* tm_table              */ XtInheritTranslations,
 	/* query_geometry        */ query_geometry,
 	/* display_accelerator   */ NULL,
-	/* extension             */ (XtPointer)NULL /*&_XmScrolledWCoreClassExtRec*/
+	/* extension             */ (XtPointer)&_XmScrolledWCoreClassExtRec
     },
     /* Composite class part */
     {
@@ -380,7 +385,7 @@ XmScrolledWindowClassRec xmScrolledWindowClassRec = {
         /* syn_constraint_resources     */ NULL,
         /* num_syn_constraint_resources */ 0,
         /* parent_process               */ XmInheritParentProcess,
-        /* extension                    */ (XtPointer)NULL /*&_XmScrolledWMClassExtRec*/
+        /* extension                    */ (XtPointer)&_XmScrolledWMClassExtRec
     },
     /* XmScrolledWindow part */
     {
@@ -391,7 +396,7 @@ XmScrolledWindowClassRec xmScrolledWindowClassRec = {
 
 WidgetClass xmScrolledWindowWidgetClass = (WidgetClass)&xmScrolledWindowClassRec;
 
-#if 0
+#if 1
 static void
 class_initialize()
 {
@@ -1002,7 +1007,7 @@ geometry_manager(Widget w, XtWidgetGeometry *desired, XtWidgetGeometry *allowed)
 		       XdbWidgetGeometry2String(desired)));
 
     /* Whoa, nelly.  Not if we control what you do. */
-    if (w == (Widget)SW_HSB(sw) && w == (Widget)SW_VSB(sw))
+    if (w == (Widget)SW_HSB(sw) || w == (Widget)SW_VSB(sw))
     {
 	return XtGeometryNo;
     }
@@ -1055,7 +1060,7 @@ geometry_manager(Widget w, XtWidgetGeometry *desired, XtWidgetGeometry *allowed)
 	 * Taking this out gets the initial display of mfm correct and
 	 * does not seem to bother anything else
 	 */
-#ifdef MFM_BUGS
+#ifndef MFM_BUGS
 	_XmRepositionScrolledWindow(w, NULL, NULL);
 #endif
 
@@ -1069,10 +1074,21 @@ geometry_manager(Widget w, XtWidgetGeometry *desired, XtWidgetGeometry *allowed)
 
     if (_XmScrolledWGeomRequest(sw, &vals) == XtGeometryYes)
     {
+#if 1
+	/* rws 25 Mar 1998
+	 * This makes the mgv page list size correctly
+	 *
+	 * jonf 08 Apr 1998
+	 * I think I've fixed that problem, so I put this back in.
+	 */
 	_XmScrolledWLayout(sw, w, &wants, &vals);
+#else
+	_XmScrolledWLayout(sw, NULL, NULL, &vals);
+#endif
     }
     else
     {
+	DEBUGOUT(XdbDebug2(__FILE__, sw, w, "GeometryManager => No 1\n"));
 	return XtGeometryNo;
     }
 
@@ -1094,6 +1110,7 @@ geometry_manager(Widget w, XtWidgetGeometry *desired, XtWidgetGeometry *allowed)
 
     if (Wants(CWX) && Wants(CWY) && !(Wants(CWWidth) && Wants(CWHeight)))
     {
+	DEBUGOUT(XdbDebug2(__FILE__, sw, w, "GeometryManager => No 2\n"));
 	return XtGeometryNo;
     }
 
@@ -1102,6 +1119,9 @@ geometry_manager(Widget w, XtWidgetGeometry *desired, XtWidgetGeometry *allowed)
     {
 	_XmScrolledWConfigureChildren(sw, w, &wants, &vals);
 
+	DEBUGOUT(XdbDebug2(__FILE__, sw, w, "GeometryManager => Yes-1 %s\n",
+	    XdbWidgetGeometry2String(allowed)));
+
 	return XtGeometryYes;
     }
     else if (Wants((CWWidth|CWHeight)) == CWWidth &&
@@ -1109,6 +1129,9 @@ geometry_manager(Widget w, XtWidgetGeometry *desired, XtWidgetGeometry *allowed)
     {
 	_XmScrolledWConfigureChildren(sw, w, &wants, &vals);
 
+	DEBUGOUT(XdbDebug2(__FILE__, sw, w, "GeometryManager => Yes-2 %s\n",
+	    XdbWidgetGeometry2String(allowed)));
+
 	return XtGeometryYes;
     }
     else if (Wants((CWWidth|CWHeight)) == CWHeight &&
@@ -1116,9 +1139,18 @@ geometry_manager(Widget w, XtWidgetGeometry *desired, XtWidgetGeometry *allowed)
     {
 	_XmScrolledWConfigureChildren(sw, w, &wants, &vals);
 
+	DEBUGOUT(XdbDebug2(__FILE__, sw, w, "GeometryManager => Yes-3 %s\n",
+	    XdbWidgetGeometry2String(allowed)));
+
 	return XtGeometryYes;
     }
 
+    /* Strip off unwanted bits */
+    allowed->request_mode &= wants.request_mode;
+
+    DEBUGOUT(XdbDebug2(__FILE__, sw, w, "GeometryManager -> Almost %s\n",
+	XdbWidgetGeometry2String(allowed)));
+
     return XtGeometryAlmost;
 }
 
@@ -1355,8 +1387,8 @@ _XmRepositionScrolledWindow(Widget w, XtPointer client, XtPointer call)
 void
 _XmFixupScrollBars(Widget w, Dimension ww, Dimension wh)
 {
-    int max, min, percent, n;
-    Arg args[4];
+    int value, max, min, percent, n;
+    Arg args[5];
 
     if (!SW_ClipWindow(w))
     {
@@ -1421,6 +1453,8 @@ _XmFixupScrollBars(Widget w, Dimension ww, Dimension wh)
 	    min = 0;
 	    percent = (((SW_CWWidth(w) * 10000) / ww) * ww) / 10000;
 	}
+	percent = percent > max - min ? max - min : percent;
+	percent = percent < 1 ? 1 : percent;
 
 	n = 0;
 	XtSetArg(args[n], XmNminimum, min); n++;
@@ -1428,10 +1462,24 @@ _XmFixupScrollBars(Widget w, Dimension ww, Dimension wh)
 	XtSetArg(args[n], XmNsliderSize, percent); n++;
 	if (SW_CWWidth(w) > 0)
 	{
-	    XtSetArg(args[n], XmNpageIncrement, SW_CWWidth(w));
-	    n++;
+	    if (SW_CWWidth(w) > max)
+	    {
+		XtSetArg(args[n], XmNpageIncrement, max);
+		n++;
+	    }
+	    else
+	    {
+		XtSetArg(args[n], XmNpageIncrement, SW_CWWidth(w));
+		n++;
+	    }
 	}
 
+	XtVaGetValues((Widget)SW_HSB(w),
+		XmNvalue, &value,
+		NULL);
+	value = value > max - percent ? max - percent : value;
+	value = value < min ? min : value;
+	XtSetArg(args[n], XmNvalue, value); n++;
 	XtSetValues((Widget)SW_HSB(w), args, n);
 
 	SW_HSBMinimum(w) = min;
@@ -1456,9 +1504,8 @@ _XmFixupScrollBars(Widget w, Dimension ww, Dimension wh)
 	    min = 0;
 	    percent = (((SW_CWHeight(w) * 10000) / wh) * wh) / 10000;
 	}
-
-	if (percent < 1)
-	    percent = 1;
+	percent = percent > max - min ? max - min : percent;
+	percent = percent < 1 ? 1 : percent;
 
 	n = 0;
 	XtSetArg(args[n], XmNminimum, min); n++;
@@ -1478,6 +1525,12 @@ _XmFixupScrollBars(Widget w, Dimension ww, Dimension wh)
 	    }
 	}
 
+	XtVaGetValues((Widget)SW_VSB(w),
+		XmNvalue, &value,
+		NULL);
+	value = value > max - percent ? max - percent : value;
+	value = value < min ? min : value;
+	XtSetArg(args[n], XmNvalue, value); n++;
 	XtSetValues((Widget)SW_VSB(w), args, n);
 
 	SW_VSBMinimum(w) = min;
@@ -1485,8 +1538,8 @@ _XmFixupScrollBars(Widget w, Dimension ww, Dimension wh)
 	SW_VSBSliderSize(w) = percent;
 
 	DEBUGOUT(XdbDebug(__FILE__, w,
-			  "FixupScrollBars VSB min %d max %d percent %d\n",
-			  min, max, percent));
+			  "FixupScrollBars VSB min %d max %d percent %d value %d\n",
+			  min, max, percent, value));
     }
 }
 
@@ -1793,12 +1846,22 @@ _XmScrolledWPreferredSize(Widget w, Widget child, XtWidgetGeometry *childgeom,
 	/* If child is bigger than we are, grow (if we can) */
 	if (!SW_FromResize(w) && SW_VisualPolicy(w) == XmVARIABLE)
 	{
-	    if (vals->WorkW + 2 * (SW_MarginWidth(w) +
-				   MGR_ShadowThickness(w)) > vals->SwW)
+	    /* rws 25 Mar 1998
+	     * Where does this 3 come from??? Just like FileSB:ListPreferredWidth
+	     * we need a fudge factor in here to make the mgv page list come out
+	     * the correct size.
+	     * 
+	     * jonf 08 Apr 1998
+	     * The 3 is no longer needed, since the mgv page list problem is now
+	     * fixed.
+	     */
+	    if (/*3 +*/ vals->WorkW + 2 * (SW_MarginWidth(w) +
+		       MGR_ShadowThickness(w)) + 
+		       (vals->ShowVSB ? (SW_Spacing(w) + vals->VsbW) : 0) > vals->SwW)
 	    {
 		Dimension xx = vals->SwW;
 
-		vals->SwW = vals->WorkW +
+		vals->SwW =/* 3 +*/ vals->WorkW +
 			2 * (SW_MarginWidth(w) + MGR_ShadowThickness(w)) +
 			(vals->ShowVSB ? (SW_Spacing(w) + vals->VsbW) : 0);
 
@@ -1807,7 +1870,8 @@ _XmScrolledWPreferredSize(Widget w, Widget child, XtWidgetGeometry *childgeom,
 				   xx, vals->SwW));
 	    }
 	    if (vals->WorkH + 2 * (SW_MarginHeight(w) +
-				   MGR_ShadowThickness(w)) > vals->SwH)
+				   MGR_ShadowThickness(w)) +
+			(vals->ShowHSB ? (SW_Spacing(w) + vals->HsbH) : 0) > vals->SwH)
 	    {
 		Dimension xx = vals->SwH;
 
@@ -1880,6 +1944,7 @@ _XmScrolledWGeomRequest(Widget w, XmSWValues *vals)
     request.width = vals->SwW;
     request.height = vals->SwH;
 
+#if 1
 #ifdef MFM_BUGS
     if (SW_GivenWidth(w) != 0)
 #else
@@ -1896,6 +1961,7 @@ _XmScrolledWGeomRequest(Widget w, XmSWValues *vals)
     {
 	request.height = SW_GivenHeight(w);
     }
+#endif
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Request geo %s from parent %s %s\n",
 		      XdbWidgetGeometry2String(&request),
@@ -1962,6 +2028,10 @@ _XmScrolledWLayout(Widget w, Widget child, XtWidgetGeometry *childgeom,
      * took me weeks to figure out, because it turned out that our scrollbars
      * weren't traversable (which means they didn't highlight), so it actually
      * looked like the scrollbars were in the right place. Sigh.
+     *
+     * jonf  08 Apr 1998
+     *	Is there a test in test/Xm that will show the need for 
+     *	these adjustments?
      */
     bw = vals->HasHSB ? Prim_HighlightThickness(SW_HSB(w)) : 2;
     bh = vals->HasVSB ? Prim_HighlightThickness(SW_VSB(w)) : 2;
@@ -2641,15 +2711,31 @@ XmScrollVisible(Widget scrollw_widget, Widget widget,
 
     if (SW_HSB(scrollw_widget))
     {
-	XtVaGetValues((Widget)SW_HSB(scrollw_widget), XmNvalue, &val, NULL);
+    int min, max;
+
+	XtVaGetValues((Widget)SW_HSB(scrollw_widget), 
+		XmNvalue, &val, 
+		XmNmaximum, &max, 
+		XmNminimum, &min, 
+		NULL);
 	val -= dx;
+	val = val < min ? min : val;
+	val = val > max ? max : val;
 	XtVaSetValues((Widget)SW_HSB(scrollw_widget), XmNvalue, val, NULL);
     }
     if (SW_VSB(scrollw_widget))
     {
-	XtVaGetValues((Widget)SW_VSB(scrollw_widget), XmNvalue, &val, NULL);
+    int min, max;
+
+	XtVaGetValues((Widget)SW_VSB(scrollw_widget), 
+		XmNvalue, &val, 
+		XmNmaximum, &max, 
+		XmNminimum, &min, 
+		NULL);
 	val -= dy;
-	XtVaSetValues((Widget)SW_VSB(scrollw_widget), XmNvalue, dy, NULL);
+	val = val < min ? min : val;
+	val = val > max ? max : val;
+	XtVaSetValues((Widget)SW_VSB(scrollw_widget), XmNvalue, val, NULL);
     }
 }
 
@@ -2687,7 +2773,7 @@ XmScrolledWindowSetAreas(Widget widget,
 	_widgetBackground.pixel = XtBackground(widget);
  
 	XQueryColor(XtDisplay(widget),
-		    DefaultColormapOfScreen(XtScreen(widget)),
+		    CoreColormap(widget),
 		    &_widgetBackground);
  
 	troughColor.blue = _widgetBackground.blue * .80;
@@ -2695,11 +2781,10 @@ XmScrolledWindowSetAreas(Widget widget,
 	troughColor.red = _widgetBackground.red * .80;
  
 	if (!XAllocColor(XtDisplay(widget),
-			 DefaultColormapOfScreen(XtScreen(widget)),
+			 CoreColormap(widget),
 			 &troughColor))
 	{
-	    troughColor.pixel =
-		WhitePixelOfScreen(DefaultScreenOfDisplay(XtDisplay(widget)));
+	    troughColor.pixel = _XmWhitePixelOfObject(widget);
 	}
     }
  
