@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBox.c,v 1.24 1998/03/12 23:45:21 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBox.c,v 1.28 1998/04/15 00:20:03 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBox.c,v 1.24 1998/03/12 23:45:21 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBox.c,v 1.28 1998/04/15 00:20:03 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -53,6 +53,8 @@ static void class_part_initialize(WidgetClass w_class);
 static void initialize(Widget request, Widget new_w,
 		       ArgList args, Cardinal *num_args);
 
+static void destroy(Widget w);
+
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
 
@@ -306,7 +308,7 @@ XmSelectionBoxClassRec xmSelectionBoxClassRec = {
 	/* compress_exposure     */ XtExposeCompressMaximal,
 	/* compress_enterleave   */ True,
 	/* visible_interest      */ False,
-	/* destroy               */ NULL,
+	/* destroy               */ destroy,
 	/* resize                */ XtInheritResize,
 	/* expose                */ XtInheritExpose,
 	/* set_values            */ set_values,
@@ -402,7 +404,7 @@ _XmSelectionBoxMatch(XmSelectionBoxWidget w)
     {
 	return False;
     }
-    xms = XmStringCreateSimple(s);
+    xms = XmStringCreateLtoR(s, XmSTRING_DEFAULT_CHARSET);
     r = XmListItemExists(SB_List(w), xms);
 
     XtFree(s);
@@ -621,7 +623,6 @@ initialize(Widget request,
 
     SB_List(sb) = NULL;
     SB_ListLabel(sb) = NULL;
-    SB_ListLabelString(sb) = NULL;
 
     if (SB_DialogType(sb) != XmDIALOG_PROMPT)
     {
@@ -668,12 +669,25 @@ initialize(Widget request,
 
     if (SB_DialogType(sb) != XmDIALOG_COMMAND)
     {
+#if 0
+	/* rws 14 Apr 1998
+	   Since SelectionBox is a sub-class of BulletinBoard, set this
+	   directly, otherwise we get into nasty situations with sub-classes
+	   of SelectionBox, namely FileSelectionBox.
+	 */
 	XtVaSetValues(new_w, XmNdefaultButton, SB_OkButton(sb), NULL);
+#else
+	BB_DefaultButton(new_w) = SB_OkButton(sb);
+#endif
     }
     if (SB_TextString(sb) == (XmString)XmUNSPECIFIED)
     {
     	SB_TextString(sb) = XmStringCreateSimple("");
     }
+    else
+    {
+    	SB_TextString(sb) = XmStringCopy(SB_TextString(sb));
+    }
 
     if (SB_List(sb) && SB_ListItems(sb) && SB_ListItemCount(sb) > 0)
     {
@@ -684,6 +698,16 @@ initialize(Widget request,
     }
 }
 
+static void
+destroy(Widget w)
+{
+    /* rws 14 Apr 1998
+       Motif does not have this method.  Then again we all know Motif has
+       memory leaks.
+     */
+    XmStringFree(SB_TextString(w));
+}
+
 /*
  * handle element updates
  */
