@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/Text.c,v 1.50 1998/02/22 01:00:58 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/Text.c,v 1.57 1998/06/03 19:27:49 danny Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: Text.c,v 1.50 1998/02/22 01:00:58 rwscott Exp $";
+static char rcsid[] = "$Id: Text.c,v 1.57 1998/06/03 19:27:49 danny Exp $";
 
 #undef VERBOSE
 
@@ -32,6 +32,8 @@ static char rcsid[] = "$Id: Text.c,v 1.50 1998/02/22 01:00:58 rwscott Exp $";
 #include <Xm/TextOutP.h>
 #include <Xm/TextF.h>
 
+#include <Xm/CutPaste.h>
+
 #include <Xm/TransltnsP.h>
 #include <Xm/ScrolledW.h>
 #include <XmI/XmI.h>
@@ -103,6 +105,22 @@ void _XmChangeVSB(XmTextWidget w, XmTextPosition pos);
 
 void _XmRedisplayHBar(XmTextWidget w, int offset);
 
+#if XmVERSION > 1
+#include <Xm/TraitP.h>
+#include <Xm/AccTextT.h>
+static XtPointer _XmText_TraitGetValue(Widget w, int format);
+static void _XmText_TraitSetValue(Widget w, XtPointer value, int format);
+static int _XmText_TraitPreferredFormat(Widget w);
+
+/* Trait record */
+XmAccessTextualTraitRec _XmTextTraitRec =
+	{       /* version */		0,
+		/* getValue */		_XmText_TraitGetValue,
+		/* setValue */		_XmText_TraitSetValue,
+		/* preferredFormat */	_XmText_TraitPreferredFormat,
+	};
+#endif
+
 #define Offset(_name) XtOffsetOf(XmTextRec, text._name)
 
 /* Resources for the Text  class */
@@ -260,113 +278,6 @@ static XmSyntheticResource syn_resources[] =
 };
 #undef Offset
 
-char _XmText_EventBindings1[] =
-"<Key>F2:    backward-word()\n" \
-"<Key>F3:    backward-paragraph()\n" \
-"<Key>F4:    beginning-of-line()\n" \
-"<Key>F5:    forward-word()\n" \
-"<Key>F6:    forward-paragraph()\n" \
-"<Key>F7:	end-of-line()\n" \
-"Ctrl<Key>5:    forward-paragraph()\n" \
-"Ctrl<Key>A:    beginning-of-line()\n" \
-"Ctrl<Key>B:    backward-character()\n" \
-"Ctrl<Key>C:    paste-clipboard()\n" \
-"Ctrl<Key>D:    delete-next-character()\n" \
-"Ctrl<Key>E:	end-of-line()\n" \
-"Ctrl<Key>F:    forward-character()\n" \
-"Ctrl<Key>H:    delete-previous-character()\n" \
-"Ctrl<Key>J:	newline-and-indent()\n" \
-"Ctrl<Key>K:    kill-to-end-of-line()\n" \
-"Ctrl<Key>L:    redraw-display()\n" \
-"Ctrl<Key>M:	newline()\n" \
-"Ctrl<Key>N:	next-line()\n" \
-"Ctrl<Key>O:	newline-and-backup()\n" \
-"Ctrl<Key>P:	previous-line()\n" \
-"Ctrl<Key>V:	next-page()\n" \
-"Ctrl<Key>W:    kill-selection()\n" \
-"Meta<Key>B:    backward-word()\n" \
-"Meta<Key>F:    forward-word()\n" \
-"Meta<Key>V:    previous-page()\n" \
-":Meta<Key>d:	delete-next-word()\n" \
-":Meta<Key>h:	delete-previous-word()\n" \
-":m <Key>osfPrimaryPaste: cut-primary()\n\
-:a <Key>osfPrimaryPaste: cut-primary()\n\
-:<Key>osfPrimaryPaste:	copy-primary()\n\
-:m <Key>osfCut:		cut-primary()\n\
-:a <Key>osfCut:		cut-primary()\n\
-:<Key>osfCut:		cut-clipboard()\n\
-:<Key>osfPaste:		paste-clipboard()\n\
-:m <Key>osfCopy:	copy-primary()\n\
-:a <Key>osfCopy:	copy-primary()\n\
-:<Key>osfCopy:		copy-clipboard()\n\
-s <Key>osfBeginLine:	beginning-of-line(extend)\n\
-:<Key>osfBeginLine:	beginning-of-line()\n\
-s <Key>osfEndLine:	end-of-line(extend)\n\
-:<Key>osfEndLine:	end-of-line()\n\
-s <Key>osfPageLeft:	page-left(extend)\n\
-:<Key>osfPageLeft:	page-left()\n\
-s c<Key>osfPageUp:	previous-page(extend)\n\
-:c <Key>osfPageUp:	previous-page()\n\
-s <Key>osfPageRight:	page-right(extend)\n\
-:<Key>osfPageRight:	page-right()\n\
-s c <Key>osfPageDown:	next-page(extend)\n\
-:c <Key>osfPageDown:	next-page()\n\
-:<Key>osfClear:		clear-selection()\n\
-:<Key>osfBackSpace:	delete-previous-character()\n\
-s m <Key>osfDelete:	cut-primary()\n\
-s a <Key>osfDelete:	cut-primary()\n\
-s <Key>osfDelete:	cut-clipboard()\n\
-:c <Key>osfDelete:	delete-to-end-of-line()\n\
-:<Key>osfDelete:	delete-next-character()";
-
-char _XmText_EventBindings2[] =
-":c m <Key>osfInsert:	copy-primary()\n\
-:c a <Key>osfInsert:	copy-primary()\n\
-s <Key>osfInsert:	paste-clipboard()\n\
-:c <Key>osfInsert:	copy-clipboard()\n\
-:s <Key>osfSelect:	key-select()\n\
-:<Key>osfSelect:	set-anchor()\n\
-:<Key>osfActivate:	activate()\n\
-:<Key>osfAddMode:	toggle-add-mode()\n\
-:<Key>osfHelp:		Help()\n\
-:<Key>osfCancel:	process-cancel()\n\
-s c <Key>osfLeft:	backward-word(extend)\n\
-:c <Key>osfLeft:	backward-word()\n\
-s <Key>osfLeft:		key-select(left)\n\
-:<Key>osfLeft:		backward-character()\n\
-s c <Key>osfRight:	forward-word(extend)\n\
-:c <Key>osfRight:	forward-word()\n\
-s <Key>osfRight:	key-select(right)\n\
-:<Key>osfRight:		forward-character()\n\
-:<Key>osfUp:		previous-line()\n\
-:<Key>osfDown:		next-line()\n\
-c ~m ~a <Key>slash:	select-all()\n\
-c ~m ~a <Key>backslash:	deselect-all()\n\
-s ~m ~a <Key>Tab:	prev-tab-group()\n\
-~m ~a <Key>Tab:		next-tab-group()\n\
-~s ~m ~a <Key>Return:	newline()\n\
-c ~s ~m ~a <Key>space:	set-anchor()\n\
-c s ~m ~a <Key>space:	key-select()\n\
-s ~c ~m ~a <Key>space:	self-insert()\n\
-<Key>:			self-insert()";
-
-char _XmText_EventBindings3[] =
-"<Unmap>:		unmap()\n\
-<Enter>:		enter-window()\n\
-<Leave>:		leave-window()\n\
-<FocusIn>:		focus-in()\n\
-<FocusOut>:		focus-out()\n\
-~c s ~m ~a <Btn1Down>:	extend-start()\n\
-c ~s ~m ~a <Btn1Down>:	move-destination()\n\
-~c ~s ~m ~a <Btn1Down>:	grab-focus()\n\
-~c ~m ~a <Btn1Motion>:	extend-adjust()\n\
-~c ~m ~a <Btn1Up>:	extend-end()\n\
-<Btn2Down>:		process-bdrag()\n\
-m ~a <Btn2Motion>:	secondary-adjust()\n\
-~m a <Btn2Motion>:	secondary-adjust()\n\
-~s <Btn2Up>:		copy-to()\n\
-~c <Btn2Up>:		move-to()";
-
 /* *INDENT-OFF* */
 static XmBaseClassExtRec textBaseClassExtRec = {
     /* next_extension            */ NULL,
@@ -606,6 +517,13 @@ ClassInitialize()
     (void)strcpy(cp, _XmTextIn_XmTextEventBindings3);
 
     xmTextClassRec.core_class.tm_table = buf;
+
+#if XmVERSION > 1
+    if (!XmeTraitSet(xmTextWidgetClass, XmQTaccessTextual, &_XmTextTraitRec)) {
+	DEBUGOUT(XdbDebug(__FILE__, NULL,
+		"ClassInitialize: XmeTraitSet failed\n"));
+    }
+#endif
 }
 
 static void
@@ -1217,6 +1135,17 @@ _XmTextUpdateLineTable(Widget w,
     Text_NeedsRefigureLines(w) = True;
     Text_NeedsRedisplay(w) = True;
 
+    /* This is a hack. Xinvest becomes happier */
+    if (start == 0 && end == 0) {
+	/* We must initialize the whole line table; let's make sure it ends
+	 * with zeroes.
+	 */
+	for (index=0; index < Text_TableSize(tw); index++) {
+		Text_LineTable(tw)[index].start_pos = 0;
+		Text_LineTable(tw)[index].virt_line = False;
+	}
+    }
+
     /* Initialize ourselves */
     if (start == 0)
     {
@@ -1685,7 +1614,7 @@ XmTextClearSelection(Widget w, Time time)
 	XmTextFieldClearSelection(w, time);
 	return;
     }       
-        
+
     _XmWarning(w, "XmTextClearSelection: widget has invalid class");
 
 }
@@ -1693,15 +1622,84 @@ XmTextClearSelection(Widget w, Time time)
 Boolean
 XmTextCopy(Widget w, Time time)
 {
-    _XmWarning(w, "XmTextCopy: not implemented");
-    return False;		/* FIX ME */
+    Display		*dpy;
+    Window		win;
+    int			r;
+    long		item_id, data_id;
+    XmString		lb = NULL;
+    char		*fmt = NULL;
+    Boolean		sel;
+    XmTextPosition	left, right;
+    XmTextBlockRec	block;
+
+    if (!XmIsText(w))
+	return False;
+    if (w == NULL || XtWindow(w) == NULL)
+	return False;
+
+    dpy = XtDisplay(w);
+    win = XtWindow(w);
+
+    sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
+    if (!sel)
+	return False;
+
+    (*Text_Source(w)->ReadSource) (Text_Source(w), left, right, &block);
+
+    lb = XmStringCreateLocalized("XmText");
+
+    while ((r = XmClipboardStartCopy(dpy, win, lb, time,
+		NULL, NULL, &item_id)) == XmClipboardLocked)
+	;
+
+    XmStringFree(lb);
+    if (r != XmClipboardSuccess) {
+	XtFree(block.ptr);
+	return False;
+    }
+
+    while ((r = XmClipboardCopy(dpy, win, item_id, "STRING",
+		block.ptr, right - left + 1,
+		0, NULL)) == XmClipboardLocked)
+	;
+
+    XtFree(block.ptr);
+
+    if (r != XmClipboardSuccess) {
+	(void) XmClipboardEndCopy(dpy, win, item_id);
+	return False;
+    }
+
+    while ((r = XmClipboardEndCopy(dpy, win, item_id)) == XmClipboardLocked)
+	;
+
+    if (r == XmClipboardSuccess)
+	return True;
+    else
+	return False;
 }
 
 Boolean
 XmTextCut(Widget w, Time time)
 {
-    _XmWarning(w, "XmTextCut: not implemented");
-    return False;		/* FIX ME */
+    Boolean		sel, r = False;
+    XmTextPosition	left, right;
+
+    if (!XmIsText(w)) {
+        return False;   
+    }
+
+    if ((r = XmTextCopy(w, time)) == False)
+        return False;
+
+    sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
+    if (!sel)
+	return False;
+
+    /* Now also remove the string from the buffer. */
+    _XmTextDelete((XmTextWidget)w, NULL, left, right);
+
+    return r;
 }
 
 Boolean
@@ -2064,15 +2062,60 @@ XmTextPaste(Widget w)
 {
     if (XtIsSubclass(w, xmTextWidgetClass))
     {
-	_XmWarning(w, "XmTextPaste: not implemented");
-	return False;		/* FIX ME */
-    }
-    else if (XtIsSubclass(w, xmTextFieldWidgetClass))
-    {
+	Display		*dpy = XtDisplay(w);
+	Window		win = XtWindow(w);
+	int		r;
+	char		*buf;
+	unsigned long	private_id, len, nbytes;
+	XmTextPosition	pos;
+
+	if (win == NULL || !Text_Editable(w))
+	    return False;
+
+	while ((r = XmClipboardStartRetrieve(XtDisplay(w), XtWindow(w),
+                XtLastTimestampProcessed(XtDisplay(w)))) == XmClipboardLocked)
+	    ;
+	if (r != XmClipboardSuccess)
+	    return False;
+
+	/* Figure out how much we need to paste */
+	while ((r = XmClipboardInquireLength(dpy, win, "STRING", &len))
+		== XmClipboardLocked)
+	    ;
+	if (r != XmClipboardSuccess)
+	    return False;
+
+	buf = XtMalloc(len + 1);
+
+	while ((r = XmClipboardRetrieve(dpy, win, "STRING",
+		buf, len, &nbytes, &private_id)) == XmClipboardLocked)
+	    ;
+	buf[len] = '\0';
+
+	if (r != XmClipboardSuccess) {
+	    XtFree(buf);
+	    return False;
+	}
+	DEBUGOUT(XdbDebug(__FILE__, w,
+	    "XmTextPaste: should be getting %ld bytes\n", nbytes));
+
+	while ((r = XmClipboardEndRetrieve(dpy, win)) == XmClipboardLocked)
+	    ;
+	if (r != XmClipboardSuccess) {
+	    XtFree(buf);
+	    return False;
+	}
+
+	/* Now insert the text ... */
+	pos = Text_CursorPos(w);
+
+	_XmTextInsert(w, pos, buf, NULL);
+
+	XtFree(buf);
+	return True;
+    } else if (XtIsSubclass(w, xmTextFieldWidgetClass)) {
 	return XmTextFieldPaste(w);
-    }
-    else
-    {
+    } else {
 	_XmWarning(w, "XmTextPaste: widget has invalid class");
     }
 
@@ -2163,10 +2206,14 @@ XmTextReplace(Widget w, XmTextPosition from_pos, XmTextPosition to_pos, char *va
 
     RefigureLines(tw);
 
+#if 1
+    XmTextShowPosition(w, from_pos + block.length);
+#else
     if (XtIsRealized(w))
     {
 	Redisplay(tw);
     }
+#endif
 }
 
 void
@@ -2622,3 +2669,66 @@ TextGetValues(Widget w, ArgList args, Cardinal *num_args)
     DEBUGOUT(XdbDebug(__FILE__, w, "TextGetValues :\n"));
     DEBUGOUT(XdbPrintArgList(__FILE__, w, args, *num_args, True));
 }
+
+#if XmVERSION > 1
+/*
+ * Not sure whether I need to do all this copying.
+ * FIX ME
+ */ 
+static XtPointer
+_XmText_TraitGetValue(Widget w, int format)
+{
+	DEBUGOUT(XdbDebug(__FILE__, w, "_XmText_TraitGetValue(%s)\n",
+		(format == XmFORMAT_XmSTRING) ? "XmFORMAT_XmSTRING" :
+		(format == XmFORMAT_MBYTE) ? "XmFORMAT_MBYTE" :
+		(format == XmFORMAT_WCS) ? "XmFORMAT_WCS" : "??"));
+
+	switch (format) {
+	case XmFORMAT_XmSTRING:
+		return XmStringCreateSimple(Text_Value(w));
+	case XmFORMAT_MBYTE:
+		return XtNewString(XmTextGetString(w)); /* FIX ME */
+	case XmFORMAT_WCS:
+		return XtNewString(XmTextGetString(w)); /* FIX ME */
+	default:
+		return NULL;
+	}  
+}
+
+static void
+_XmText_TraitSetValue(Widget w, XtPointer value, int format)
+{
+	char    *s;
+
+	DEBUGOUT(XdbDebug(__FILE__, w, "_XmText_TraitSetValue(%s)\n",
+		(format == XmFORMAT_XmSTRING) ? "XmFORMAT_XmSTRING" :
+		(format == XmFORMAT_MBYTE) ? "XmFORMAT_MBYTE" :
+		(format == XmFORMAT_WCS) ? "XmFORMAT_WCS" : "??"));
+
+	switch (format) {
+	case XmFORMAT_XmSTRING:
+	    if (XmStringGetLtoR((XmString)value, XmFONTLIST_DEFAULT_TAG, &s)) {
+		XmTextSetString(w, s);
+	    }
+	    return;
+	case XmFORMAT_MBYTE:
+	    /* FIX ME */
+	    XmTextSetString(w, value);
+	    return;
+	case XmFORMAT_WCS:
+	    /* FIX ME */
+	    XmTextSetString(w, value);
+	    return;
+	default:
+	    return;
+	}
+}
+
+static int
+_XmText_TraitPreferredFormat(Widget w)
+{
+	DEBUGOUT(XdbDebug(__FILE__, w, "_XmText_TraitPreferredFormat\n"));
+
+	return XmFORMAT_WCS;
+}
+#endif
