@@ -1,7 +1,7 @@
 /*                   -*-Mode: c; c-file-style: "GNU"-*-              */
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.26 1997/11/17 23:40:23 miers Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.29 1998/05/28 22:14:24 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.26 1997/11/17 23:40:23 miers Exp $";
+static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.29 1998/05/28 22:14:24 u27113 Exp $";
 
 #include <LTconfig.h>
 #include <Xm/XmP.h>
@@ -545,6 +545,12 @@ DoInsert(XmTextWidget w, XEvent *ev, char *buf, int len)
 	return;
     }
 
+    if (!Text_Editable(w))
+    {
+	VerifyBell(w);
+	return;
+    }
+
     (*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w), off);
 
     blockrec.ptr = XtMalloc(len + 1);
@@ -835,6 +841,7 @@ static void
 CopyClipboard(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "CopyClipboard\n"));
+    XmTextCopy(w, ev->xkey.time);
 }
 
 static void
@@ -885,6 +892,9 @@ ProcessCopy(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "ProcessCopy\n"));
 
+    if (! Text_Editable(w))
+	return;
+
     XtGetSelectionValue(w, XA_PRIMARY, XA_STRING, _XmTextGetSelection,
 			(XtPointer)ev, ev->xbutton.time);
 }
@@ -899,6 +909,7 @@ static void
 CutClipboard(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "CutClipboard\n"));
+    XmTextCut(w, ev->xbutton.time);
 }
 
 static void
@@ -1701,6 +1712,7 @@ static void
 PasteClipboard(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "PasteClipboard\n"));
+    XmTextPaste(w);
 }
 
 static void
@@ -1892,12 +1904,6 @@ SelfInsert(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
     DEBUGOUT(XdbDebug(__FILE__, w, "SelfInsert"));
 
-    if (!Text_Editable(w))
-    {
-	VerifyBell((XmTextWidget)w);
-	return;
-    }
-
     len = XmImMbLookupString((Widget)w, (XKeyPressedEvent *)ev, buf, INSERTCHARBUFSIZ, &keysym, &status);
 
     if (XdbInDebug(__FILE__, w))
