@@ -1,7 +1,7 @@
 /*                   -*-Mode: c; c-file-style: "GNU"-*-              */
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.41 1997/12/05 02:35:57 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.42 1998/05/06 21:32:11 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.41 1997/12/05 02:35:57 rwscott Exp $";
+static char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.42 1998/05/06 21:32:11 u27113 Exp $";
 
 #include <LTconfig.h>
 #include <Xm/XmP.h>
@@ -642,6 +642,15 @@ _XmTextNextX(XmTextWidget w, int x, char *ptr, int len)
 	    if (c < fs->min_char_or_byte2 || c > fs->max_char_or_byte2)
 	    {
 		c = fs->default_char;
+		/* FIX ME
+		 * Hack by vogt@kvi.nl */
+		if (c < 32) {
+			DEBUGOUT(XdbDebug(__FILE__, (Widget)w, 
+				"_XmTextNextX: fs->default_char %d -> 32\n",
+				c));
+			c = 32;
+		}
+		/* end hack */
 	    }
 
 	    if (fs->per_char)
