@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextStrSo.c,v 1.15 1998/02/22 01:01:02 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextStrSo.c,v 1.17 1998/05/02 21:01:08 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: TextStrSo.c,v 1.15 1998/02/22 01:01:02 rwscott Exp $";
+static const char rcsid[] = "$Id: TextStrSo.c,v 1.17 1998/05/02 21:01:08 rwscott Exp $";
 
 /*
  * Empirical evidence (fooling around with the structures found in Motif
@@ -602,10 +602,20 @@ static Boolean
 GetSelection(XmTextSource source, XmTextPosition *left,
 	     XmTextPosition *right)
 {
+    *left = source->data->left;
+    *right = source->data->right;
     if (source->data->hasselection)
     {
+	/* rws 2 May 1998
+	   In ml the Insert Text button in the Reply window calls
+	   XmTextGetSelectionPosition and ignores the return value.  Instead
+	   if compares the value of left and right to determine if anything
+	   is selected.  Therefore always setting left and right makes it 
+	   happy.  If we do not have the selection left == right anyway, so
+	   everyone should be happy.
 	*left = source->data->left;
 	*right = source->data->right;
+	*/
 	return True;
     }
 
@@ -854,6 +864,15 @@ _XmStringSourceSetValue(XmTextWidget w, char *value)
     {
 	Insert(d, 0, value, len);
     }
+    else
+    {
+	/* This may not be necessary.
+	   d->ptr may still contain something that confuses us later.
+	   Get rid of it.
+	 */
+	if (d->length && d->ptr)
+	    d->ptr[0] = '\0';
+    }
 
     /* Need to call XmNvalueChangedCallback here; not possible a level lower. */
     if (Text_ValueChangedCallback(w))
