@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ToggleB.c,v 1.28 1998/03/01 19:22:13 rwscott Exp $
+ * $Id: ToggleB.c,v 1.29 1998/03/16 21:52:18 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: ToggleB.c,v 1.28 1998/03/01 19:22:13 rwscott Exp $";
+static const char rcsid[] = "$Id: ToggleB.c,v 1.29 1998/03/16 21:52:18 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -39,6 +39,12 @@ static char rcsid[] = "$Id: ToggleB.c,v 1.28 1998/03/01 19:22:13 rwscott Exp $";
 
 extern void _XmRadioCallback(Widget, XtPointer, XtPointer);
 
+#ifndef __STDC__
+#include <varargs.h>
+#else
+#include <stdarg.h>
+#endif
+
 #include <XmI/DebugUtil.h>
 
 #include <Xm/ToggleBGP.h>
@@ -959,7 +965,6 @@ static void
 ButtonDown(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     int validButton;
-    Widget child;
     XtExposeProc exp = XtClass(w)->core_class.expose;
 
     /* queue events until the next button event. */
@@ -1101,29 +1106,21 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
 #else
-MenuProcEntry(function, widget, va_alist)
-     int function;
-     Widget widget;
+MenuProcEntry(proc, w, va_alist)
+     int proc;
+     Widget w;
      va_dcl
 {
     va_list arglist;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
-    va_start(arg_list);
+    va_start(arglist);
 #endif
 
     switch (proc)
