@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ToggleBG.c,v 1.26 1998/03/01 19:22:14 rwscott Exp $
+ * $Id: ToggleBG.c,v 1.28 1998/03/18 20:33:56 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: ToggleBG.c,v 1.26 1998/03/01 19:22:14 rwscott Exp $";
+static const char rcsid[] = "$Id: ToggleBG.c,v 1.28 1998/03/18 20:33:56 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -39,6 +39,12 @@ static char rcsid[] = "$Id: ToggleBG.c,v 1.26 1998/03/01 19:22:14 rwscott Exp $"
 #include <X11/ShellP.h>
 #include <Xm/ToggleBP.h>
 
+#ifndef __STDC__
+#include <varargs.h>
+#else
+#include <stdarg.h>
+#endif
+
 #include <XmI/DebugUtil.h>
 
 extern void _XmRadioCallback(Widget, XtPointer, XtPointer);
@@ -1502,29 +1508,21 @@ static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
     va_start(arg_list, w);
 #else
-MenuProcEntry(function, widget, va_alist)
-     int function;
-     Widget widget;
+MenuProcEntry(proc, w, va_alist)
+     int proc;
+     Widget w;
      va_dcl
 {
     va_list arglist;
-    Display *dpy = XtDisplay(w);
-    Widget top_menu;
-    Widget *shell;
-    Boolean *was_torn;
     XEvent *event;
     extern void _XmTearOffArm();
  
-    va_start(arg_list);
+    va_start(arglist);
 #endif
 
     switch (proc)
@@ -1611,7 +1609,7 @@ XmToggleButtonGadgetSetState(Widget w, Boolean state, Boolean notify)
 			       TBG_ValueChangedCallback(w),
 			       (XtPointer)&cbs);
 	}
-	if (XtIsRealized(w))
+	if (XtIsRealized(w) && XtIsManaged(w))
 	{
 	    (*exp) (w, NULL, (Region)NULL);
 	}
@@ -1634,7 +1632,7 @@ _XmToggleButtonGadgetSetState(Widget w, Boolean state)
 
 	TBG_VisualSet(w) = TBG_Set(w) = state ? True : False;
 
-	if (XtIsRealized(w))
+	if (XtIsRealized(w) && XtIsManaged(w))
 	{
 	    (*exp) (w, NULL, (Region)NULL);
 	}
