@@ -1,6 +1,6 @@
 /**
  *
- * $Id: VirtKeys.c,v 1.6 1997/11/21 18:01:19 miers Exp $
+ * $Id: VirtKeys.c,v 1.8 1998/04/16 21:46:41 u27113 Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static char rcsid[] = "$Id: VirtKeys.c,v 1.6 1997/11/21 18:01:19 miers Exp $";
+static char rcsid[] = "$Id: VirtKeys.c,v 1.8 1998/04/16 21:46:41 u27113 Exp $";
 
 #include <LTconfig.h>
 
@@ -269,7 +269,7 @@ _XmVirtualToActualKeysym(Display *Dsp,
 
     wd = (XmDisplayRec *)XmGetXmDisplay(Dsp);
     /*
-     * Because the table is soooooo small (only 27 entries), we can affort
+     * Because the table is soooooo small (only 27 entries), we can afford
      * it to do a linear search.
      */
     for (i = 0; i < XtNumber(VirtualKeysyms); i++)
@@ -278,6 +278,11 @@ _XmVirtualToActualKeysym(Display *Dsp,
 	{
 	    *RealKeysymReturn = (wd->display.bindings)[i].keysym;
 	    *ModifierReturn = (wd->display.bindings)[i].modifiers;
+
+	    DEBUGOUT(XdbDebug(__FILE__, NULL,
+		"_XmVirtualToActualKeysym %d -> 0x%X\n",
+		VirtualKeysym, *RealKeysymReturn));
+
 	    return;
 	}
     }
@@ -1271,9 +1276,25 @@ CheckForVirtualKey(Display *Dsp, KeyCode Keycode,
 	return;
     }
 
+#if 0
+/*
+ * This is a hack to make nedit support PageUp and PageDown.
+ * A more permanent solution seems to have been found. It's not
+ * nice though. Look in XmTranslateKey and _XmVirtualHandler.
+ */
+    if (Key == 0xFF56 && CurrentModifiers == 4) {	/* PageDown */
+	*KeysymReturn = 0x1004FF42;			/* osfXK_PageDown */
+	return;
+    }
+    if (Key == 0xFF55 && CurrentModifiers == 4) {	/* PageUp */
+	*KeysymReturn = 0x1004FF41;			/* osfXK_PageUp */
+	return;
+    }
+#endif
+
     /*
-     * Because the table is soooooo small (only 27 entries), we can affort
-     * it to do a linear search (well, we can affort it once again, like in
+     * Because the table is soooooo small (only 27 entries), we can afford
+     * it to do a linear search (well, we can afford it once again, like in
      * _XmVirtualToActualKeysym()). For every entry in the bindings table
      * of the appropiate XmDisplay widget we check for a keysym match. If
      * we get one, we must make sure, that the modifier flags are allright.
@@ -1296,6 +1317,10 @@ CheckForVirtualKey(Display *Dsp, KeyCode Keycode,
 	    *ModifiersReturn |= Bindings->modifiers;
 	}
     }
+
+    DEBUGOUT(XdbDebug(__FILE__, NULL,
+	"CheckForVirtualKey 0x%X, mod 0x%X -> 0x%X, mod 0x%X\n",
+	Key, CurrentModifiers, *KeysymReturn, BestMods));
 }
 
 /*
@@ -1310,10 +1335,34 @@ XmTranslateKey(Display *Dsp, KeyCode Keycode,
 	       Modifiers modifiers, Modifiers *modifiers_return,
 	       KeySym *Keysym_return)
 {
+#if 1
+    /*
+     * Tie the keysym and the current modifier (from the lastKeyEvent) into
+     * eachother to figure out whether this maps into a virtual keysym.
+     */
+    XmDisplay d = (XmDisplay)XmGetXmDisplay(Dsp);
+
+    DEBUGOUT(XdbDebug(__FILE__, NULL,
+	"XmTranslateKey (KeyCode %d, modifiers 0x%X), event %d\n",
+	Keycode, modifiers, d->display.lastKeyEvent->state));
+
+    XtTranslateKey(Dsp, Keycode, modifiers, modifiers_return, Keysym_return);
+
+    CheckForVirtualKey(Dsp, Keycode, (Modifiers)d->display.lastKeyEvent->state,
+		       modifiers_return, Keysym_return);
+#else
+    /*
+     * Same as above without the funky stuff.
+     * This doesn't seem to work in all cases.
+     * Nedit shows that PageUp/PageDown don't work. Instead they behave as
+     * PageRight and PageLeft. Which lead me to look at the modifiers, which
+     * eventually got me to find all this. Sigh.
+     */
     XtTranslateKey(Dsp, Keycode, modifiers, modifiers_return, Keysym_return);
 
     CheckForVirtualKey(Dsp, Keycode, modifiers,
 		       modifiers_return, Keysym_return);
+#endif
 }
 
 void
@@ -1326,17 +1375,37 @@ _XmVirtKeysStoreBindings(Widget shell, String binding)
  * with the appropiate display widget object. This one is in mainly for
  * compatibility reasons -- maybe someone needs the information collected
  * here lateron...
+ *
+ * Yes we do. This appears to be the only way to get hold of which
+ * modifiers are valid right now (i.e. when the user presses this key).
  */
 void
 _XmVirtKeysHandler(Widget w, XtPointer Data, XEvent *Event,
 		   Boolean *ContDispatch)
 {
-    XmDisplayRec *wd;
+    XmDisplay	d;
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmVirtKeysHandler\n"));
 
     if (!w->core.being_destroyed && (Event->xany.type == KeyPress))
     {
-	wd = (XmDisplayRec *)XmGetXmDisplay(XtDisplay(w));
+	d = (XmDisplay)XmGetXmDisplay(XtDisplay(w));
 
-	bcopy(Event, wd->display.lastKeyEvent, sizeof(XKeyEvent));
+	*(d->display.lastKeyEvent) = *((XKeyEvent *)Event);
+
+	/*
+	 * Looks like Xt has a cache to prevent it from calling XmTranslateKey
+	 * all the time.
+	 *
+	 * Disable it for now.
+	 * That might be the only way to get XmTranslateKey processing (i.e.
+	 * our virtual bindings) to work right: we need to tie the modifier(s)
+	 * currently valid with the keysym that Xt comes up with to figure out
+	 * if this is one of our virtual keysyms.
+	 * Looks like this Xt caching business prevents this from working.
+	 *
+	 * There must be a better way to do this - FIX ME. Sigh.
+	 */
+	XtSetKeyTranslator(XtDisplay(w), (XtKeyProc)XmTranslateKey);
     }
 }
