@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Visual.c,v 1.9 1998/01/06 23:51:49 miers Exp $
+ * $Id: Visual.c,v 1.10 1998/03/24 23:01:13 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,11 +22,12 @@
  *
  **/
 
-static char rcsid[] = "$Id: Visual.c,v 1.9 1998/01/06 23:51:49 miers Exp $";
+static char rcsid[] = "$Id: Visual.c,v 1.10 1998/03/24 23:01:13 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
 #include <stdio.h>
+#include <limits.h>
 
 #include <Xm/XmP.h>
 #include <Xm/PrimitiveP.h>
@@ -203,9 +204,7 @@ XmChangeColor(Widget widget,
     Pixel select;
     Colormap color_map;
 
-    XtVaGetValues(widget,
-		  XmNcolormap, &color_map,
-		  NULL);
+    color_map = ColormapOfObject(widget);
 
     XmGetColors(XtScreenOfObject(widget),
 		color_map,
@@ -509,6 +508,7 @@ _XmForegroundColorDefault(Widget w,
 {
     double h, s, v;
     XColor _widgetBackground;
+    Colormap colormap;
     static XColor foregroundColor;
     static int inited = False;
     static Pixel last_background;
@@ -522,18 +522,15 @@ _XmForegroundColorDefault(Widget w,
      * have the same foreground.
      */
     if (XmIsGadget(w))
-    {
 	_widgetBackground.pixel = XmParentBackground(w);
-    }
     else
-    {
 	_widgetBackground.pixel = XtBackground(w);
-    }
+    colormap = ColormapOfObject(w);
 
     /* check cache */
     if (inited && 
 	_widgetBackground.pixel == last_background &&
-	last_colormap == DefaultColormapOfScreen(XtScreenOfObject(w)) &&
+	last_colormap == colormap &&
 	last_display == XtDisplayOfObject(w))
     {
 	val->addr = (XtPointer)&foregroundColor.pixel;
@@ -543,12 +540,12 @@ _XmForegroundColorDefault(Widget w,
     {
 	inited = True;
 	last_background = _widgetBackground.pixel;
-	last_colormap = DefaultColormapOfScreen(XtScreenOfObject(w));
+	last_colormap = colormap;
 	last_display = XtDisplayOfObject(w);
     }
 
     XQueryColor(XtDisplayOfObject(w),
-		DefaultColormapOfScreen(XtScreenOfObject(w)),
+		colormap,
 		&_widgetBackground);
 
     __XmRGB2HSV(_widgetBackground.red,
@@ -557,18 +554,12 @@ _XmForegroundColorDefault(Widget w,
 		&h, &s, &v);
 
     if (v < 0.5)
-    {
-	foregroundColor.pixel =
-	    WhitePixelOfScreen(DefaultScreenOfDisplay(XtDisplayOfObject(w)));
-    }
+	foregroundColor.pixel = _XmWhitePixelOfObject(w);
     else
-    {
-	foregroundColor.pixel =
-	    BlackPixelOfScreen(DefaultScreenOfDisplay(XtDisplayOfObject(w)));
-    }
+	foregroundColor.pixel = _XmBlackPixelOfObject(w);
 
     XQueryColor(XtDisplayOfObject(w),
-		DefaultColormapOfScreen(XtScreenOfObject(w)),
+		colormap,
 		&foregroundColor);
 
     val->addr = (XtPointer)&foregroundColor.pixel;
@@ -581,6 +572,8 @@ _XmHighlightColorDefault(Widget w,
 {
     double h, s, v;
     XColor _widgetBackground;
+    Colormap colormap;
+    Screen *screen;
     static XColor highlightColor;
     static int inited = False;
     static Pixel last_background;
@@ -594,19 +587,15 @@ _XmHighlightColorDefault(Widget w,
      * have the same highlight color.
      */
     if (XmIsGadget(w))
-    {
 	_widgetBackground.pixel = XtBackground(XtParent(w));
-    }
     else
-    {
 	_widgetBackground.pixel = XtBackground(w);
-    }
-
+    colormap = ColormapOfObject(w);
 
     /* check cache */
     if (inited && 
 	_widgetBackground.pixel == last_background &&
-	last_colormap == DefaultColormapOfScreen(XtScreenOfObject(w)) &&
+	last_colormap == colormap &&
 	last_display == XtDisplayOfObject(w))
     {
 	val->addr = (XtPointer)&highlightColor.pixel;
@@ -616,12 +605,12 @@ _XmHighlightColorDefault(Widget w,
     {
 	inited = True;
 	last_background = _widgetBackground.pixel;
-	last_colormap = DefaultColormapOfScreen(XtScreenOfObject(w));
+	last_colormap = colormap;
 	last_display = XtDisplayOfObject(w);
     }
 
     XQueryColor(XtDisplayOfObject(w),
-		DefaultColormapOfScreen(XtScreenOfObject(w)),
+		colormap,
 		&_widgetBackground);
 
     __XmRGB2HSV(_widgetBackground.red,
@@ -629,19 +618,14 @@ _XmHighlightColorDefault(Widget w,
 		_widgetBackground.blue,
 		&h, &s, &v);
 
+    screen = XtScreenOfObject(w);
     if (v < 0.5)
-    {
-	highlightColor.pixel =
-	    WhitePixelOfScreen(DefaultScreenOfDisplay(XtDisplayOfObject(w)));
-    }
+	highlightColor.pixel = _XmWhitePixelOfObject(w);
     else
-    {
-	highlightColor.pixel =
-	    BlackPixelOfScreen(DefaultScreenOfDisplay(XtDisplayOfObject(w)));
-    }
-
+	highlightColor.pixel = _XmBlackPixelOfObject(w);
+       
     XQueryColor(XtDisplayOfObject(w),
-		DefaultColormapOfScreen(XtScreenOfObject(w)),
+		colormap,
 		&highlightColor);
 
     val->addr = (XtPointer)&highlightColor.pixel;
@@ -652,6 +636,7 @@ _XmBackgroundColorDefault(Widget w,
 			  int offset,
 			  XrmValue *val)
 {
+    Colormap colormap = CoreColormap(w);
     static XColor backgroundColor;
     static int inited = False;
     static Colormap last_colormap;
@@ -665,7 +650,7 @@ _XmBackgroundColorDefault(Widget w,
      */
     /* check cache */
     if (inited && 
-	last_colormap == DefaultColormapOfScreen(XtScreenOfObject(w)) &&
+	last_colormap == colormap &&
 	last_display == XtDisplayOfObject(w))
     {
 	val->addr = (XtPointer)&backgroundColor.pixel;
@@ -674,22 +659,21 @@ _XmBackgroundColorDefault(Widget w,
     else
     {
 	inited = True;
-	last_colormap = DefaultColormapOfScreen(XtScreenOfObject(w));
+	last_colormap = colormap;
 	last_display = XtDisplayOfObject(w);
     }
 
     if (!XParseColor(XtDisplayOfObject(w),
-		     DefaultColormapOfScreen(XtScreenOfObject(w)),
+		     colormap,
 		     "rgb:72/9f/ff",
 		     &backgroundColor))
     {
-	backgroundColor.pixel =
-	    WhitePixelOfScreen(DefaultScreenOfDisplay(XtDisplayOfObject(w)));
+	backgroundColor.pixel = _XmWhitePixelOfObject(w);
     }
     else
     {
 	XAllocColor(XtDisplayOfObject(w),
-		    DefaultColormapOfScreen(XtScreenOfObject(w)),
+		    colormap,
 		    &backgroundColor);
     }
 
@@ -703,6 +687,7 @@ _XmTopShadowColorDefault(Widget w,
 			 XrmValue *val)
 {
     XColor _widgetBackground;
+    Colormap colormap;
     static XColor topShadowColor;
     static int inited = False;
     static Pixel last_background;
@@ -716,18 +701,15 @@ _XmTopShadowColorDefault(Widget w,
      * have the same tsc.
      */
     if (XmIsGadget(w))
-    {
 	_widgetBackground.pixel = XtBackground(XtParent(w));
-    }
     else
-    {
 	_widgetBackground.pixel = XtBackground(w);
-    }
+    colormap = ColormapOfObject(w);
 
     /* check cache */
     if (inited && 
 	_widgetBackground.pixel == last_background &&
-	last_colormap == DefaultColormapOfScreen(XtScreenOfObject(w)) &&
+	last_colormap == colormap &&
 	last_display == XtDisplayOfObject(w))
     {
 	val->addr = (XtPointer)&topShadowColor.pixel;
@@ -737,12 +719,12 @@ _XmTopShadowColorDefault(Widget w,
     {
 	inited = True;
 	last_background = _widgetBackground.pixel;
-	last_colormap = DefaultColormapOfScreen(XtScreenOfObject(w));
+	last_colormap = colormap;
 	last_display = XtDisplayOfObject(w);
     }
 
     XQueryColor(XtDisplayOfObject(w),
-		DefaultColormapOfScreen(XtScreenOfObject(w)),
+		colormap,
 		&_widgetBackground);
 
     if (_widgetBackground.red > 65535 / 1.5)
@@ -786,11 +768,10 @@ _XmTopShadowColorDefault(Widget w,
 
 
     if (!XAllocColor(XtDisplayOfObject(w),
-		     DefaultColormapOfScreen(XtScreenOfObject(w)),
+		     colormap,
 		     &topShadowColor))
     {
-	topShadowColor.pixel =
-	    WhitePixelOfScreen(DefaultScreenOfDisplay(XtDisplayOfObject(w)));
+	topShadowColor.pixel = _XmWhitePixelOfObject(w);
     }
 
     val->addr = (XtPointer)&topShadowColor.pixel;
@@ -803,6 +784,7 @@ _XmBottomShadowColorDefault(Widget w,
 			    XrmValue *val)
 {
     XColor _widgetBackground;
+    Colormap colormap;
     static XColor bottomShadowColor;
     static int inited = False;
     static Pixel last_background;
@@ -816,18 +798,15 @@ _XmBottomShadowColorDefault(Widget w,
      * have the same bsc.
      */
     if (XmIsGadget(w))
-    {
 	_widgetBackground.pixel = XtBackground(XtParent(w));
-    }
     else
-    {
 	_widgetBackground.pixel = XtBackground(w);
-    }
+    colormap = ColormapOfObject(w);
 
     /* check cache */
     if (inited && 
 	_widgetBackground.pixel == last_background &&
-	last_colormap == DefaultColormapOfScreen(XtScreenOfObject(w)) &&
+	last_colormap == colormap &&
 	last_display == XtDisplayOfObject(w))
     {
 	val->addr = (XtPointer)&bottomShadowColor.pixel;
@@ -837,12 +816,12 @@ _XmBottomShadowColorDefault(Widget w,
     {
 	inited = True;
 	last_background = _widgetBackground.pixel;
-	last_colormap = DefaultColormapOfScreen(XtScreenOfObject(w));
+	last_colormap = colormap;
 	last_display = XtDisplayOfObject(w);
     }
 
     XQueryColor(XtDisplayOfObject(w),
-		DefaultColormapOfScreen(XtScreenOfObject(w)),
+		colormap,
 		&_widgetBackground);
 
 #if COLORED_SHADOWS
@@ -856,11 +835,10 @@ _XmBottomShadowColorDefault(Widget w,
 #endif
 
     if (!XAllocColor(XtDisplayOfObject(w),
-		     DefaultColormapOfScreen(XtScreenOfObject(w)),
+		     colormap,
 		     &bottomShadowColor))
     {
-	bottomShadowColor.pixel =
-	    WhitePixelOfScreen(DefaultScreenOfDisplay(XtDisplayOfObject(w)));
+	bottomShadowColor.pixel = _XmWhitePixelOfObject(w);
     }
 
     val->addr = (XtPointer)&bottomShadowColor.pixel;
@@ -1311,6 +1289,7 @@ _XmSelectColorDefault(Widget w,
 		      XrmValue *val)
 {
     XColor _widgetBackground;
+    Colormap colormap;
     static XColor selectColor;
     static int inited = False;
     static Pixel last_background;
@@ -1324,22 +1303,17 @@ _XmSelectColorDefault(Widget w,
      * have the same select color.
      */
     if (XmIsGadget(w))
-    {
 	_widgetBackground.pixel = XmParentBackground(w);
-    }
     else if (XmIsExtObject(w))
-    {
 	_widgetBackground.pixel = XmParentBackground(w);
-    }
     else
-    {
 	_widgetBackground.pixel = XtBackground(w);
-    }
+    colormap = ColormapOfObject(w);
 
     /* check cache */
     if (inited && 
 	_widgetBackground.pixel == last_background &&
-	last_colormap == DefaultColormapOfScreen(XtScreenOfObject(w)) &&
+	last_colormap == colormap &&
 	last_display == XtDisplayOfObject(w))
     {
 	val->addr = (XtPointer)&selectColor.pixel;
@@ -1349,12 +1323,12 @@ _XmSelectColorDefault(Widget w,
     {
 	inited = True;
 	last_background = _widgetBackground.pixel;
-	last_colormap = DefaultColormapOfScreen(XtScreenOfObject(w));
+	last_colormap = colormap;
 	last_display = XtDisplayOfObject(w);
     }
 
     XQueryColor(XtDisplayOfObject(w),
-		DefaultColormapOfScreen(XtScreenOfObject(w)),
+		colormap,
 		&_widgetBackground);
 
     /* FIX THESE -- they're the same as the bottom shadow color */
@@ -1369,13 +1343,74 @@ _XmSelectColorDefault(Widget w,
 #endif
 
     if (!XAllocColor(XtDisplayOfObject(w),
-		     DefaultColormapOfScreen(XtScreenOfObject(w)),
+		     colormap,
 		     &selectColor))
     {
-	selectColor.pixel =
-	    WhitePixelOfScreen(DefaultScreenOfDisplay(XtDisplayOfObject(w)));
+	selectColor.pixel = _XmWhitePixelOfObject(w);
     }
 
 
     val->addr = (XtPointer)&selectColor.pixel;
 }
+
+/*
+ * These functions (_XmBlackPixel and _XmWhitePixel) appear in
+ * libXm:Obso2_0.o in the Motif 2 libraries.  The prototypes are in
+ * Xm/XmP.h but there were no implementations.
+ *
+ * The final XColor parameter makes no sense to me at all but the Motif
+ * headers on HP-UX 9.05 specify a raw XColor (as opposed to the more
+ * sensible XColor*) too so the signatures are correct.
+ *
+ * I don't know the proper behavior of these functions so I'm guessing.
+ *
+ *	--mu@echo-on.net, 1998.03.15
+ */
+Pixel
+_XmWhitePixel(Screen *screen, Colormap colormap, XColor whitecolor)
+{
+    whitecolor.red   =
+    whitecolor.green =
+    whitecolor.blue  = USHRT_MAX;
+
+    if (colormap == DefaultColormapOfScreen(screen) ||
+	!XAllocColor(DisplayOfScreen(screen), colormap, &whitecolor))
+    {
+	whitecolor.pixel = WhitePixelOfScreen(screen);
+    }
+    return whitecolor.pixel;
+}
+
+Pixel
+_XmBlackPixel(Screen *screen, Colormap colormap, XColor blackcolor)
+{
+    blackcolor.red   =
+    blackcolor.green =
+    blackcolor.blue  = 0;
+
+    if (colormap == DefaultColormapOfScreen(screen) ||
+	!XAllocColor(DisplayOfScreen(screen), colormap, &blackcolor))
+    {
+	blackcolor.pixel = BlackPixelOfScreen(screen);
+    }
+    return blackcolor.pixel;
+}
+
+/*
+ * More convenient forms of _XmWhitePixel and _XmBlackPixel.
+ * These can't be macros because the stupid last parameter is a raw structure
+ * rather than a pointer.
+ */
+Pixel
+_XmWhitePixelOfObject(Widget w)
+{
+    XColor unused;
+    return _XmWhitePixel(XtScreenOfObject(w), ColormapOfObject(w), unused);
+}
+
+Pixel
+_XmBlackPixelOfObject(Widget w)
+{
+    XColor unused;
+    return _XmBlackPixel(XtScreenOfObject(w), ColormapOfObject(w), unused);
+}
